Package: nethet
Type: Package
Title: A bioconductor package for high-dimensional exploration of
        biological network heterogeneity
Version: 1.45.0
Date: 2020-09-27
Author: Nicolas Staedler, Frank Dondelinger
Maintainer: Nicolas Staedler <staedler.n@gmail.com>,
 Frank Dondelinger <fdondelinger.work@gmail.com>
Description: Package nethet is an implementation of statistical solid
    methodology enabling the analysis of network heterogeneity from
    high-dimensional data. It combines several implementations of recent
    statistical innovations useful for estimation and comparison of networks in
    a heterogeneous, high-dimensional setting. In particular, we provide code
    for formal two-sample testing in Gaussian graphical models (differential
    network and GGM-GSA; Stadler and Mukherjee, 2013, 2014) and make a novel
    network-based clustering algorithm available (mixed graphical lasso,
    Stadler and Mukherjee, 2013).
Imports: glasso, mvtnorm, GeneNet, huge, CompQuadForm, ggm, mclust,
        parallel, GSA, limma, multtest, ICSNP, glmnet, network,
        ggplot2, grDevices, graphics, stats, utils
Suggests: knitr, xtable, BiocStyle, testthat
biocViews: Clustering, GraphAndNetwork
VignetteBuilder: knitr
License: GPL-2
Packaged: 2026-04-29 00:05:54 UTC; biocbuild
RoxygenNote: 6.1.1
git_url: https://git.bioconductor.org/packages/nethet
git_branch: devel
git_last_commit: b4bb182
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: yes

# Generated by roxygen2: do not edit by hand

S3method(plot,diffnet)
S3method(plot,diffregr)
S3method(plot,ggmgsa)
S3method(plot,nethetclustering)
S3method(print,nethetsummary)
S3method(summary,diffnet)
S3method(summary,diffregr)
S3method(summary,ggmgsa)
S3method(summary,nethetclustering)
export(aggpval)
export(bwprun_mixglasso)
export(diffnet_multisplit)
export(diffnet_singlesplit)
export(diffregr_multisplit)
export(diffregr_singlesplit)
export(dot_plot)
export(export_network)
export(generate_2networks)
export(generate_inv_cov)
export(ggmgsa_multisplit)
export(gsea.iriz)
export(het_cv_glasso)
export(invcov2parcor)
export(invcov2parcor_array)
export(logratio)
export(mixglasso)
export(plot_2networks)
export(scatter_plot)
export(screen_aic.glasso)
export(screen_bic.glasso)
export(screen_cv.glasso)
export(screen_cv1se.lasso)
export(screen_cvfix.lasso)
export(screen_cvmin.lasso)
export(screen_cvsqrt.lasso)
export(screen_cvtrunc.lasso)
export(sim_mix)
export(sim_mix_networks)
import(CompQuadForm)
import(GSA)
import(GeneNet)
import(ICSNP)
import(ggm)
import(ggplot2)
import(glasso)
import(glmnet)
import(grDevices)
import(graphics)
import(huge)
import(limma)
import(mclust)
import(multtest)
import(mvtnorm)
import(network)
import(parallel)
import(stats)
import(utils)
useDynLib(nethet)

Package for analysing heterogeneous network data

Consists of:

mixglasso - Mixture Graphical Lasso

ggm_gsa - Graphical Gaussian Model Gene Set Analysis

DiffNet and DiffRegr - Statistical Test for difference in network and regression models.

Plotting and exporting - Convenient display and analysis of results.

Easy installation from github: 

    devtools::install_github('FrankD/nethet')

Also available on Bioconductor:

    if (!requireNamespace("BiocManager", quietly=TRUE))
        install.packages("BiocManager")
        
    BiocManager::install("nethet")

#####################################################################################
#####################################################################################
##HMMGLasso; Greedy Backward Pruning
## +different chromosomes, with possible different prob.trans
## +type={gamma}
## +backward pruning; merge/delete 'similar' components
##Revised:  Date:10/10/2012
##Changes: * in -vers2.R the SigInv are wrongly truncated !
##         * agglo_hmmglasso: don't save fit.hmmgl[[i]] for all number of states i
##         * new merge & delete operation
##         * take prob.init=rep(1/K,K) for computing test-loglikelihood using loglik_chr
##         * new name: bwprun_hmmglasso
##         * pen.wi, pen.ci, pen.pc
##         * merge-step was wrong !
##         * if (any(statesize<=min.statesize)){ KEEP 'OLD' PARAMETERS }
##         * do 'full' optimization when deleting/merging
##         * like func_hmmgl_bwpruning-24/04/2012 but: extra function for backward-pruning with only deleting states/backward-pruning with approx merge/del (see func_hmmgl_bwpruning-31032012)
##         * -19062012.r: miniter option
##         * -25062012: f.init (function for initialisation); small change in loglik_chr (if Phi[i,k]=0 for all k then set Phi[i,k]=1 for all k)
##         * -09082012: nstart.kmeans option, term-option in glasso.invcov, glasso.invcor, glasso.parcor
##         * -09082012: re-initialize bwprun if 'state too small'
##         * -14082012: cleaning-up;
##                      change in glasso.parcor [ww <- diag(s);param <- as.vector(diag(s))];
##                      change in loglik_chr: Phi[which(rowSums(Phi==0)==n.comp),] <- 1, instead of Phi[which(rowSums(Phi==0)==2),] <- 1;
##                      reinit.in/reinit.out-option in bwprun
##         * -15082012: (bwprun) output which state(s) are merged/deleted; plot_bwprun function; mer/del-option
##         * -10102012: mstep: if \lambda=0 then use cov(x.compl) and solve(cov(x.compl)) (and do not run glasso)
##                      add: mixglasso functions
##         * -11102012: mstep: if \lambda=Inf then use diag(cov(x.compl)) and solve(diag(cov(x.compl)))
##         * -12102012: mstep: if we consider consider chromosomes independent we have to update prob.init=apply(u[!duplicated(chromosome),,drop=FALSE],2,mean)
##         * -21102012: estep: if p is large then emission probablities get very small (underflow !);
##                             EXPStep.mix, EXPStep.hmm take log(emission) as input and perform estep by "scaling"
##                      bwprun_hmmglasso: take always prob.init.init=rep(1/(k-1),k-1) (do not take prob.init from previous hmmgl-solution)
##         * -22102012: bwprun_hmmglasso: include option equal.prob.trans=FALSE
##                      func.uinit: new options
##         * -04112012: hmmgl_mixgl-22102012.r originates from func_hmmgl_mixgl_diagcov-22102012.r
##         * -05112012: if (equal.prob.trans==FALSE) then it can happen that prob.trans[k,]=='nearly 0' (for the kth row). this happens if the per chromosome statesize (statesize.chr) is zero. this gives problem because of log in forward-backward equation. therefore: if norm.v.chr_sum_t==Inf, (for one k), then perform estep only on other chromosomes.
##
##
##

#####################
##Required Packages##
#####################
library('glasso')
library(mvtnorm)
library(mclust)
library(parallel)


##' Sum of non-diag elements of a matrix
##'
##'
##' @title Sum of non-diag elements of a matrix
##' @param m no descr
##' @return Sum of non-diag elements
##' @author n.stadler
##' @keywords internal
sumoffdiag <- function(m){
  sum(m)-sum(diag(m))
}

##' Initialization of responsibilities
##'
##' 
##' @title Initialization of MixGLasso 
##' @param x Observed data
##' @param n.comp Number of mixture components
##' @param init Method used for initialization
##' init={'cl.init','r.means','random','kmeans','kmeans.hc','hc'}
##' @param my.cl Initial cluster assignments;
##' need to be provided if init='cl.init' (otherwise this param is ignored)
##' @param nstart.kmeans Number of random starts in kmeans; default=1
##' @param iter.max.kmeans Maximal number of iteration in kmeans; default=10
##' @param modelname.hc Model class used in hc; default='EII'
##' @return  a list consisting of
##' \item{u}{responsibilities}
##' @author n.stadler
##' @keywords internal
func.uinit <- function(x,n.comp,init='kmeans',my.cl=NULL,nstart.kmeans=1,iter.max.kmeans=10,modelname.hc="EII"){
  ##Initializing EM (provides uinit)
  ##
  ##x: data-matrix
  ##n.comp:
  ##init: method of initialization {cl.init,r.means,random,kmeans,kmeans.hc,hc}
  ##cl: initial clustering (only necessary if init='cl.init')
  ##nstart.kmeans: no random start in kmeans-initialization (see init='kmeans')
  ##iter.max.kmeans: see documentation of kmeans
  ##modelname.hc: model used in hc-initialization (see init='hc')

  nonvirtual <- apply(!is.na(x),1,all)
  x.complete <- x[nonvirtual,]
  n <- nrow(x.complete)

  if (init=='cl.init'){
    cl <- my.cl
    u <- matrix(0.1,n,n.comp)
    u[col(u)==cl] <- 0.9
    u <- u/rowSums(u)
  }
  if (init=='r.means'){
    cl <- rep(NA,n)
    sub.sample <- sample(1:n,size=n.comp,replace=FALSE)
    cl[sub.sample] <- 1:n.comp
    off.sub.sample <- t(x.complete[setdiff(1:n,sub.sample),])
    init.mean <- x.complete[sub.sample,]
    ss_score <- matrix(NA,nrow=n-n.comp,n.comp)
    for (k in 1:n.comp){
      ss_score[,k]<- colSums((off.sub.sample-init.mean[k,])^2)
    }
    off_cl<- apply(ss_score,1,which.min)
    cl[setdiff(1:n,sub.sample)] <- off_cl
    u <- matrix(0.1,n,n.comp)
    u[col(u)==cl] <- 0.9
    u <- u/rowSums(u)
  }
  if(init=='random'){
    cl <- sample(1:n.comp,size=n,replace=TRUE)
    u <- matrix(0.1,n,n.comp)
    u[col(u)==cl] <- 0.9
    u <- u/rowSums(u)
  }
  if (init=='kmeans'){
    fit.kmeans <- kmeans(x.complete,n.comp,nstart=nstart.kmeans,iter.max=iter.max.kmeans)
    cl <- fit.kmeans$cluster
    u <- matrix(0.1,n,n.comp)
    u[col(u)==cl] <- 0.9
    u <- u/rowSums(u)
  }
  if (init=='kmeans.hc'){
    fit.hc <- hc(modelName = 'EII', data = x.complete)
    cl.hc <- as.vector(hclass(fit.hc,G=n.comp))
    init.center <- by(x.complete,cl.hc,colMeans)
    init.center <- do.call(rbind,init.center)
    fit.kmeans <- kmeans(x.complete,centers=init.center,iter.max=iter.max.kmeans)
    cl <- fit.kmeans$cluster
    u <- matrix(0.1,n,n.comp)
    u[col(u)==cl] <- 0.9
    u <- u/rowSums(u)
  }
  if (init=='hc'){
    fit.hc <- hc(modelName = modelname.hc, data = x.complete)
    cl <- as.vector(hclass(fit.hc,G=n.comp))
    u <- matrix(0.1,n,n.comp)
    u[col(u)==cl] <- 0.9
    u <- u/rowSums(u)
  }
  u.all <- matrix(NA,nrow(x),n.comp)
  u.all[nonvirtual,] <- u
  return(list(u=u.all))
}

##' Compute trace of matrix
##'
##' 
##' @title Compute trace of matrix
##' @param m no descr
##' @return trace of matrix
##' @author n.stadler
##' @keywords internal
tr <- function(m){sum(diag(m))}

##' Compute symmetric kull-back leibler distance
##'
##' 
##' @title Compute symmetric kull-back leibler distance
##' @param mu1 no descr
##' @param mu2 no descr
##' @param sig1 no descr
##' @param sig2 no descr
##' @return symmetric kull-back leibler distance
##' @author n.stadler
##' @keywords internal
symmkldist <- function(mu1,mu2,sig1,sig2){
    symmkl <- 0.5*tr((sig1-sig2)%*%(solve(sig2)-solve(sig1)))+0.5*t(mu1-mu2)%*%(solve(sig1)+solve(sig2))%*%(mu1-mu2)
    return(symmkl)
}

##' Distance between comps based on symm. kl-distance
##'
##' 
##' @title Distance between comps based on symm. kl-distance
##' @param Mu no descr
##' @param Sig no descr
##' @return list consisting of
##' \item{comp.kldist}{}
##' \item{min.comp.kldist}{}
##' @author n.stadler
##' @keywords internal
w.kldist <- function(Mu,Sig){
    n.comp <- ncol(Mu)
    res<- matrix(NA,n.comp,n.comp)
    for (k in 1:n.comp){
        for (kk in 1:n.comp){
            res[k,kk] <- symmkldist(Mu[,k],Mu[,kk],Sig[,,k],Sig[,,kk])
        }
    }
    list(comp.kldist=res,min.comp.kldist=min(res[upper.tri(res)]))
}

##' Performs EStep
##'
##' 
##' @title Performs EStep 
##' @param logphi no descr
##' @param mix.prob no descr
##' @return list consiting of
##' \item{u}{responsibilities}
##' \item{LL}{loglikelihood}
##' @author n.stadler
##' @keywords internal
EXPStep.mix <- function(logphi,mix.prob){
    u <- logphi+matrix(log(mix.prob),nrow(logphi),ncol(logphi),byrow=TRUE)
    max.u <- apply(u,1,max)
    u.scale <- exp(u-max.u)
    u <- u.scale/rowSums(u.scale)
    loglik <- sum(log(rowSums(u.scale))+max.u)
    return(list(u=u,loglik=loglik))
}

##' Graphical Lasso based on partial correlation penalty
##'
##' 
##' @title Graphical Lasso based on partial correlation penalty
##' @param s no descr
##' @param rho no descr
##' @param penalize.diagonal no descr
##' @param maxiter no descr
##' @param term no descr
##' @param verbose set to TRUE to print out progress.
##' @return w; wi; iter
##' @author n.stadler
##' @keywords internal
glasso.parcor <- function(s,rho,penalize.diagonal,maxiter=1000,term=10^{-3},
													verbose=FALSE){
  ww <- diag(s)
  iter <- 0
  err <- Inf #convergence of parameters
  param <- as.vector(diag(s))
  
  if(verbose) cat('Start\n')
  
  while((err>term)&(iter<maxiter)){
    
  	if(verbose) cat(iter, ' ', mean(ww), ' ', sd(ww), '\n')
    
  	gl <- glasso(s,rho=rho*ww,penalize.diagonal=penalize.diagonal, 
    						 approx=FALSE)
    
    ww <- 1/(diag(gl$wi))
    param.old <- param
    param <- as.vector(gl$w)
    err <- max(abs(param-param.old)/(1+abs(param)))
    iter <- iter+1
  }
  

  list(w=gl$w,wi=gl$wi,iter=iter)
}

##' Graphical Lasso based on inverse covariance penalty
##'
##' 
##' @title Graphical Lasso based on inverse covariance penalty
##' @param s no descr
##' @param rho no descr
##' @param penalize.diagonal no descr
##' @param term no descr
##' @return w; wi; iter
##' @author n.stadler
##' @keywords internal
glasso.invcor <- function(s,rho,penalize.diagonal,term=10^{-3}){
  if(penalize.diagonal==FALSE){
    ww <- diag(s)
    gl <- glasso(s,rho=rho*ww,penalize.diagonal=penalize.diagonal)
    return(list(w=gl$w,wi=gl$wi))
  }
  if(penalize.diagonal==TRUE){
    ww <- diag(s)/(1-rho)
    gl <- glasso(s,rho=rho*ww,penalize.diagonal=penalize.diagonal)
    return(list(w=gl$w,wi=gl$wi))
  }
}

##' Graphical Lasso based on inverse correlation penalty
##'
##' 
##' @title Graphical Lasso based on inverse correlation penalty
##' @param s no descr
##' @param rho no descr
##' @param penalize.diagonal no descr
##' @param term no descr
##' @return w; wi; iter
##' @author n.stadler
##' @keywords internal
glasso.invcov <- function(s,rho,penalize.diagonal,term=10^{-3}){
  gl <- glasso(s,rho=rho,penalize.diagonal=penalize.diagonal)
  list(w=gl$w,wi=gl$wi)
}


##' MStep of MixGLasso
##'
##' 
##' @title MStep of MixGLasso
##' @param x no descr
##' @param chromosome no descr
##' @param u no descr
##' @param v no descr
##' @param lambda no descr
##' @param gamma no descr
##' @param pen no descr
##' @param penalize.diagonal no descr
##' @param equal.prob.trans no descr
##' @param term no descr
##' @param model no descr
##' @return list consisting of mix.prob, Mu, Sig, SigInv
##' @author n.stadler
##' @keywords internal
MStepGlasso <- function(x,chromosome=NULL,u,v=NULL,lambda,gamma,pen,penalize.diagonal,equal.prob.trans=NULL,term,model='hmm'){
    ##Mstep for HMMGLasso/MixGLasso (optimizes -completeloglik+lambda*pen)
    ##different chromosomes
    ##x: nxp-data
    ##chromosome: a sequence of the form 1,1,1,1,1,...,2,2,2,2,2,...,3,3,3,3,3,... ; set to NULL for model='mixture'
    ##equal.prob.trans: TRUE (=same prob.trans for (independent) chromosomes); FALSE (=different prob.trans for (independent) chromosomes) ; set to NULL for model='mixture'
    ##u[t,j]=P[S_t=j|obs]
    ##v[t,j,j']=P[S_t-1=j,S_t=j'|obs] ; set to NULL for model='mixture'
    ##model={'hmm','mixture'}

  if(model=='hmm'){
    n.comp <- ncol(u)
    nonvirtual <- apply(!is.na(x),1,all)
    x.compl <- x[nonvirtual,]
    n <- dim(x.compl)[1]
    p <- dim(x.compl)[2]

    Mu <- matrix(NA,ncol=n.comp,nrow=p)
    SigInv <- Sig <- array(NA,dim=c(p,p,n.comp))
    prob.init <- matrix(NA,n.comp,length(levels(chromosome)))
    prob.trans <- array(NA,dim=c(n.comp,n.comp,length(levels(chromosome))))
    #exp.cloglik <- rep(NA,n.comp)

    for (l in 1:n.comp){
      pi.comps <- mean(u[nonvirtual,l])
      obj <- cov.wt(x.compl, wt = u[nonvirtual,l], cor = FALSE, center =TRUE, method = c("ML"))
      Mu[,l] <- obj$center
      samplecov <- obj$cov
      if ((lambda!=0)&(lambda!=Inf)){
        fit.glasso <- eval(as.name(pen))(samplecov,rho=2*(pi.comps^{gamma})*lambda/(sum(u[nonvirtual,l])),
                                         penalize.diagonal=penalize.diagonal,term=term)
        SigInv[,,l] <- fit.glasso$wi
        Sig[,,l] <- fit.glasso$w
      }
      if(lambda==0){
        SigInv[,,l] <- solve(samplecov)
        Sig[,,l] <- samplecov
      }
      if(lambda==Inf){
        SigInv[,,l] <- diag(1/diag(samplecov))
        Sig[,,l] <- diag(diag(samplecov))
      }
      #exp.cloglik[l] <- -(sum(u[nonvirtual,l])/2)*log(det(fit.glasso$w))-0.5*sum(diag(samplecov%*%fit.glasso$wi))#-lambda*(pi.comps^{gamma})*sumoffdiag(abs(fit.glasso$wi))
    }
    if (equal.prob.trans==TRUE){
        prob.init[,1:length(levels(chromosome))] <- apply(u[!duplicated(chromosome),,drop=FALSE],2,mean)
        v_sum_t <- colSums(v,dims=1)
        norm.v_sum_t <- 1/rowSums(v_sum_t)
        prob.trans[,,1:length(levels(chromosome))] <- diag(norm.v_sum_t)%*%v_sum_t
    }
    if (equal.prob.trans==FALSE){
        for (chr in 1:length(levels(chromosome))){
            index.chr <- which(chromosome==chr)
            n.chr <- length(index.chr)
            u.chr <- u[chromosome==chr,]
            prob.init[,chr] <- u.chr[1,]
            v.chr <- v[chromosome==chr,,,drop=FALSE]
            v.chr_sum_t <- colSums(v.chr,dims=1)
            norm.v.chr_sum_t <- 1/rowSums(v.chr_sum_t)
            prob.trans[,,chr] <- diag(norm.v.chr_sum_t)%*%v.chr_sum_t
            prob.trans[norm.v.chr_sum_t==Inf,,chr] <- 0 #set prob.trans[k,k']=0 if number of obs. in comp k is zero (otherwise we get Inf)
        }
    }
    return(list(prob.init=prob.init,prob.trans=prob.trans,Mu=Mu,Sig=Sig,SigInv=SigInv))#,exp.cloglik=exp.cloglik)
  }
  if(model=='mixture'){
    n.comp <- ncol(u)
    n <- dim(x)[1]
    p <- dim(x)[2]
    Mu <- matrix(0,ncol=n.comp,nrow=p)
    SigInv <- Sig <- array(0,dim=c(p,p,n.comp))
    mix.prob <- rep(NA,n.comp)
    for (l in 1:n.comp){
      pi.comps <- mean(u[,l])
      mix.prob[l] <- pi.comps
      obj <- cov.wt(x, wt = u[,l], cor = FALSE, center =TRUE, method = c("ML"))
      Mu[,l] <- obj$center
      samplecov <- obj$cov
      if((lambda!=0)&(lambda!=Inf)){
        fit.glasso <- eval(as.name(pen))(samplecov,rho=2*(pi.comps^{gamma})*lambda/(sum(u[,l])),
                                         penalize.diagonal=penalize.diagonal)
        SigInv[,,l] <- fit.glasso$wi
        Sig[,,l] <- fit.glasso$w
      }
      if(lambda==0){
        SigInv[,,l] <- solve(samplecov)
        Sig[,,l] <- samplecov
      }
      if(lambda==Inf){
        SigInv[,,l] <- diag(1/diag(samplecov))
        Sig[,,l] <- diag(diag(samplecov))
      }
    }
    return(list(mix.prob=mix.prob,Mu=Mu,Sig=Sig,SigInv=SigInv))
  }
}

##' mixglasso_ncomp_fixed
##'
##' This function runs mixglasso
##' @title mixglasso_ncomp_fixed
##' @param x Input data matrix
##' @param n.comp Number of mixture components
##' @param lambda Regularization parameter. Default=sqrt(2*n*log(p))/2
##' @param pen Determines form of penalty: glasso.parcor (default), glasso.invcov, glasso.invcor
##' @param init Initialization. Method used for initialization
##' init={'cl.init','r.means','random','kmeans','kmeans.hc','hc'}. Default='kmeans'
##' @param my.cl Initial cluster assignments;
##' need to be provided if init='cl.init' (otherwise this param is ignored). Default=NULL
##' @param modelname.hc Model class used in hc. Default="VVV"
##' @param nstart.kmeans Number of random starts in kmeans; default=1
##' @param iter.max.kmeans Maximal number of iteration in kmeans; default=10
##' @param term Termination criterion of EM algorithm. Default=10^-3
##' @param min.compsize Stop EM if any(compsize)<min.compsize; Default=5
##' @param ... Other arguments. See mixglasso_init
##' @return see return mixglasso_init. list consisting of
##' \item{mix.prob}{}
##' \item{Mu}{}
##' \item{Sig}{}
##' \item{SigInv}{}
##' \item{iter}{}
##' \item{loglik}{}
##' \item{bic}{-loglik+log(n)*DF/2}
##' \item{mmdl}{-loglik+penmmdl/2}
##' \item{u}{responsibilities}
##' \item{comp}{component assignments}
##' \item{compsize}{size of components}
##' \item{pi.comps}{}
##' \item{warn}{warnings during optimization}
##' @author n.stadler
##' @keywords internal
mixglasso_ncomp_fixed <- function(x,n.comp,
                      lambda=sqrt(2*nrow(x)*log(ncol(x)))/2,
                      pen='glasso.parcor',init='kmeans.hc',my.cl=NULL,modelname.hc="VVV",nstart.kmeans=1,iter.max.kmeans=10,
                      term=10^{-3},min.compsize=5,...){

  ##MixGLasso (optimizes -loglik+lambda*pen using EM)
  
  ##x: nxp-data
  ##n.comp: = no of mixture components
  ##lambda:
  ##init: initialization
  ##modelname.hc:
  ##nstart.kmeans:
  ##pen: 'glasso.invcov','glasso.parcor','glasso.corinv'
  ##term: see termination of EM
  ##min.compsize: stop EM if any(compsize)<min.compsize
  
  nonvirtual <- apply(!is.na(x),1,all)
  x <- x[nonvirtual,]
  n <- nrow(x)
  p <- ncol(x)
  
  u <- mix.prob <- NULL
  if (n.comp>1){
    fit.init.u <- func.uinit(x,n.comp,init=init,my.cl=my.cl,nstart.kmeans=nstart.kmeans,iter.max.kmeans=iter.max.kmeans,modelname.hc=modelname.hc)
    u <- fit.init.u$u
    mix.prob <- rep(1/n.comp,n.comp)
  }
  fit.mixgl <-  mixglasso_init(x=x,n.comp=n.comp,lambda=lambda,
                               pen=pen,u.init=u,mix.prob.init=mix.prob,
                               min.compsize=min.compsize,term=term,...)
  
  class(fit.mixgl) <- 'nethetclustering'
  return(fit.mixgl)
}

##' mixglasso_init (initialization and lambda set by user)
##'
##' This function runs mixglasso; requires initialization (u.init,mix.prob.init)
##' @title mixglasso_init
##' @param x Input data matrix
##' @param n.comp Number of mixture components
##' @param lambda Regularization parameter
##' @param u.init Initial responsibilities
##' @param mix.prob.init Initial component probablities
##' @param gamma Determines form of penalty
##' @param pen Determines form of penalty: glasso.parcor (default), glasso.invcov, glasso.invcor
##' @param penalize.diagonal Should the diagonal of the inverse covariance matrix be penalized ? Default=FALSE (recommended)
##' @param term Termination criterion of EM algorithm. Default=10^-3
##' @param miniter Minimal number of EM iteration before 'stop EM if any(compsize)<min.compsize' applies. Default=5
##' @param maxiter Maximal number of EM iteration. Default=1000
##' @param min.compsize Stop EM if any(compsize)<min.compsize; Default=5
##' @param show.trace Should information during execution be printed ? Default=FALSE
##' @return list consisting of
##' \item{mix.prob}{Component probabilities}
##' \item{Mu}{Component specific mean vectors}
##' \item{Sig}{Component specific covariance matrices}
##' \item{SigInv}{Component specific inverse covariance matrices}
##' \item{iter}{Number of EM iterations}
##' \item{loglik}{Log-likelihood}
##' \item{bic}{-loglik+log(n)*DF/2}
##' \item{mmdl}{-loglik+penmmdl/2}
##' \item{u}{Component responsibilities}
##' \item{comp}{Component assignments}
##' \item{compsize}{Size of components}
##' \item{pi.comps}{Component probabilities}
##' \item{warn}{Warnings during EM algorithm}
##' @author n.stadler
mixglasso_init<- function(x,n.comp,lambda,
                          u.init,mix.prob.init,
                          gamma=0.5,pen='glasso.parcor',penalize.diagonal=FALSE,
                          term=10^{-3},miniter=5,maxiter=1000,min.compsize=5,
                          show.trace=FALSE){

  ##MixGLasso (optimizes -loglik+lambda*pen using EM); Requires u.init as initialization
  ##x: nxp-data
  ##n.comp
  ##lambda
  ##u.init, mix.prob.init: initialization
  ##gamma: see penalty
  ##pen: 'glasso','glasso.parcor','glasso.corinv'
  ##                                    more detailed: for gamma=0 use PENALIZE.DIAGONAL=FALSE (PENALIZE.DIAGONAL=TRUE does NOT work !);
  ##                                                   for gamma=1 use PENALIZE.DIAGONAL=TRUE (PENALIZE.DIAGONAL=FALSE does also work)
  ##term: see termination of EM
  ##miniter: minimal number of EM iteration before 'stop EM if any(compsize)<min.compsize' applies
  ##maxiter: maximal number of EM iteration
  ##min.compsize: stop EM if any(compsize)<min.compsize; default value: min.compsize=5; minimial compsize possible is 5

  n <- nrow(x)
  p <- ncol(x)
  Mu <- matrix(NA,p,n.comp)
  Sig <- SigInv <- array(NA,dim=c(p,p,n.comp))

  ##If n.comp=1 do Glasso
  if (n.comp==1){
    obj <- cov.wt(x, cor = FALSE, center =TRUE, method = c("ML"))
    Mu <- obj$center
    samplecov <- obj$cov
    if((lambda!=0)&(lambda!=Inf)){
      fit.glasso <- eval(as.name(pen))(samplecov,rho=2*lambda/n,
                                       penalize.diagonal=penalize.diagonal,term=term)
      Sig <- fit.glasso$w
      SigInv <- fit.glasso$wi
    }
    if(lambda==0){
      Sig <- samplecov
      SigInv <- solve(samplecov)
    }
    if(lambda==Inf){
      SigInv <- diag(1/diag(samplecov))
      Sig <- diag(diag(samplecov))
    }
    loglik <- sum(dmvnorm(x,Mu,Sig,log=TRUE))
    SigInv[abs(SigInv)<10^{-3}] <- 0
    n.zero <- sum(SigInv==0)
    DF <- p+(p*(p+1)/2)-n.zero/2#df.mean+df.inv.covariance
    list(Mu=matrix(Mu,p,1),Sig=array(Sig,dim=c(p,p,1)),SigInv=array(SigInv,dim=c(p,p,1)),
         loglik=loglik,bic=-loglik+log(n)*DF/2,mmdl=-loglik+log(n)*DF/2,warn='NONE')
  }#end if (n.comp==1){
  else{
    ##Initialisation of parameters
    if(any(colSums(u.init)<=min.compsize)){cat('         -mixglasso: n.init_k <= min.compsize','\n')}
    u <- u.init
    for (l in 1:n.comp){
      obj <- cov.wt(x, wt = u[,l], cor = FALSE, center =TRUE, method = c("ML"))
      Mu[,l] <- obj$center
      samplecov <- obj$cov
      pi.comps <- mean(u[,l])
      if((lambda!=0)&(lambda!=Inf)){
        fit.glasso <- eval(as.name(pen))(samplecov,rho=2*(pi.comps^{gamma})*lambda/(sum(u[,l])),
                                         penalize.diagonal=penalize.diagonal,term=term)
        Sig[,,l] <- fit.glasso$w
        SigInv[,,l] <- fit.glasso$wi
      }
      if(lambda==0){
        Sig[,,l] <- samplecov
        SigInv[,,l] <- solve(samplecov)
      }
      if(lambda==Inf){
        SigInv[,,l] <- diag(1/diag(samplecov))
        Sig[,,l] <- diag(diag(samplecov))
      }
    }
    mix.prob <- mix.prob.init
    
    ##Start EM iteration
    iter <- 0
    err1 <- Inf #convergence of parameters
    param <- as.vector(Sig)
    warn <- 'NONE'

    while((err1>term)&(iter<maxiter)){
      if (show.trace==TRUE){
        cat('         -mixglasso: iter',iter,'\n')
      }
      ##Estep
      logphi <- matrix(NA,n,n.comp)
      for (l in 1:n.comp){
        logphi[,l] <- dmvnorm(x,Mu[,l],Sig[,,l],log=TRUE)
      }
      fit.E <- EXPStep.mix(logphi,mix.prob)
      unew <- fit.E$u
      loglik <- fit.E$loglik

      if (((any(colSums(unew)<=min.compsize))&(iter>miniter))|any(colSums(unew)<=5)){#min.compsize is a tuning-param for stoping EM; EM stops always if compsize<5
        compsize <- colSums(u)
        cat("         -mixglasso: comp too small; min(n_k)=",min(colSums(unew)),'\n')
        warn <- "comp too small"
        ##compute Bic&Mmdl
        SigInv[abs(SigInv)<10^{-3}] <- 0
        n.zero <- sum(SigInv==0)
        DF <- p*n.comp+(n.comp-1)+n.comp*(p*(p+1)/2)-n.zero/2#df.mean+df.mix.prob+df.inv.covariance
        n.zero.percomp <- apply(SigInv==0,3,sum)
        penmmdl <- p*sum(log(compsize))+log(n)*(n.comp-1)+sum(log(compsize)*((p*(p+1)/2)-n.zero.percomp/2))
  
        return(list(mix.prob=mix.prob,Mu=Mu,Sig=Sig,SigInv=SigInv,iter=iter,
                    loglik=loglik,bic=-loglik+log(n)*DF/2,mmdl=-loglik+penmmdl/2,
                    u=u,comp=apply(u,1,which.max),
                    compsize=compsize,pi.comps=colMeans(u),
                    warn=warn))
        break
      }#if (((any(colSums(unew)<=min.compsize))&(iter>miniter))|any(colSums(unew)<=5)){
      
      u<-unew#if compsize>min.compsize then continue EM
      ##Mstep
      fit.M <- MStepGlasso(x=x,u=u,lambda=lambda,gamma=gamma,pen=pen,penalize.diagonal=penalize.diagonal,term=term,model='mixture')
      mix.prob <- fit.M$mix.prob
      Mu <- fit.M$Mu
      Sig <- fit.M$Sig
      SigInv <- fit.M$SigInv

      param.old <- param
      param <- as.vector(Sig)
      err1 <- max(abs(param-param.old)/(1+abs(param)))
      iter <- iter+1
    }#end while((err1>term)&(iter<maxiter)){
    if(maxiter==0){
      logphi <- matrix(NA,n,n.comp)
      for (l in 1:n.comp){
        logphi[,l] <- dmvnorm(x,Mu[,l],Sig[,,l],log=TRUE)
      }
      loglik <- EXPStep.mix(logphi,mix.prob)$loglik
    }#end if(maxiter==0){

    ##compute Bic/Mmdl
    SigInv[abs(SigInv)<10^{-3}] <- 0
    n.zero <- sum(SigInv==0)
    compsize <- colSums(u)
    DF <- p*n.comp+(n.comp-1)+n.comp*(p*(p+1)/2)-n.zero/2#df.mean+df.mix.prob+df.inv.covariance
    n.zero.percomp <- apply(SigInv==0,3,sum)
    penmmdl <- p*sum(log(compsize))+log(n)*(n.comp-1)+sum(log(compsize)*((p*(p+1)/2)-n.zero.percomp/2))
    
    list(mix.prob=mix.prob,Mu=Mu,Sig=Sig,SigInv=SigInv,iter=iter,
         loglik=loglik,bic=-loglik+log(n)*DF/2,mmdl=-loglik+penmmdl/2,
         u=u,comp=apply(u,1,which.max),
         compsize=compsize,pi.comps=colMeans(u),
         warn=warn)
  }#end else { [(n.comp!=1)]
}



##' mixglasso
##'
##' Runs mixture of graphical lasso network clustering with one or several
##' numbers of mixture components.
##' 
##' @title mixglasso
##' @param x Input data matrix
##' @param n.comp Number of mixture components. If n.comp is a vector, 
##' \code{mixglasso} will estimate a model for each number of mixture components,
##' and return a list of models, as well as their BIC and MMDL scores and the index of the
##' best model according to each score.
##' @param lambda Regularization parameter. Default=sqrt(2*n*log(p))/2
##' @param pen Determines form of penalty: glasso.parcor (default) to penalise the
##' partial correlation matrix, glasso.invcov to penalise the inverse covariance 
##' matrix (this corresponds to classical graphical lasso), glasso.invcor to 
##' penalise the inverse correlation matrix.
##' @param init Initialization. Method used for initialization
##' init={'cl.init','r.means','random','kmeans','kmeans.hc','hc'}. Default='kmeans'
##' @param my.cl Initial cluster assignments;
##' need to be provided if init='cl.init' (otherwise this param is ignored). Default=NULL
##' @param modelname.hc Model class used in hc. Default="VVV"
##' @param nstart.kmeans Number of random starts in kmeans; default=1
##' @param iter.max.kmeans Maximal number of iteration in kmeans; default=10
##' @param term Termination criterion of EM algorithm. Default=10^-3
##' @param min.compsize Stop EM if any(compsize)<min.compsize; Default=5
##' @param save.allfits If TRUE, save output of mixglasso for all k's.
##' @param filename If \code{save.allfits} is TRUE, output of mixglasso will be
##' saved as \code{paste(filename, _fit.mixgl_k.rda, sep='')}.
##' @param mc.flag If \code{TRUE} use parallel execution for each n.comp via function 
##' \code{mclapply} of package \code{parallel}.
##' @param mc.set.seed See mclapply. Default=FALSE
##' @param mc.preschedule See mclapply. Default=FALSE
##' @param mc.cores Number of cores to use in parallel execution. Defaults to
##' mc.cores option if set, or 2 otherwise.
##' @param ... Other arguments. See mixglasso_init
##' @return A list with elements:
##' \item{models}{List with each element i containing an S3 object of class 
##' 'nethetclustering' that contains the result of fitting the mixture 
##' graphical lasso model with n.comps[i] components. See the documentation of
##' mixglasso_ncomp_fixed for the description of this object.}
##' \item{bic}{BIC for all fits.}
##' \item{mmdl}{Minimum description length score for all fits.}
##' \item{comp}{Component assignments for all fits.}
##' \item{bix.opt}{Index of model with optimal BIC score.}
##' \item{mmdl.opt}{Index of model with optimal MMDL score.}
##' @author n.stadler
##' @export
##' @import grDevices graphics stats utils
##' @example tests/Examples/mixglasso_ex.R
mixglasso <- function(x,n.comp,
                          lambda=sqrt(2*nrow(x)*log(ncol(x)))/2,
                          pen='glasso.parcor',
                          init='kmeans.hc',my.cl=NULL,modelname.hc="VVV",nstart.kmeans=1,iter.max.kmeans=10,
                          term=10^{-3},min.compsize=5,
                          save.allfits=FALSE,filename='mixglasso_fit.rda', mc.flag=FALSE,
                          mc.set.seed=FALSE, mc.preschedule = FALSE, 
											    mc.cores=getOption("mc.cores", 2L), ...){
                  
	if(mc.flag && .Platform$OS.type == "unix") {
    res <- mclapply(1:length(n.comp),
                  FUN=function(k){
                    fit.mixgl <-mixglasso_ncomp_fixed(x,n.comp[k],lambda=lambda,pen=pen,
                                          init=init,my.cl=my.cl,modelname.hc=modelname.hc,
                                          nstart.kmeans=nstart.kmeans,iter.max.kmeans=iter.max.kmeans,
                                          term=term,min.compsize=min.compsize,...)
                    return(fit.mixgl)},
                  mc.set.seed=mc.set.seed, mc.preschedule = mc.preschedule, mc.cores=mc.cores)
	} else {
		if(mc.flag) warning('Windows does not support parallelisation via mclapply, using sequential lapply instead.')
		res <- lapply(1:length(n.comp),
										FUN=function(k){
											fit.mixgl <-mixglasso_ncomp_fixed(x,n.comp[k],lambda=lambda,pen=pen,
																												init=init,my.cl=my.cl,modelname.hc=modelname.hc,
																												nstart.kmeans=nstart.kmeans,iter.max.kmeans=iter.max.kmeans,
																												term=term,min.compsize=min.compsize,...)
											return(fit.mixgl)})
	}

	if(save.allfits) save(res,file=filename)
	
  res.mmdl <- sapply(res,function(x){x[['mmdl']]})
  res.bic <- sapply(res,function(x){x[['bic']]})
  res.comp <- sapply(res,function(x){x[['comp']]})
  #res.iter <- sapply(res,function(x){x[['iter']]})
  #res.warn <- sapply(res,function(x){x[['warn']]})
  return(list(models=res, n.comp=n.comp, bic=res.bic, mmdl=res.mmdl, comp=res.comp, 
  						bic.opt=which.min(res.bic), mmdl.opt=which.min(res.mmdl)))
}

##' Mixglasso with backward pruning
##'
##' This function runs mixglasso with various number of mixture components:
##' It starts with a too large number of components and iterates towards solutions
##' with smaller number of components by initializing using previous solutions.
##' @title bwprun_mixglasso
##' @param x Input data matrix
##' @param n.comp.min Minimum number of components. Take n.comp.min=1 !
##' @param n.comp.max Maximum number of components
##' @param lambda Regularization parameter. Default=sqrt(2*n*log(p))/2
##' @param pen Determines form of penalty: glasso.parcor (default), glasso.invcov, glasso.invcor
##' @param selection.crit Selection criterion. Default='mmdl'
##' @param term Termination criterion of EM algorithm. Default=10^-3
##' @param min.compsize Stop EM if any(compsize)<min.compsize; Default=5
##' @param init Initialization. Method used for initialization
##' init={'cl.init','r.means','random','kmeans','kmeans.hc','hc'}. Default='kmeans.hc'
##' @param my.cl Initial cluster assignments; need to be provided if init='cl.init' (otherwise this param is ignored). Default=NULL
##' @param modelname.hc Model class used in hc. Default="VVV"
##' @param nstart.kmeans Number of random starts in kmeans; default=1
##' @param iter.max.kmeans Maximal number of iteration in kmeans; default=10
##' @param reinit.out Re-initialization if compsize<min.compsize (at the start of algorithm) ?
##' @param reinit.in Re-initialization if compsize<min.compsize (at the bwprun-loop level of algorithm) ?
##' @param mer Merge closest comps for initialization
##' @param del Delete smallest comp for initialization
##' @param ... Other arguments. See mixglasso_init
##' @return list consisting of
##' \item{selcrit}{Selcrit for all models with number of components between n.comp.min and n.comp.max}
##' \item{res.init}{Initialization for all components}
##' \item{comp.name}{List of names of components. Indicates which states where merged/deleted during backward pruning}
##' \item{re.init.in}{Logical vector indicating whether re-initialization was performed or not}
##' \item{fit.mixgl.selcrit}{Results for model with optimal number of components. List see mixglasso_init}
##' @author n.stadler
##' @export
##' @examples
##' ##generate data
##' set.seed(1)
##' n <- 1000
##' n.comp <- 3
##' p <- 10
##'
##' # Create different mean vectors
##' Mu <- matrix(0,p,n.comp)
##'
##' nonzero.mean <- split(sample(1:p),rep(1:n.comp,length=p))
##' for(k in 1:n.comp){
##'   Mu[nonzero.mean[[k]],k] <- -2/sqrt(ceiling(p/n.comp))
##' }
##'
##' sim <- sim_mix_networks(n, p, n.comp, Mu=Mu)
##'
##' ##run mixglasso
##'
##' fit <-  bwprun_mixglasso(sim$data,n.comp=1,n.comp.max=5,selection.crit='bic')
##' plot(fit$selcrit,ylab='bic',xlab='Num.Comps',type='b')
bwprun_mixglasso <- function(x,
                             n.comp.min=1,n.comp.max,
                             lambda=sqrt(2*nrow(x)*log(ncol(x)))/2,
                             pen='glasso.parcor',selection.crit='mmdl',
                             term=10^{-3},min.compsize=5,
                             init='kmeans.hc',my.cl=NULL,modelname.hc="VVV",nstart.kmeans=1,iter.max.kmeans=10,
                             reinit.out=FALSE,reinit.in=FALSE,mer=TRUE,del=TRUE,...){
  ##Backward Pruning MixGLasso 8!!!!! so far only working for equal.prob.trans=TRUE
  ##
  ##x: nxp-data
  ##n.comp.min:  should be smaller than the optimal number of comps, e.g. n.comp.min=1
  ##n.comp.max:  should be considerably larger than the optimal number of comps
  ##lambda:
  ##pen: default 'glasso.parcor'
  ##selection.crit: 'mmdl'/'bic'
  ##term: termination of algorithm
  ##min.compsize: default =5
  ##init: initialisation at Kmax
  ##reinit.out: do re-initialization, if compsize<min.compsize, at the start of algorithm ?
  ##reinit.in: do re-initialization, if compsize<min.compsize, at the bwprun-loop level of algorithm ?
  ##mer:
  ##del:

  ##initialization
  nonvirtual <- apply(!is.na(x),1,all)
  x <- x[nonvirtual,]
  n <- nrow(x)
  p <- ncol(x)
  fit.init.u <- func.uinit(x,n.comp.max,init=init,my.cl=my.cl,nstart.kmeans=nstart.kmeans,iter.max.kmeans=iter.max.kmeans,modelname.hc=modelname.hc)
  u <- fit.init.u$u
  mix.prob <- rep(1/n.comp.max,n.comp.max)
  fit.mixgl <-  mixglasso_init(x=x,n.comp=n.comp.max,lambda=lambda,
                               gamma=0.5,pen=pen,
                               u.init=u,mix.prob.init=mix.prob,
                               min.compsize=min.compsize,term=term,...)

  if(reinit.out==TRUE){
    while(fit.mixgl$warn=='comp too small'){#if 'compsize<min.compsize' then set Kmax<-Kmax-1 and re-initialize with 'init'
      cat('bwprun at Kmax: comp too small, re-initialize & Kmax<-Kmax-1','\n')
      n.comp.max <- n.comp.max-1
      fit.init.u <- func.uinit(x,n.comp.max,init=init,my.cl=my.cl,nstart.kmeans=nstart.kmeans,iter.max.kmeans=iter.max.kmeans,modelname.hc=modelname.hc)
      u <- fit.init.u$u
      mix.prob <- rep(1/n.comp.max,n.comp.max)
      fit.mixgl <-  mixglasso_init(x=x,n.comp=n.comp.max,lambda=lambda,
                                   gamma=0.5,pen=pen,
                                   u.init=u,mix.prob.init=mix.prob,
                                   min.compsize=min.compsize,term=term,...)
    }
  }
  cat('Kmax=',n.comp.max,'\n')

  ##Start with Bwprun
  ##save initialization,comp.name,selcrit.mixgl,re_init_in for k=1,...,K_max
  res.init<-list()
  comp.name <- list()
  comp.name[[paste('K=',n.comp.max,sep='')]] <- as.character(1:n.comp.max)
  selcrit.mixgl <- rep(NA,n.comp.max)
  re_init_in <- rep(FALSE,n.comp.max)
  
  fit.mixgl.del <- fit.mixgl.mer <- list()
  fit.mixgl.del$warn <- fit.mixgl.mer$warn <- FALSE
  selcrit.del <- selcrit.mer <- Inf

  k <- n.comp.max
  while (k>=n.comp.min){
    cat('bwprun: nr comps',k,'\n')
    u <- fit.mixgl$u
    mix.prob <- fit.mixgl$mix.prob
    compsize <- fit.mixgl$compsize
    pi.comps <- fit.mixgl$pi.comps
    selcrit.mixgl[k] <- fit.mixgl[[selection.crit]]
    res.init[[paste('K=',k,sep='')]]<-list(u=u,mix.prob=fit.mixgl$mix.prob)
    
    if(k!=n.comp.min){

      if(del==TRUE){
        ##delete smallest comp
        sm.comp <- which.min(compsize)
        u.del <- u[,-sm.comp,drop=FALSE]
        u.del[which(rowSums(u.del)==0),]<-1/(k-1)
        u.del <- u.del/rowSums(u.del)
        mix.prob.del <- mix.prob[-sm.comp]
        mix.prob.del[sum(mix.prob.del)==0]<-1/(k-1)
        fit.mixgl.del <-  mixglasso_init(x,n.comp=k-1,lambda=lambda,
                                         gamma=0.5,pen=pen,
                                         u.init=u.del,mix.prob.init=mix.prob.del,
                                         min.compsize=min.compsize,term=term,...)
        selcrit.del<-fit.mixgl.del[[selection.crit]]
      }
      if(mer==TRUE){
        ##merge closest comps
        bwcomp.kldist <- w.kldist(fit.mixgl$Mu,fit.mixgl$Sig)
        sel.k <- which(bwcomp.kldist$comp.kldist==bwcomp.kldist$min.comp.kldist,arr.ind=TRUE)[1,]#row(matrix(,k,k))[bwcomp.kldist$comp.kldist==bwcomp.kldist$min.comp.kldist]
        u.mer <- cbind(u[,-sel.k],rowSums(u[,sel.k]))
        mix.prob.mer <- c(mix.prob[-sel.k],sum(mix.prob[sel.k]))
        fit.mixgl.mer <-  mixglasso_init(x=x,n.comp=k-1,lambda=lambda,
                                         gamma=0.5,pen=pen,
                                         u.init=u.mer,mix.prob.init=mix.prob.mer,
                                         min.compsize=min.compsize,term=term,...)
        selcrit.mer<-fit.mixgl.mer[[selection.crit]]
      }
      
      if((reinit.in)&((fit.mixgl.del$warn=="comp too small")|(fit.mixgl.mer$warn=="comp too small"))){#if 'compsize<min.compsize' then set K<-K-1 and re-initialize with 'init'
        cat('        comp too small, re-initialize at K-1:','\n')
        cat('        warn(mixglasso.del) ',fit.mixgl.del$warn,'\n')
        cat('        warn(mixglasso.mer) ',fit.mixgl.mer$warn,'\n')
        re_init_in[k-1] <- TRUE
        comp.name[[paste('K=',k-1,sep='')]] <- as.character(1:(k-1))
        fit.init.u <- func.uinit(x,k-1,init=init,my.cl=my.cl,nstart.kmeans=nstart.kmeans,iter.max.kmeans=iter.max.kmeans,modelname.hc=modelname.hc)
        fit.mixgl <-  mixglasso_init(x=x,n.comp=k-1,lambda=lambda,
                                     gamma=0.5,pen=pen,
                                     u.init=fit.init.u$u,
                                     mix.prob.init=rep(1/(k-1),(k-1)),
                                     min.compsize=min.compsize,term=term,...)
      }else{#if 'no re-initialization'
        if (selcrit.del<selcrit.mer){
          cat('        merge or delete?','delete comp ',comp.name[[paste('K=',k,sep='')]][sm.comp],'\n\n')
          fit.mixgl<-fit.mixgl.del
          comp.name[[paste('K=',k-1,sep='')]] <- comp.name[[paste('K=',k,sep='')]][-sm.comp]
        }else{
          cat('        merge or delete?','merge comps ',comp.name[[paste('K=',k,sep='')]][sel.k],'\n\n')
          fit.mixgl<-fit.mixgl.mer
          comp.name[[paste('K=',k-1,sep='')]] <- c(comp.name[[paste('K=',k,sep='')]][-sel.k],paste(comp.name[[paste('K=',k,sep='')]][sel.k],collapse='+'))
        }
      }
    }#end if(k!=n.comp.min){
    k <- k-1
  }
  #optimal selcrit-solution
  fit.mixgl.selcrit <-  mixglasso_init(x,n.comp=which.min(selcrit.mixgl),lambda=lambda,
                                       gamma=0.5,pen=pen,
                                       u.init=res.init[[paste('K=',which.min(selcrit.mixgl),sep='')]]$u,
                                       mix.prob.init=res.init[[paste('K=',which.min(selcrit.mixgl),sep='')]]$mix.prob,
                                       min.compsize=min.compsize,term=term,...)
  
  return(list(selcrit=selcrit.mixgl,res.init=res.init,comp.name=comp.name,re.init.in=re_init_in,fit.mixgl.selcrit=fit.mixgl.selcrit))
}

##' Log-likelihood for mixture model
##'
##' 
##' @title Log-likelihood for mixture model
##' @param x no descr
##' @param mix.prob no descr
##' @param Mu no descr
##' @param Sig no descr
##' @return log-likelihood
##' @author n.stadler
##' @keywords internal
loglik_mix<- function(x,mix.prob,Mu,Sig){
  ##computes loglikelihood of model
  n.comp <- ncol(Mu)
  n <- nrow(x)
  if(n.comp==1){
    return(sum(dmvnorm(x,Mu[,1],Sig[,,1],log=TRUE)))
  }
  else{
    logphi <- matrix(NA,n,n.comp)
    for (l in 1:n.comp){
      logphi[,l] <- dmvnorm(x,Mu[,l],Sig[,,l],log=TRUE)
    }
    loglik <- EXPStep.mix(logphi,mix.prob)$loglik
    return(loglik)
  }
}

##' Generates sparse inverse covariance matrices
##'
##' 
##' @title Generates sparse inverse covariance matrices
##' @param p Dimensionality of inverse covariance matrix
##' @param K Number of inverse covariance matrices
##' @param s Number of non-zero entries per inverse covariance matrix
##' @param s.common Number of non-zero entries shared across different inverse covariance matrices
##' @param magn.nz Magnitude of non-zero elements
##' @param scale.parcor Should SigInv be scaled to have diagonal equal one, siginv=parcor ?
##' @return SigInv: list of inverse covariance matrices
##' @author n.stadler
##' @keywords internal
sparse_conc <- function(p,K,s,s.common,magn.nz=0.5,scale.parcor=TRUE){
    ##Generate K different Sparse Inverse Covariance-Matrices of dimension p:
    ##
    ##-condition number=p (necessary when comparing different performance wrt various p's; if scale.parcor=TRUE, then sparse_conc does not depend on magn.nz)
    ##-for each SigInv there are s non-zero entries
    ##-s.common locations of non-zero entries are common among all SigInv;
    ## whereas s-s.common non-zero entries are at different locations
    

  if(s==0){
    SigInv <- list()
    for (k in 1:K){
      SigInv[[k]] <- diag(1,p)
    }
  }else{
    

    ind.upper.tri <- which(upper.tri(matrix(NA,p,p)))
    
    B.list <- list()
    B <- matrix(0,p,p)
    comp.nonzero <- tot.nonzero <- sample(ind.upper.tri,size=s,replace=FALSE)
    same.nonzero <- sample(comp.nonzero,size=s.common,replace=FALSE)
    remain.zero <- setdiff(ind.upper.tri,comp.nonzero)
    B[comp.nonzero] <- magn.nz
    B.list[[1]] <- B+t(B)
    if (K>1){
      for (k in 2:K){
        B <- matrix(0,p,p)
        comp.nonzero <- c(same.nonzero,sample(remain.zero,size=s-s.common,replace=FALSE))
        tot.nonzero <- union(tot.nonzero,comp.nonzero)
        B[comp.nonzero] <- magn.nz
        B.list[[k]] <- B+t(B)
        remain.zero <- setdiff(ind.upper.tri,tot.nonzero)
      }
    }

    SigInv <- list()
    for (k in 1:K){
      SigInv[[k]] <- B.list[[k]]
      ev <- eigen(SigInv[[k]])$values
      del <- (max(ev)-min(ev)*p)/(p-1)
      diag(SigInv[[k]]) <- del
      if(scale.parcor==TRUE){
        SigInv[[k]] <- SigInv[[k]]/del
      }
    }
  }
    
    return(SigInv)
}

###############################################################################
# High-Dimensional Two-Sample Testing for Gaussian Graphical Model
#------------------------------------------------------------------
#
#
###############################################################################

#####################
# Required Packages 
#####################
library(mvtnorm)
library(glasso)
#library(parcor)
library(GeneNet)
library(huge)
library(CompQuadForm)
library(ggm)


#############################
##-------Screening---------##
#############################

##' Lambdamax
##'
##' 
##' @title Lambdamax
##' @param x no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
lambda.max <- function(x){
  n <- nrow(x)
  s.var <- var(x)
  diag(s.var) <- 0
  return(n*max(abs(s.var))/2)
}

##' Lambda-grid (log scale)
##'
##' 
##' @title Lambda-grid
##' @param lambda.min no descr
##' @param lambda.max no descr
##' @param nr.gridpoints no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
lambdagrid_mult <- function(lambda.min,lambda.max,nr.gridpoints){
    mult.const <- (lambda.max/lambda.min)^(1/(nr.gridpoints-1))
    return(lambda.min*mult.const^((nr.gridpoints-1):0))
}

##' Lambda-grid (linear scale)
##'
##' 
##' @title Lambda-grid
##' @param lambda.min no descr
##' @param lambda.max no descr
##' @param nr.gridpoints no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
lambdagrid_lin <- function(lambda.min,lambda.max,nr.gridpoints){
    return(seq(lambda.max,lambda.min,length=nr.gridpoints))
}

##' Make grid
##'
##' 
##' @title Make grid
##' @param lambda.min no descr
##' @param lambda.max no descr
##' @param nr.gridpoints no descr
##' @param method no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
make_grid <- function(lambda.min,lambda.max,nr.gridpoints,method='lambdagrid_mult'){
  eval(as.name(method))(lambda.min,lambda.max,nr.gridpoints)
}

##' Error bars for plotCV
##'
##' 
##' @title Error bars for plotCV
##' @param x no descr
##' @param upper no descr
##' @param lower no descr
##' @param width no descr
##' @param ... no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
error.bars <- function (x, upper, lower, width = 0.02, ...)
{
    xlim <- range(x)
    barw <- diff(xlim) * width
    segments(x, upper, x, lower, ...)
    segments(x - barw, upper, x + barw, upper, ...)
    segments(x - barw, lower, x + barw, lower, ...)
    range(upper, lower)
}

##' plotCV
##'
##' 
##' @title plotCV
##' @param lambda no descr
##' @param cv no descr
##' @param cv.error no descr
##' @param se no descr
##' @param type no descr
##' @param ... no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
plotCV <- function(lambda,cv,cv.error,se=TRUE,type='b',...){
  if (se==TRUE){ylim <- range(cv,cv+cv.error,cv-cv.error)}
  else {ylim <- range(cv)}
  plot(lambda,cv,type=type,ylim=ylim,...)
  if (se)
    error.bars(lambda,cv+cv.error,cv-cv.error,width=1/length(lambda))
  invisible()
}

##' Make folds
##'
##' 
##' @title Make folds
##' @param n no descr
##' @param folds no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
cv.fold <- function(n,folds=10){
  split(sample(1:n),rep(1:folds,length=n))
}

##' Graphical Lasso path with huge package
##'
##' 
##' @title Graphical Lasso path with huge package
##' @param s no descr
##' @param rholist no descr
##' @param penalize.diagonal no descr
##' @param trace no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
hugepath <- function(s,rholist,penalize.diagonal=NULL,trace=NULL){
  #fit.huge <- huge(s,method = "glasso",cov.output =TRUE,verbose = FALSE)
  fit.huge <- huge(s,lambda=sort(rholist,decreasing=TRUE),method = "glasso",cov.output =TRUE,verbose = FALSE)
  wi <- sapply(fit.huge$icov,as.matrix,simplify='array')
  w <- sapply(fit.huge$cov,as.matrix,simplify='array')
  #return(list(wi=wi[,,length(fit.huge$lambda):1],w=w[,,length(fit.huge$lambda):1]))
  return(list(rholist=rholist,wi=wi[,,length(rholist):1],w=w[,,length(rholist):1]))
}

##' Additional thresholding
##'
##' 
##' @title Additional thresholding
##' @param n no descr
##' @param wi no descr
##' @param method no descr
##' @param trunc.k no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
mytrunc.method <- function(n,wi,method='linear.growth',trunc.k=5){
  p <- ncol(wi)
  if (method=='none'){
    return(list(wi=wi))
  }
  if(method=='linear.growth'){
    wi.trunc <- wi
    diag(wi.trunc) <- 0
    nonzero <- min(2*ceiling(p*ceiling(n/trunc.k)/2),sum(wi.trunc!=0))
    wi.trunc[-order(abs(wi.trunc),decreasing=TRUE)[1:nonzero]] <- 0
    diag(wi.trunc) <- diag(wi)
    return(list(wi=wi.trunc))
  }
  if(method=='sqrt.growth'){
    wi.trunc <- wi
    diag(wi.trunc) <- 0
    nonzero <- min(2*ceiling(p*sqrt(n)/2),sum(wi.trunc!=0))
    wi.trunc[-order(abs(wi.trunc),decreasing=TRUE)[1:nonzero]] <- 0
    diag(wi.trunc) <- diag(wi)
    return(list(wi=wi.trunc))
  }
}

##' Crossvalidation for GLasso 
##'
##' 8! lambda-grid has to be increasing (see glassopath)
##' @title Crossvalidation for GLasso
##' @param x no descr
##' @param folds no descr
##' @param lambda lambda-grid (increasing!)
##' @param penalize.diagonal no descr
##' @param plot.it no descr
##' @param se no descr
##' @param include.mean no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
cv.glasso <- function(x,folds=10,lambda,penalize.diagonal=FALSE,plot.it=FALSE,se=TRUE,include.mean=FALSE)
{
  colnames(x)<-paste('x',1:ncol(x),sep='')  
  all.folds <- cv.fold(nrow(x),folds)
  residmat <- matrix(NA,folds,length(lambda))
  for (cvfold in 1:folds){
    omit <- all.folds[[cvfold]]
    s <- var(x[-omit,])
    if (include.mean==TRUE){
      mu <- colMeans(x[-omit,,drop=FALSE])
    }else{
      mu <- rep(0,ncol(x))
    }
    fit.path <- glassopath(s,rholist=2*lambda/nrow(x),penalize.diagonal=penalize.diagonal,trace=0)
    if(length(omit)==1){
      residmat[cvfold,] <- -2*apply(fit.path$w,3,dmvnorm,log=TRUE,mean=mu,x=x[omit,,drop=FALSE])
    }else{
      residmat[cvfold,] <- apply(-2*apply(fit.path$w,3,dmvnorm,log=TRUE,mean=mu,x=x[omit,,drop=FALSE]),2,sum)
    }
  }
  cv <- apply(residmat,2,mean)
  cv.error <- sqrt(apply(residmat,2,var)/folds)
  gl.opt<-glasso(var(x),rho=2*lambda[which.min(cv)]/nrow(x),penalize.diagonal=penalize.diagonal)
  cat('la.opt:',lambda[which.min(cv)],'\n')
  w<-gl.opt$w
  wi<-gl.opt$wi
  wi[abs(wi)<10^{-3}]<-0
  colnames(w)<-rownames(w)<-colnames(wi)<-rownames(wi)<-colnames(x)  

  object <- list(rho.opt=2*lambda[which.min(cv)]/nrow(x),lambda=lambda,residmat=residmat,cv=cv,cv.error=cv.error,w=w,wi=wi,mu=colMeans(x))
  if (plot.it){
    plotCV(lambda,cv,cv.error,se=se)
  }
  invisible(object)
}

##' Cross-validated glasso with additional thresholding
##'
##'
##' Run glasso on a single dataset, using cross-validation to estimate the
##' penalty parameter lambda. Performs additional thresholding (optionally).
##' 
##' @title Cross-validated glasso with additional thresholding
##' @param x The input data. Needs to be a num.samples by dim.samples matrix.
##' @param include.mean Include mean in likelihood. TRUE / FALSE (default).
##' @param folds Number of folds in the cross-validation (default=10).
##' @param length.lambda Length of lambda path to consider (default=20).
##' @param lambdamin.ratio Ratio lambda.min/lambda.max.
##' @param penalize.diagonal If TRUE apply penalization to diagonal of inverse
##'        covariance as well. (default=FALSE)
##' @param trunc.method None / linear.growth (default) / sqrt.growth
##' @param trunc.k truncation constant, number of samples per predictor (default=5)
##' @param plot.it TRUE / FALSE (default)
##' @param se default=FALSE.
##' @param use.package 'glasso' or 'huge' (default).
##' @param verbose If TRUE, output la.min, la.max and la.opt (default=FALSE).
##' @return  Returns a list with named elements 'rho.opt', 'w', 'wi', 'wi.orig', 
##'          'mu'. Variable rho.opt is the optimal (scaled) penalization 
##'          parameter (rho.opt=2*la.opt/n). Variable w is the estimated 
##'          covariance matrix. The variables wi and wi.orig are 
##'          matrices of size dim.samples by dim.samples containing the 
##'          truncated and untruncated inverse covariance matrix. Variable 
##'          mu is the mean of the input data.
##' @author n.stadler
##' @export
##' @examples
##' n=50
##' p=5
##' x=matrix(rnorm(n*p),n,p)
##' wihat=screen_cv.glasso(x,folds=2)$wi
screen_cv.glasso <- function(x,include.mean=FALSE,
                             folds=min(10,dim(x)[1]),
														 length.lambda=20,lambdamin.ratio=ifelse(ncol(x)>nrow(x),0.01,0.001),penalize.diagonal=FALSE,
                             trunc.method='linear.growth',trunc.k=5,plot.it=FALSE,se=FALSE,use.package='huge',verbose=FALSE)
{ 
	
	if(dim(x)[1] < 5) stop('Sample size too small to perform cross-validation')

  gridmax <- lambda.max(x)
  gridmin <- lambdamin.ratio*gridmax
  lambda <- make_grid(gridmin,gridmax,length.lambda)[length.lambda:1]
  
  colnames(x)<-paste('x',1:ncol(x),sep='')  
  all.folds <- cv.fold(nrow(x),folds)
  residmat <- matrix(NA,folds,length(lambda))
  for (cvfold in 1:folds){
    omit <- all.folds[[cvfold]]
    s <- var(x[-omit,])
    if (include.mean==TRUE){
      mu <- colMeans(x[-omit,,drop=FALSE])
    }else{
      mu <- rep(0,ncol(x))
    }
    fit.path <- eval(as.name(paste(use.package,'path',sep='')))(s,rholist=2*lambda/nrow(x),penalize.diagonal=penalize.diagonal,trace=0)
    if(length(omit)==1){
      residmat[cvfold,] <- -2*apply(fit.path$w,3,dmvnorm,log=TRUE,mean=mu,x=x[omit,,drop=FALSE])
    }else{
      residmat[cvfold,] <- apply(-2*apply(fit.path$w,3,dmvnorm,log=TRUE,mean=mu,x=x[omit,,drop=FALSE]),2,sum)
    }
  }
  cv <- apply(residmat,2,mean)
  cv.error <- sqrt(apply(residmat,2,var)/folds)
  gl.opt<-glasso(var(x),rho=2*lambda[which.min(cv)]/nrow(x),penalize.diagonal=penalize.diagonal)
  if(verbose){
    cat('la.min:',gridmin,'\n')
    cat('la.max:',gridmax,'\n')
    cat('la.opt:',lambda[which.min(cv)],'\n')
  }
  w<-gl.opt$w
  wi<-gl.opt$wi
  wi[abs(wi)<10^{-3}]<-0
  wi <- (wi+t(wi))/2
  colnames(w)<-rownames(w)<-colnames(wi)<-rownames(wi)<-colnames(x)
  wi.trunc <- mytrunc.method(n=nrow(x),wi=wi,method=trunc.method,trunc.k=trunc.k)$wi
  
  if (plot.it){
    plotCV(lambda,cv,cv.error,se=se)
  }
  list(rho.opt=2*lambda[which.min(cv)]/nrow(x),w=w,wi=wi.trunc,wi.orig=wi,
  		 mu=colMeans(x))
}

##' BIC.glasso
##'
##' 
##' @title BIC.glasso
##' @param x no descr
##' @param lambda no descr
##' @param penalize.diagonal no descr
##' @param plot.it no descr
##' @param use.package no descr
##' @param include.mean no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
bic.glasso <- function(x,lambda,penalize.diagonal=FALSE,plot.it=TRUE,use.package='huge',include.mean=FALSE)
{
  ##glasso; lambda.opt with bic
  if(include.mean==TRUE){
    Mu <- colMeans(x)
  }
  if(include.mean==FALSE){
    Mu <-  rep(0,ncol(x))
  }
  samplecov <- var(x)

  if(is.null(lambda)){
    la <- lambda
  }else{
    la <- 2*lambda/nrow(x)
  }

  fit.path <- eval(as.name(paste(use.package,'path',sep='')))(samplecov,rholist=la,penalize.diagonal=penalize.diagonal,trace=0)

  loglik <- lapply(seq(length(fit.path$rholist)),
                   function(i){
                     return(sum(dmvnorm(x,mean=Mu,sigma=fit.path$w[,,i],log=TRUE)))
                   }
                   )
  degfree <- lapply(seq(length(fit.path$rholist)),
                    function(i){
                      wi <- fit.path$wi[,,i]
                      wi[abs(wi)<10^{-3}]<-0
                      p <- ncol(wi)
                      n.zero<-sum(wi==0)
                      if(include.mean==TRUE){
                        dfree <- p+(p*(p+1)/2)-n.zero/2
                      }
                      if(include.mean==FALSE){
                        dfree <- (p*(p+1)/2)-n.zero/2
                      }
                      return(dfree)
                    }
                    )
  loglik <- simplify2array(loglik,higher=TRUE)
  degfree <- simplify2array(degfree,higher=TRUE)
  myscore <- -loglik+log(nrow(x))*degfree/2

  if(is.null(lambda)){
    lambda <- 0.5*nrow(x)*fit.path$rholist
  }
  index.opt <- which.min(myscore)
  wi <- fit.path$wi[,,index.opt]
  wi[abs(wi)<10^{-3}]<-0
  w <- fit.path$w[,,index.opt]
  
  if (plot.it){
    plot(lambda,myscore,type='b',xlab='lambda')
  }
  
  list(rho.opt=2*lambda[which.min(myscore)]/nrow(x),lambda=lambda,la.opt=lambda[index.opt],bic.score=myscore,Mu=Mu,wi=wi,w=w)
}

##' AIC.glasso
##'
##' 
##' @title AIC.glasso
##' @param x no descr
##' @param lambda no descr
##' @param penalize.diagonal no descr
##' @param plot.it no descr
##' @param use.package no descr
##' @param include.mean no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
aic.glasso <- function(x,lambda,penalize.diagonal=FALSE,plot.it=TRUE,use.package='huge',include.mean=FALSE)
{
  ##glasso; lambda.opt with aicc
  if(include.mean==TRUE){
    Mu <- colMeans(x)
  }
  if(include.mean==FALSE){
    Mu <-  rep(0,ncol(x))
  }
  
  samplecov <- var(x)

  if(is.null(lambda)){
    la <- lambda
  }else{
    la <- 2*lambda/nrow(x)
  }

  fit.path <- eval(as.name(paste(use.package,'path',sep='')))(samplecov,rholist=la,penalize.diagonal=penalize.diagonal,trace=0)

  loglik <- lapply(seq(length(fit.path$rholist)),
                   function(i){
                     return(sum(dmvnorm(x,mean=Mu,sigma=fit.path$w[,,i],log=TRUE)))
                   }
                   )
  degfree <- lapply(seq(length(fit.path$rholist)),
                    function(i){
                      wi <- fit.path$wi[,,i]
                      wi[abs(wi)<10^{-3}]<-0
                      p <- ncol(wi)
                      n.zero<-sum(wi==0)
                      if(include.mean==TRUE){
                        dfree <- p+(p*(p+1)/2)-n.zero/2
                      }
                      if(include.mean==FALSE){
                        dfree <- (p*(p+1)/2)-n.zero/2
                      }
                      return(dfree)
                    }
                    )
  loglik <- simplify2array(loglik,higher=TRUE)
  degfree <- simplify2array(degfree,higher=TRUE)
  myscore <- -loglik+2*degfree/2
  if(is.null(lambda)){
    lambda <- 0.5*nrow(x)*fit.path$rholist
  }
  index.opt <- which.min(myscore)
  wi <- fit.path$wi[,,index.opt]
  wi[abs(wi)<10^{-3}]<-0
  w <- fit.path$w[,,index.opt]

  if (plot.it){
    plot(lambda,myscore,type='b',xlab='lambda')
  }
  
  list(rho.opt=2*lambda[which.min(myscore)]/nrow(x),lambda=lambda,la.opt=lambda[index.opt],bic.score=myscore,Mu=Mu,wi=wi,w=w)
}

##' BIC-tuned glasso with additional thresholding
##'
##' 
##' @title BIC-tuned glasso with additional thresholding
##' @param x  The input data. Needs to be a num.samples by dim.samples matrix.
##' @param include.mean Include mean in likelihood. TRUE / FALSE (default).
##' @param length.lambda Length of lambda path to consider (default=20).
##' @param lambdamin.ratio Ratio lambda.min/lambda.max.
##' @param penalize.diagonal  If TRUE apply penalization to diagonal of inverse
##'        covariance as well. (default=FALSE)
##' @param plot.it TRUE / FALSE (default)
##' @param trunc.method None / linear.growth (default) / sqrt.growth
##' @param trunc.k truncation constant, number of samples per predictor (default=5)
##' @param use.package 'glasso' or 'huge' (default).
##' @param verbose If TRUE, output la.min, la.max and la.opt (default=FALSE).
##' @return  Returns a list with named elements 'rho.opt', 'wi', 'wi.orig', 
##'          Variable rho.opt is the optimal (scaled) penalization parameter (rho.opt=2*la.opt/n). 
##'          The variables wi and wi.orig are matrices of size dim.samples by dim.samples 
##'          containing the truncated and untruncated inverse covariance matrix. 
##' @author n.stadler
##' @export
##' @examples
##' n=50
##' p=5
##' x=matrix(rnorm(n*p),n,p)
##' wihat=screen_bic.glasso(x,length.lambda=5)$wi
screen_bic.glasso <- function(x,include.mean=TRUE,
                              length.lambda=20,lambdamin.ratio=ifelse(ncol(x)>nrow(x),0.01,0.001),penalize.diagonal=FALSE,
                              plot.it=FALSE,trunc.method='linear.growth',trunc.k=5,use.package='huge',verbose=FALSE){

  gridmax <- lambda.max(x)
  gridmin <- gridmax*lambdamin.ratio
  my.grid <- make_grid(gridmin,gridmax,length.lambda)[length.lambda:1]
  
  fit.bicgl <- bic.glasso(x,lambda=my.grid,penalize.diagonal=penalize.diagonal,plot.it=plot.it,
                          use.package=use.package,include.mean=include.mean)
  if(verbose){
    cat('la.min:',gridmin,'\n')
    cat('la.max:',gridmax,'\n')
    cat('la.opt:',fit.bicgl$la.opt,'\n')
  }
  wi <- fit.bicgl$wi
  wi.trunc <- mytrunc.method(n=nrow(x),wi=wi,method=trunc.method,trunc.k=trunc.k)$wi
  list(rho.opt=fit.bicgl$rho.opt,wi=wi.trunc,wi.orig=wi) 
}

##' AIC-tuned glasso with additional thresholding
##'
##' 
##' @title AIC-tuned glasso with additional thresholding
##' @param x  The input data. Needs to be a num.samples by dim.samples matrix.
##' @param include.mean Include mean in likelihood. TRUE / FALSE (default).
##' @param length.lambda Length of lambda path to consider (default=20).
##' @param lambdamin.ratio Ratio lambda.min/lambda.max.
##' @param penalize.diagonal  If TRUE apply penalization to diagonal of inverse
##'        covariance as well. (default=FALSE)
##' @param plot.it TRUE / FALSE (default)
##' @param trunc.method None / linear.growth (default) / sqrt.growth
##' @param trunc.k truncation constant, number of samples per predictor (default=5)
##' @param use.package 'glasso' or 'huge' (default).
##' @param verbose If TRUE, output la.min, la.max and la.opt (default=FALSE).
##' @return  Returns a list with named elements 'rho.opt', 'wi', 'wi.orig'.
##'          Variable rho.opt is the optimal (scaled) penalization parameter (rho.opt=2*la.opt/n). 
##'          The variables wi and wi.orig are matrices of size dim.samples by dim.samples 
##'          containing the truncated and untruncated inverse covariance matrix. 
##' @author n.stadler
##' @export
##' @examples
##' n=50
##' p=5
##' x=matrix(rnorm(n*p),n,p)
##' wihat=screen_aic.glasso(x,length.lambda=5)$wi
screen_aic.glasso <- function(x,include.mean=TRUE,length.lambda=20,lambdamin.ratio=ifelse(ncol(x)>nrow(x),0.01,0.001),
                              penalize.diagonal=FALSE,plot.it=FALSE,
                              trunc.method='linear.growth',trunc.k=5,use.package='huge',verbose=FALSE){

  gridmax <- lambda.max(x)
  gridmin <- gridmax*lambdamin.ratio
  my.grid <- make_grid(gridmin,gridmax,length.lambda)[length.lambda:1]
  
  fit.aicgl <- aic.glasso(x,lambda=my.grid,penalize.diagonal=penalize.diagonal,plot.it=plot.it,
                          use.package=use.package,include.mean=include.mean)
  if(verbose){
    cat('la.min:',gridmin,'\n')
    cat('la.max:',gridmax,'\n')
    cat('la.opt:',fit.aicgl$la.opt,'\n')
  }
  wi <- fit.aicgl$wi
  wi.trunc <- mytrunc.method(n=nrow(x),wi=wi,method=trunc.method,trunc.k=trunc.k)$wi
  list(rho.opt=fit.aicgl$rho.opt,wi=wi.trunc,wi.orig=wi) 
}


##' Shrinkage approach for estimating Gaussian graphical model
##'
##' 
##' @title Shrinkage approach for estimating Gaussian graphical model
##' @param x  The input data. Needs to be a num.samples by dim.samples matrix.
##' @param include.mean Include mean in likelihood. TRUE / FALSE (default).
##' @param trunc.method None / linear.growth (default) / sqrt.growth
##' @param trunc.k truncation constant, number of samples per predictor (default=5)
##' @return Returns a list with named elements 'rho.opt', 'wi', 'wi.orig'.
##'          Variable rho.opt=NULL (no tuning parameter involved). 
##'          The variables wi and wi.orig are matrices of size dim.samples by dim.samples 
##'          containing the truncated and untruncated inverse covariance matrix.
##' @author n.stadler
##' @keywords internal
##' 
##' Removed until parcor package is fixed.
##' export
##' examples
##' n=50
##' p=5
##' x=matrix(rnorm(n*p),n,p)
##' wihat=screen_shrink(x)$wi
screen_shrink <- function(x,include.mean=NULL,
                          trunc.method='linear.growth',trunc.k=5){
  wi <- ggm.estimate.pcor(x)
  adj <- performance.pcor(wi, fdr=TRUE,verbose=FALSE,plot.it=FALSE)$adj
  wi[adj==0] <- 0
  wi.trunc <- mytrunc.method(n=nrow(x),wi=wi,method=trunc.method,trunc.k=trunc.k)$wi
  list(rho.opt=NULL,wi=wi.trunc,wi.orig=wi)
}

##' Screen_full
##'
##' 
##' @title Screen_full
##' @param x no descr
##' @param include.mean no descr
##' @param length.lambda no descr
##' @param trunc.method no descr
##' @param trunc.k no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
screen_full <- function(x,include.mean=NULL,length.lambda=NULL,trunc.method=NULL,trunc.k=NULL){
 wi <- diag(1,ncol(x))
 list(rho.opt=NULL,wi=wi)
}


###################################################################
##-------covMethod------
###################################################################

##' Compute covariance matrix
##'
##' 
##' @title Compute covariance matrix
##' @param x no descr
##' @param include.mean no descr
##' @param covMethod no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
mcov <- function(x,include.mean,covMethod='ML'){
  if(covMethod=='ML'){
    return(cov.wt(x,method='ML',center=include.mean)$cov)
  }
  if(covMethod=='var'){
    return(var(x))
  }
}
  
#################################################################
##--------P-VALUES----------
#################################################################

##' MLE in GGM
##'
##' 
##' @title MLE in GGM
##' @param x no descr
##' @param wi no descr
##' @param algorithm no descr
##' @param rho no descr
##' @param include.mean no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
mle.ggm <- function(x,wi,algorithm='glasso_rho0',rho=NULL,include.mean){
  if(is.null(rho)){
    algorithm <- 'glasso_rho0'
    cat('screening method with rho.opt=NULL; set algorithm="glasso_rho0" in mle.ggm')
  }
  if (algorithm=='glasso'){
    fit.mle <- glasso(mcov(x=x,include.mean=include.mean),rho=rho,zero=which(wi==0,arr.ind=TRUE))
    return(list(w=fit.mle$w,wi=fit.mle$wi))
  }
  if (algorithm=='glasso_rho0'){
    fit.mle <- glasso(mcov(x=x,include.mean=include.mean),rho=10^{-10},zero=which(wi==0,arr.ind=TRUE))
    return(list(w=fit.mle$w,wi=fit.mle$wi))
  }
  if (algorithm=='fitcongraph'){
    s <- mcov(x=x,include.mean=include.mean)
    adj <- wi!=0
    colnames(s) <- rownames(s) <- colnames(adj) <- rownames(adj) <- 1:ncol(x)
    fit.mle <- fitConGraph(adj,s,nrow(x))
    w <- fit.mle$Shat
    wi <- solve(w)
    wi[!adj] <- 0
    return(list(w=fit.mle$Shat,wi=wi))
  }
}

##' Log-likelihood-ratio statistics used in Differential Network
##'
##' 
##' @title Log-likelihood-ratio statistics used in DiffNet
##' @param x1 data-matrix sample 1
##' @param x2 data-matrix sample 2
##' @param x pooled data-matrix
##' @param sig1 covariance sample 1
##' @param sig2 covariance sample 2
##' @param sig pooled covariance
##' @param mu1 mean sample 1
##' @param mu2 mean sample 2
##' @param mu pooled mean
##' @return Returns a list with named elements 'twiceLR', 'sig1', 'sig2', 'sig'.
##'         'twiceLR' is twice the log-likelihood-ratio statistic.
##' @author n.stadler
##' @export
##' @examples
##' x1=matrix(rnorm(100),50,2)
##' x2=matrix(rnorm(100),50,2)
##' logratio(x1,x2,rbind(x1,x2),diag(1,2),diag(1,2),diag(1,2),c(0,0),c(0,0),c(0,0))$twiceLR
logratio <- function(x1,x2,x,sig1,sig2,sig,mu1,mu2,mu){
  twiceLR <- 2*(sum(dmvnorm(x1,mean=mu1,sigma=sig1,log=TRUE))+sum(dmvnorm(x2,mean=mu2,sigma=sig2,log=TRUE))-sum(dmvnorm(x,mean=mu,sigma=sig,log=TRUE)))
  list(twiceLR=twiceLR,sig1=sig1,sig2=sig2,sig=sig)
}

##' Compute Information Matrix of Gaussian Graphical Model
##'
##' computes E_0[s(Y;Omega)s(Y;Omega)'] where s(Y;Omega)=(d/dOmega) LogLik
##' @title Information Matrix of Gaussian Graphical Model
##' @param Sig Sig=solve(SigInv) true covariance matrix under H0
##' @param include.mean no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
inf.mat<-function(Sig,include.mean=FALSE){
  k <- ncol(Sig)
  uptri.rownr <- row(Sig)[upper.tri(Sig,diag=TRUE)]
  uptri.colnr <- col(Sig)[upper.tri(Sig,diag=TRUE)]

  if (include.mean==TRUE){
    p <- k+k*(k+1)/2
    infmat <- matrix(0,p,p)
    for (i in seq(p-k)){
      colnr.i <- uptri.colnr[i]
      rownr.i <- uptri.rownr[i]
      for (j in seq(p-k)){
        colnr.j <- uptri.colnr[j]
        rownr.j <- uptri.rownr[j]
        infmat[i,j] <- Sig[colnr.i,rownr.j]*Sig[rownr.i,colnr.j]+Sig[colnr.i,colnr.j]*Sig[rownr.i,rownr.j]
      }
    }
    infmat[(p-k+1):p,(p-k+1):p] <- solve(Sig)
  }else{
    p <- k*(k+1)/2
    infmat <- matrix(NA,p,p)
    for (i in seq(p)){
      colnr.i <- uptri.colnr[i]
      rownr.i <- uptri.rownr[i]
      for (j in seq(p)){
        colnr.j <- uptri.colnr[j]
        rownr.j <- uptri.rownr[j]
        infmat[i,j] <- Sig[colnr.i,rownr.j]*Sig[rownr.i,colnr.j]+Sig[colnr.i,colnr.j]*Sig[rownr.i,rownr.j]
      }
    }
  }
  return(infmat)
}

##' Calculates weight-matrix and eigenvalues
##'
##' calculation based on true information matrix
##' @title Weight-matrix and eigenvalues
##' @param imat no descr
##' @param act I_uv
##' @param act1 I_u
##' @param act2 I_v
##' @return no descr
##' @author n.stadler
##' @keywords internal
ww.mat <- function(imat,act,act1,act2){
 
  bfg <- rbind(imat[act1,act,drop=FALSE],imat[act2,act,drop=FALSE])
  bf <- matrix(0,length(act1)+length(act2),length(act1)+length(act2))
  bf[1:length(act1),1:length(act1)] <- imat[act1,act1]
  bf[length(act1)+(1:(length(act2))),length(act1)+(1:(length(act2)))] <- imat[act2,act2]
  bg <- 2*imat[act,act,drop=FALSE]
  mat <- rbind(cbind(diag(1,length(act1)+length(act2)),bfg%*%solve(bg)),cbind(-t(bfg)%*%solve(bf),diag(-1,length(act))))
  eval <- Re(eigen(mat)$values)
  eval[abs(eval)<10^{-6}] <- 0
  
  return(list(ww.mat=mat,eval=eval))
}

##' Calculates eigenvalues of weight-matrix (using 1st order simplification)
##'
##' calculation based on true information matrix
##' @title Calculates eigenvalues of weight-matrix (using 1st order simplification)
##' @param imat no descr
##' @param act I_uv
##' @param act1 I_u
##' @param act2 I_v
##' @return no descr
##' @author n.stadler
##' @keywords internal
ww.mat2 <- function(imat,act,act1,act2){

  dimf <- length(act1)+length(act2)
  dimg <- length(act)

  bfg <- rbind(imat[act1,act,drop=FALSE],imat[act2,act,drop=FALSE])
  bgf <- t(bfg)
  bf <- matrix(0,length(act1)+length(act2),length(act1)+length(act2))
  bf[1:length(act1),1:length(act1)] <- imat[act1,act1]
  bf[length(act1)+(1:(length(act2))),length(act1)+(1:(length(act2)))] <- imat[act2,act2]
  bg <- 2*imat[act,act]

  if (dimf>=dimg){
    mat <- bgf%*%solve(bf)%*%bfg%*%solve(bg)
    eval<-rep(1,dimf-dimg)
  }
  if (dimf<dimg){
    mat <- bfg%*%solve(bg)%*%bgf%*%solve(bf)
    eval<-rep(-1,dimg-dimf)
  }
  eval.mu <- Re(eigen(mat)$values)
  eval2 <- 1-eval.mu
  eval2[abs(eval2)<10^{-6}] <- 0
  eval <- c(eval,sqrt(eval2),-sqrt(eval2))
  return(list(ww.mat=mat,eval=eval))
}

##' Compute beta-matrix 
##'
##' beta-matrix=E[s_ind1(Y;sig1) s_ind2(Y;sig2)'|sig]
##' @title Compute beta-matrix 
##' @param ind1 no descr
##' @param ind2 no descr
##' @param sig1 no descr
##' @param sig2 no descr
##' @param sig no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
beta.mat<-function(ind1,ind2,sig1,sig2,sig){
  uptri.rownr <- row(sig)[upper.tri(sig,diag=TRUE)]
  uptri.colnr <- col(sig)[upper.tri(sig,diag=TRUE)]
  betamat <- .C('betamat_diffnet',
  							betamat=as.numeric(matrix(0,length(ind1),length(ind2))),
                ind1=as.integer(ind1-1),ind2=as.integer(ind2-1),##8!indexing
                uptrirownr=uptri.rownr,uptricolnr=uptri.colnr,
                lind1=length(ind1),lind2=length(ind2),
                sig1=sig1,sig2=sig2,sig=sig,k=ncol(sig))$betamat
   
#   betamat.old <- .C('betamat_diffnet',betamat=matrix(0,length(ind1),length(ind2)),
#    							ind1=as.integer(ind1-1),ind2=as.integer(ind2-1),##8!indexing
#    							uptrirownr=uptri.rownr,uptricolnr=uptri.colnr,
#    							lind1=length(ind1),lind2=length(ind2),
#    							sig1=sig1,sig2=sig2,sig=sig,k=ncol(sig))$betamat
#   
  return(matrix(betamat, length(ind1), length(ind2)))
}

## beta.mat<-function(ind1,ind2,sig1,sig2,sig){
##   k <- ncol(sig)
##   p <- k*(k+1)/2
##   uptri.rownr <- row(sig)[upper.tri(sig,diag=TRUE)]
##   uptri.colnr <- col(sig)[upper.tri(sig,diag=TRUE)]

##   bmat <- matrix(NA,p,p)
##   for (i in ind1){
##     colnr.i <- uptri.colnr[i]
##     rownr.i <- uptri.rownr[i]
##     for (j in ind2){
##       colnr.j <- uptri.colnr[j]
##       rownr.j <- uptri.rownr[j]
##       bmat[i,j] <- sig[colnr.i,rownr.i]*sig[colnr.j,rownr.j]-sig1[colnr.i,rownr.i]*sig[colnr.j,rownr.j]-sig[colnr.i,rownr.i]*sig2[colnr.j,rownr.j]+sig1[colnr.i,rownr.i]*sig2[colnr.j,rownr.j]+sig[colnr.i,rownr.j]*sig[rownr.i,colnr.j]+sig[colnr.i,colnr.j]*sig[rownr.i,rownr.j]
##     }
##   }
##   return(bmat[ind1,ind2,drop=FALSE])
## }

##' Compute Q-matrix 
##'
##' 
##' @title Compute Q-matrix 
##' @param sig no descr
##' @param sig.a no descr
##' @param sig.b no descr
##' @param act.a no descr
##' @param act.b no descr
##' @param ss no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
q.matrix3 <- function(sig,sig.a,sig.b,act.a,act.b,ss){
  b.ab<-beta.mat(act.a,act.b,sig.a,sig.b,sig)
  aa<-seq(1,length(act.a))[!(act.a%in%ss)]
  bb<-seq(1,length(act.b))[!(act.b%in%ss)]
  s.a<-seq(1,length(act.a))[(act.a%in%ss)]
  s.b<-seq(1,length(act.b))[(act.b%in%ss)]
  
  if(length(ss)==0){
    return(b.ab[aa,bb,drop=FALSE])
  }else{
    return(b.ab[aa,bb,drop=FALSE]-(b.ab[aa,s.b,drop=FALSE]%*%solve(b.ab[s.a,s.b,drop=FALSE])%*%b.ab[s.a,bb,drop=FALSE]))
  }
}

##' q.matrix4
##'
##' 
##' @title q.matrix4
##' @param b.mat no descr
##' @param act.a no descr
##' @param act.b no descr
##' @param ss no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
q.matrix4 <- function(b.mat,act.a,act.b,ss){
 ##Estimate Q
    aa<-seq(1,length(act.a))[!(act.a%in%ss)]
    bb<-seq(1,length(act.b))[!(act.b%in%ss)]
    s.a<-seq(1,length(act.a))[(act.a%in%ss)]
    s.b<-seq(1,length(act.b))[(act.b%in%ss)]

    if(length(ss)==0){
        return(b.mat[aa,bb,drop=FALSE])
    }else{
        return(b.mat[aa,bb,drop=FALSE]-(b.mat[aa,s.b,drop=FALSE]%*%solve(b.mat[s.a,s.b,drop=FALSE])%*%b.mat[s.a,bb,drop=FALSE]))
    }
}

##' Compute weights of sum-w-chi2 (1st order simplification)
##'
##' 
##' @title Weights of sum-w-chi2
##' @param sig1 no descr
##' @param sig2 no descr
##' @param sig no descr
##' @param act1 no descr
##' @param act2 no descr
##' @param act no descr
##' @param include.mean no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
est2.ww.mat2 <- function(sig1,sig2,sig,act1,act2,act,include.mean=FALSE){

  k <- nrow(sig1)
  
  #######################
  ##dimension of models##
  #######################
  dimf1 <- length(act1)
  dimf2 <- length(act2)
  dimf <- dimf1+dimf2
  dimg <- length(act)
  if(include.mean==TRUE){
    dimf <- dimf+2*k
    dimg <- dimg+k
  }

  ###############
  ##Beta-matrix##
  ###############
  b1.act<- beta.mat(act,act,sig,sig,sig1)
  b2.act<- beta.mat(act,act,sig,sig,sig2)
  b1.act1<- beta.mat(act1,act1,sig1,sig1,sig1)
  b2.act2<- beta.mat(act2,act2,sig2,sig2,sig2)
  b1.act1.act<- beta.mat(act1,act,sig1,sig,sig1)
  b2.act2.act<- beta.mat(act2,act,sig2,sig,sig2)
  
  #if (any(c(dimf1,dimf2,dimg)>min(nrow(xx1),nrow(xx2)))){
  #  warning('|active-set| > n')
  #  return(list(eval=NA))
  #}else{
  bfg <- rbind(b1.act1.act,b2.act2.act)
  bgf <- t(bfg)
  bf <- matrix(0,dimf1+dimf2,dimf1+dimf2)
  bf[1:dimf1,1:dimf1] <- b1.act1
  bf[dimf1+(1:dimf2),dimf1+(1:dimf2)] <- b2.act2
  bg <- b1.act+b2.act
  if (dimf>=dimg){
    mat <- bgf%*%solve(bf)%*%bfg%*%solve(bg)
    eval<-rep(1,dimf-dimg)
  }
  if (dimf<dimg){
    mat <- bfg%*%solve(bg)%*%bgf%*%solve(bf)
    eval<-rep(-1,dimg-dimf)
  }
  eval.mu.complex<-eigen(mat)$values
  onemineval.mu <- 1-Re(eval.mu.complex)
  onemineval.mu[abs(onemineval.mu)<10^{-6}]<-0
  eval <- c(eval,sqrt(onemineval.mu),-sqrt(onemineval.mu))
  if(include.mean==TRUE){
    eval <- c(rep(0,2*k),eval)
  }
  return(list(ww.mat=mat,eval=eval,eval.mu.complex=eval.mu.complex))
}

##' Compute weights of sum-w-chi2 (2nd order simplification)
##' 
##' *expansion of W in two directions ("dimf>dimg direction" & "dimf>dimg direction") 
##' *simplified computation of weights is obtained by assuming H0 and that X_u~X_v holds
##' @title Weights of sum-w-chi2
##' @param sig1 no descr
##' @param sig2 no descr
##' @param sig no descr
##' @param act1 no descr
##' @param act2 no descr
##' @param act no descr
##' @param include.mean no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
est2.my.ev2 <- function(sig1,sig2,sig,act1,act2,act,include.mean=FALSE){

  k <- nrow(sig1)

  #######################
  ##dimension of models##
  #######################
  dimf1 <- length(act1)
  dimf2 <- length(act2)
  dimf <- dimf1+dimf2
  dimg <- length(act)
  if(include.mean==TRUE){
    dimf <- dimf+2*k
    dimg <- dimg+k
  }
  ##########################
  ##intersection of models##
  ##########################
  ss <- intersect(act,intersect(act1,act2))
  
  if(length(ss)==0){warning('no intersection between models')}
  
  aa <- setdiff(act1,ss)
  bb <- setdiff(act2,ss)
  cc <- setdiff(act,ss)
 
  ev.aux <- ev.aux.complex <- numeric(0)
  if (dimf>=dimg){
    if (length(cc)!=0){
      qcc <- q.matrix3(sig1,sig,sig,act,act,ss)+q.matrix3(sig2,sig,sig,act,act,ss)
      aux.mat <- matrix(0,length(cc),length(cc))
      if(length(aa)!=0){
        qac <- q.matrix3(sig1,sig1,sig,act1,act,ss)
        qaa <- q.matrix3(sig1,sig1,sig1,act1,act1,ss)
        aux.mat <- aux.mat+(t(qac)%*%solve(qaa)%*%qac)%*%solve(qcc)
      }
      if(length(bb)!=0){
        qbc <- q.matrix3(sig2,sig2,sig,act2,act,ss)
        qbb <- q.matrix3(sig2,sig2,sig2,act2,act2,ss)
        aux.mat <- aux.mat+(t(qbc)%*%solve(qbb)%*%qbc)%*%solve(qcc)
      }
      ev.aux.complex <- eigen(aux.mat)$values
      ev.aux <- Re(ev.aux.complex)
      ev.aux <-  sqrt(1-ev.aux)
    }
    eval<-rep(1,dimf-dimg)
    eval <- c(eval,rep(0,2*length(ss)),ev.aux,-ev.aux)
  }## end if (dimf>=dimg){
  if (dimf<dimg){
    if (length(cc)!=0){
      qcc <- q.matrix3(sig1,sig,sig,act,act,ss)+q.matrix3(sig2,sig,sig,act,act,ss)
      if(length(aa)!=0){
        qac <- q.matrix3(sig1,sig1,sig,act1,act,ss)
        qaa <- q.matrix3(sig1,sig1,sig1,act1,act1,ss)
        oaa <- qac%*%solve(qcc)%*%t(qac)
        aux.mat.aa<- oaa%*%solve(qaa)
        aux.mat <- aux.mat.aa
      }
      if(length(bb)!=0){
        qbc <- q.matrix3(sig2,sig2,sig,act2,act,ss)
        qbb <- q.matrix3(sig2,sig2,sig2,act2,act2,ss)
        obb <- qbc%*%solve(qcc)%*%t(qbc)
        aux.mat.bb<- obb%*%solve(qbb)
        aux.mat <- aux.mat.bb
      }
      if((length(aa)!=0)&(length(bb)!=0)){
        oab <- qac%*%solve(qcc)%*%t(qbc)
        aux.mat<- rbind(cbind(aux.mat.aa,oab%*%solve(qbb)),cbind(t(oab)%*%solve(qaa),aux.mat.bb))
      }
    }
    if ((length(aa)!=0)|(length(bb)!=0)){ ##if (length(aa)==0)&(length(bb)==0) 'MI included in MJ; therefore -X^2(dim(MJ)-dim(MI)) distributed'
      ev.aux.complex <- eigen(aux.mat)$values
      ev.aux <- Re(ev.aux.complex)
      ev.aux <-  sqrt(1-ev.aux)
    }
    eval<-rep(-1,dimg-dimf)
    eval <- c(eval,rep(-1,length(ss)),rep(1,length(ss)),rep(0,2*length(ss)),ev.aux,-ev.aux)
  }## end if (dimf<dimg){
  
  if(include.mean==TRUE){
    eval <- c(rep(0,2*k),eval)
  }
  return(list(eval=eval,ev.aux.complex=ev.aux.complex))
}

##' Compute weights of sum-of-weighted-chi2s
##'
##' *'2nd order simplification':
##'   1) Factor out (1-vi)^(d1+d2) "expansion in dimf>dimg direction (old terminology)"
##'   2) Factor out (1-mu)^d0 
##' *simplified computation of weights is obtained without further invoking H0, or assuming X_u~X_v
##' 
##' @title Compute weights of sum-of-weighted-chi2s
##' @param sig1 MLE (covariance matrix) sample 1
##' @param sig2 MLE (covariance matrix) sample 2
##' @param sig Pooled MLE (covariance matrix)
##' @param act1 Active-set sample 1
##' @param act2 Active-set sample 2
##' @param act Pooled active-set
##' @param include.mean Should the mean be in cluded in the likelihood?
##' @return Eigenvalues of M, respectively the weights.
##' @author n.stadler
##' @keywords internal
est2.my.ev3 <- function(sig1,sig2,sig,act1,act2,act,include.mean=FALSE){

  show.warn <- FALSE
  k <- nrow(sig1)

  #######################
  ##dimension of models##
  #######################
  dimf1 <- length(act1)
  dimf2 <- length(act2)
  dimf <- dimf1+dimf2
  dimg <- length(act)
  if(include.mean==TRUE){
    dimf <- dimf+2*k
    dimg <- dimg+k
  }
  ##########################
  ##intersection of models##
  ##########################
  ss <- intersect(act,intersect(act1,act2))
  
  if((length(ss)==0)&show.warn){warning('no intersection between models')}
  
  aa <- setdiff(act1,ss)
  bb <- setdiff(act2,ss)
  cc <- setdiff(act,ss)
 
  ev.aux <- ev.aux.complex <- numeric(0)
  no.zero.ev.aux <- 0
  #if (dimf>=dimg){
    if (length(cc)!=0){
      qcc1 <- q.matrix3(sig1,sig,sig,act,act,ss)
      qcc2 <- q.matrix3(sig2,sig,sig,act,act,ss)
      bmat <- beta.mat(act,act,sig,sig,sig1)+beta.mat(act,act,sig,sig,sig2)
      qcc12 <- q.matrix4(bmat,act,act,ss)
      aux.mat <- diag(1,length(cc))-qcc1%*%solve(qcc12)-qcc2%*%solve(qcc12)
      if(length(aa)!=0){
        qac <- q.matrix3(sig1,sig1,sig,act1,act,ss)
        qaa <- q.matrix3(sig1,sig1,sig1,act1,act1,ss)
        aux.mat <- aux.mat+(t(qac)%*%solve(qaa)%*%qac)%*%solve(qcc12)
      }
      if(length(bb)!=0){
        qbc <- q.matrix3(sig2,sig2,sig,act2,act,ss)
        qbb <- q.matrix3(sig2,sig2,sig2,act2,act2,ss)
        aux.mat <- aux.mat+(t(qbc)%*%solve(qbb)%*%qbc)%*%solve(qcc12)
      }
      ev.aux.complex <- eigen(aux.mat)$values
      ev.aux <- Re(ev.aux.complex)
      zero.ev.aux <- abs(ev.aux)<10^{-10}
      no.zero.ev.aux <- sum(zero.ev.aux)
      ev.aux <- ev.aux[!zero.ev.aux]
      ev.aux <-  sqrt(1-ev.aux)
    }
  eval<-c(rep(1,dimf-dimg+no.zero.ev.aux),rep(-1,no.zero.ev.aux))
  eval <- c(eval,rep(0,2*length(ss)),ev.aux,-ev.aux)
  #}## end if (dimf>=dimg){
  
  if(include.mean==TRUE){
    eval <- c(rep(0,2*k),eval)
  }
  return(list(eval=eval,ev.aux.complex=ev.aux.complex))
}

##' P-value aggregation
##'
##' 
##' @title P-value aggregation (Meinshausen et al 2009)
##' @param gamma see Meinshausen et al 2009
##' @param pval vector of p-values
##' @return inf-quantile aggregated p-value
##' @author n.stadler
##' @keywords internal
agg.pval <- function(gamma,pval){
    min(quantile(pval/gamma,probs=gamma),1)
}

##' P-value calculation 
##'
##' 
##' @title P-value calculation
##' @param x1 no descr
##' @param x2 no descr
##' @param x no descr
##' @param sig1 no descr
##' @param sig2 no descr
##' @param sig no descr
##' @param mu1 no descr
##' @param mu2 no descr
##' @param mu no descr
##' @param act1 no descr
##' @param act2 no descr
##' @param act no descr
##' @param compute.evals no descr
##' @param include.mean no descr
##' @param method.compquadform no descr
##' @param acc no descr
##' @param epsabs no descr
##' @param epsrel no descr
##' @param show.warn no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
diffnet_pval <- function(x1,x2,x,sig1,sig2,sig,mu1,mu2,mu,act1,act2,act,compute.evals,include.mean,method.compquadform,acc,epsabs,epsrel,show.warn){
  ##########################
  ##compute test-statistic##
  ##########################
  teststat <- logratio(x1,x2,x,sig1,sig2,sig,mu1,mu2,mu)$twiceLR
  #################################
  ##compute weights of sum-w-chi2##
  #################################
  weights.nulldistr <- eval(as.name(compute.evals))(sig1,sig2,sig,act1,act2,act,include.mean)$eval
  weights.nulldistr <- weights.nulldistr[weights.nulldistr!=0]
  if (any(is.na(weights.nulldistr))){
    cat('warning: weight with value NA; pval=NA','\n')
    pval.onesided <- pval.twosided <- NA
  }else{
    if(method.compquadform=='davies'){
      pval.onesided <- davies(teststat,lambda=weights.nulldistr,acc=acc)$Qq;if(show.warn){cat('ifault(davies):',davies(teststat,lambda=weights.nulldistr,acc=acc)$ifault,'\n')}
    }
    if(method.compquadform=='imhof'){
      pval.onesided <-imhof(teststat, lambda=weights.nulldistr,epsabs = epsabs, epsrel = epsrel, limit = 10000)$Qq
      if(pval.onesided<0){pval.onesided <- 0}
      if(pval.onesided>1){pval.onesided <- 1}
    }
    pval.twosided <- 2*min(pval.onesided,1-pval.onesided)
  }
  return(list(pval.onesided=pval.onesided,pval.twosided=pval.twosided,weights.nulldistr=weights.nulldistr,teststat=teststat))
}

##' Differential Network for user specified data splits
##'
##' Remark:
##' 
##' * If include.mean=FALSE, then x1 and x2 have mean zero and DiffNet tests
##'   the hypothesis H0: Omega_1=Omega_2. You might need to center x1 and x2.
##' * If include.mean=TRUE, then DiffNet tests the hypothesis
##'   H0: mu_1=mu_2 & Omega_1=Omega_2
##' * However, we recommend to set include.mean=FALSE and to test equality of the means
##'   separately. 
##' * You might also want to scale x1 and x2, if you are only interested in 
##'   differences due to (partial) correlations.
##'   
##' 
##' @title Differential Network for user specified data splits
##' @param x1 Data-matrix sample 1.
##'           You might need to center and scale your data-matrix.
##' @param x2 Data-matrix sample 2.
##'           You might need to center and scale your data-matrix.
##' @param split1 Samples (condition 1) used in screening step. 
##' @param split2 Samples (condition 2) used in screening step. 
##' @param screen.meth Screening procedure. Options: 'screen_bic.glasso' (default),
##'                    'screen_cv.glasso', 'screen_shrink' (not recommended).
##' @param compute.evals Method to estimate the weights in the weighted-sum-of-chi2s distribution.
##'                      The default and (currently) the only available option 
##'                      is the method 'est2.my.ev3'.
##' @param algorithm.mleggm Algorithm to compute MLE of GGM. The algorithm 'glasso_rho' is the
##'                         default and (currently) the only available option.
##' @param include.mean Should sample specific means be included in hypothesis?
##'                     Use include.mean=FALSE (default and recommended) which assumes mu1=mu2=0
##'                     and tests the hypothesis H0: Omega_1=Omega_2.
##' @param method.compquadform Method to compute distribution function of weighted-sum-of-chi2s
##'                            (default='imhof').
##' @param acc See ?davies (default 1e-04).
##' @param epsabs See ?imhof (default 1e-10).
##' @param epsrel See ?imhof (default 1e-10).
##' @param show.warn Should warnings be showed (default=FALSE)?
##' @param save.mle Should MLEs be in the output list (default=FALSE)?
##' @param ... Additional arguments for screen.meth.
##' @return list consisting of
##' \item{pval.onesided}{p-value}
##' \item{pval.twosided}{ignore this output}
##' \item{teststat}{log-likelihood-ratio test statistic}
##' \item{weights.nulldistr}{estimated weights}
##' \item{active}{active-sets obtained in screening-step}
##' \item{sig}{constrained mle (covariance) obtained in cleaning-step}
##' \item{wi}{constrained mle (inverse covariance) obtained in cleaning-step}
##' \item{mu}{mle (mean) obtained in cleaning-step}
##' @author n.stadler
##' @export
##' @example tests/Examples/diffnet_ss_ex.R
diffnet_singlesplit<- function(x1,x2,split1,split2,screen.meth='screen_bic.glasso',
                               compute.evals='est2.my.ev3',algorithm.mleggm='glasso_rho0',include.mean=FALSE,
                               method.compquadform='imhof',acc=1e-04,epsabs=1e-10,epsrel=1e-10,
                               show.warn=FALSE,save.mle=FALSE,...){
  
  n1 <- nrow(x1)
  n2 <- nrow(x2)
  k <- ncol(x1)

  df.param <- k*(k+1)/2
  
  est.mu <- est.sig <- est.wi <- active <- list()#save est.mu;est.sig;est.wi;active
  
  ###############
  ##Joint Model##
  ###############
  xx.train <- rbind(x1[split1,],x2[split2,])
  xx.valid <- rbind(x1[-split1,],x2[-split2,])
  fit.screen <- eval(as.name(screen.meth))(xx.train,include.mean=include.mean,...)
  act <- which(fit.screen$wi[upper.tri(diag(1,k),diag=TRUE)]!=0)
  active[['modJ']] <- act
  if(include.mean==TRUE){
    est.mu[['modJ']] <- colMeans(xx.valid)
  }else{
    est.mu[['modJ']] <- rep(0,k)
  }
  if (length(active[['modJ']])==df.param){
    w <- mcov(x=xx.valid,include.mean=include.mean)
    est.sig[['modJ']] <- w
    est.wi[['modJ']] <- solve(w)
  }
  if (length(active[['modJ']])!=df.param){
    fit.mle <- mle.ggm(xx.valid,fit.screen$wi,algorithm=algorithm.mleggm,rho=fit.screen$rho.opt,include.mean=include.mean)
    w <- fit.mle$w
    est.sig[['modJ']] <- w
    est.wi[['modJ']] <- fit.mle$wi
  }
  #####################  
  ##Individual Models##
  #####################
  for (j in c('1','2')){
    split.train <- eval(as.name(paste('split',j,sep='')))
    xx.train <- eval(as.name(paste('x',j,sep='')))[split.train,]
    xx.valid <- eval(as.name(paste('x',j,sep='')))[-split.train,]
    fit.screen <- eval(as.name(screen.meth))(xx.train,include.mean=include.mean,...)
    act <- which(fit.screen$wi[upper.tri(diag(1,k),diag=TRUE)]!=0)
    active[[paste('modIpop',j,sep='')]] <- act
    if(include.mean==TRUE){
      est.mu[[paste('modIpop',j,sep='')]] <- colMeans(xx.valid)
    }else{
      est.mu[[paste('modIpop',j,sep='')]] <- rep(0,k)
    }
    if (length(active[[paste('modIpop',j,sep='')]])==df.param){
      w <- mcov(xx.valid,include.mean=include.mean)
      est.sig[[paste('modIpop',j,sep='')]] <- w
      est.wi[[paste('modIpop',j,sep='')]] <- solve(w)
    }
    if (length(active[[paste('modIpop',j,sep='')]])!=df.param){
      fit.mle <- mle.ggm(xx.valid,fit.screen$wi,algorithm=algorithm.mleggm,rho=fit.screen$rho.opt,include.mean=include.mean)
      w <- fit.mle$w
      est.sig[[paste('modIpop',j,sep='')]] <- w
      est.wi[[paste('modIpop',j,sep='')]] <- fit.mle$wi
    }
  }

  ###############
  ##Some Checks##
  ###############
  l.act <- lapply(active,length)
  n1.valid <- nrow(x1[-split1,])
  n2.valid <- nrow(x2[-split2,])
  if (any(l.act==0)){ if(show.warn){cat('warning:at least one active-set is empty','\n')}}
  if (all(l.act>= c(n1.valid+n2.valid,n1.valid,n2.valid))){if(show.warn){cat('warning:dim(model) > n-1','\n')}}

  ###########
  ##Pvalue ##
  ###########
  res.pval <- diffnet_pval(x1=x1[-split1,,drop=FALSE],x2=x2[-split2,,drop=FALSE],x=rbind(x1[-split1,,drop=FALSE],x2[-split2,,drop=FALSE]),
                           sig1=est.sig[['modIpop1']],sig2=est.sig[['modIpop2']],sig=est.sig[['modJ']],
                           mu1=est.mu[['modIpop1']],mu2=est.mu[['modIpop2']],mu=est.mu[['modJ']],
                           active[['modIpop1']],active[['modIpop2']],active[['modJ']],
                           compute.evals,include.mean,method.compquadform,acc,epsabs,epsrel,show.warn)
  
  if(save.mle==FALSE){
    est.sig <- est.wi <- est.mu <- NULL
  }
  return(list(pval.onesided=res.pval$pval.onesided,pval.twosided=res.pval$pval.twosided,
              teststat=res.pval$teststat,weights.nulldistr=res.pval$weights.nulldistr,
              active=active,sig=est.sig,wi=est.wi,mu=est.mu))
}

##' Differential Network 
##'
##' Remark:
##' 
##' * If include.mean=FALSE, then x1 and x2 have mean zero and DiffNet tests
##'   the hypothesis H0: Omega_1=Omega_2. You might need to center x1 and x2.
##' * If include.mean=TRUE, then DiffNet tests the hypothesis
##'   H0: mu_1=mu_2 & Omega_1=Omega_2
##' * However, we recommend to set include.mean=FALSE and to test equality of the means
##'   separately. 
##' * You might also want to scale x1 and x2, if you are only interested in 
##'   differences due to (partial) correlations.
##' 
##' 
##' @title Differential Network 
##' @param x1 Data-matrix sample 1.
##'           You might need to center and scale your data-matrix.
##' @param x2 Data-matrix sample 1.
##'           You might need to center and scale your data-matrix.
##' @param b.splits Number of splits (default=50).
##' @param frac.split Fraction train-data (screening) / test-data (cleaning) (default=0.5).
##' @param screen.meth Screening procedure. Options: 'screen_bic.glasso' (default),
##'                    'screen_cv.glasso', 'screen_shrink' (not recommended).
##' @param include.mean Should sample specific means be included in hypothesis?
##'                     Use include.mean=FALSE (default and recommended) which assumes mu1=mu2=0
##'                     and tests the hypothesis H0: Omega_1=Omega_2.
##' @param gamma.min Tuning parameter in p-value aggregation of Meinshausen et al (2009). (Default=0.05).
##' @param compute.evals Method to estimate the weights in the weighted-sum-of-chi2s distribution.
##'                      The default and (currently) the only available option 
##'                      is the method 'est2.my.ev3'.
##' @param algorithm.mleggm Algorithm to compute MLE of GGM. The algorithm 'glasso_rho' is the
##'                         default and (currently) the only available option.
##' @param method.compquadform Method to compute distribution function of weighted-sum-of-chi2s
##'                            (default='imhof').
##' @param acc See ?davies (default 1e-04).
##' @param epsabs See ?imhof (default 1e-10).
##' @param epsrel See ?imhof (default 1e-10).
##' @param show.warn Should warnings be showed (default=FALSE)?
##' @param save.mle If TRUE, MLEs (inverse covariance matrices for samples 1 and 2)
##'                 are saved for all b.splits. The median aggregated inverse covariance matrix
##'                 is provided in the output as 'medwi'. The default is save.mle=FALSE.
##' @param verbose If TRUE, show output progress.
##' @param mc.flag If \code{TRUE} use parallel execution for each b.splits via function 
##'                \code{mclapply} of package \code{parallel}.
##' @param mc.set.seed See mclapply. Default=TRUE
##' @param mc.preschedule See mclapply. Default=TRUE
##' @param mc.cores Number of cores to use in parallel execution. Defaults to
##'                 mc.cores option if set, or 2 otherwise.
##' @param ... Additional arguments for screen.meth.
##' @return list consisting of
##' \item{ms.pval}{p-values for all b.splits}
##' \item{ss.pval}{single-split p-value}
##' \item{medagg.pval}{median aggregated p-value}
##' \item{meinshagg.pval}{meinshausen aggregated p-value (meinshausen et al 2009)}
##' \item{teststat}{test statistics for b.splits}
##' \item{weights.nulldistr}{estimated weights}
##' \item{active.last}{active-sets obtained in last screening-step}
##' \item{medwi}{median of inverse covariance matrices over b.splits}
##' \item{sig.last}{constrained mle (covariance matrix) obtained in last cleaning-step}
##' \item{wi.last}{constrained mle (inverse covariance matrix) obtained in last cleaning-step}
##' @author n.stadler
##' @export
##' @example tests/Examples/diffnet_ex.R
diffnet_multisplit<- function(x1,x2,b.splits=50,frac.split=1/2,screen.meth='screen_bic.glasso',include.mean=FALSE,
                              gamma.min=0.05,compute.evals='est2.my.ev3',algorithm.mleggm='glasso_rho0',
                              method.compquadform='imhof',acc=1e-04,epsabs=1e-10,epsrel=1e-10,
                              show.warn=FALSE,save.mle=FALSE,verbose=TRUE,
                              mc.flag=FALSE,mc.set.seed=TRUE, mc.preschedule = TRUE,mc.cores=getOption("mc.cores", 2L),...){

  ##????Important Notes: Pval can be NA, because...
  ##????
  ##????                 1) Eval=sqrt(neg. value)=NA
  ##????                 2) W cannot be estimated (|active-set| > n, problems matrix inversion) 
  n1 <- nrow(x1)
  n2 <- nrow(x2)

  if(mc.flag && .Platform$OS.type == "unix"){
      res.multisplit <- mclapply(seq(b.splits),
                                 FUN=function(i){
                                     if(verbose){cat(' split: ',i,'\n\n')}
                                     split1 <- sample(1:n1,round(n1*frac.split),replace=FALSE)
                                     split2 <- sample(1:n2,round(n2*frac.split),replace=FALSE)
                                     res.singlesplit <- diffnet_singlesplit(x1,x2,split1,split2,screen.meth,
                                                                            compute.evals,algorithm.mleggm,include.mean,
                                                                            method.compquadform,acc,epsabs,epsrel,show.warn,save.mle,...)
                                 },mc.set.seed=mc.set.seed, mc.preschedule = mc.preschedule, mc.cores=mc.cores)
                                 
  }else{
  	  if(mc.flag) warning('Windows does not support parallelisation via mclapply, using sequential lapply instead.')
      res.multisplit <- lapply(seq(b.splits),
                               function(i){
                                   if(verbose){cat(' split: ',i,'\n\n')}
                                   split1 <- sample(1:n1,round(n1*frac.split),replace=FALSE)
                                   split2 <- sample(1:n2,round(n2*frac.split),replace=FALSE)
                                   res.singlesplit <- diffnet_singlesplit(x1,x2,split1,split2,screen.meth,
                                                                          compute.evals,algorithm.mleggm,include.mean,
                                                                          method.compquadform,acc,epsabs,epsrel,show.warn,save.mle,...)
                                   
                               })
  }

  pval.onesided <- sapply(res.multisplit,function(x){x[['pval.onesided']]},simplify='array')
  pval.twosided <- sapply(res.multisplit,function(x){x[['pval.twosided']]},simplify='array')
  teststat <- sapply(res.multisplit,function(x){x[['teststat']]},simplify='array')
  weights.nulldistr <- sapply(res.multisplit,function(x){x[['weights.nulldistr']]},simplify='array')
  aggpval.onesided <- min(1,(1-log(gamma.min))*optimize(f=agg.pval,interval=c(gamma.min,1),maximum=FALSE,pval=pval.onesided[!is.na(pval.onesided)])$objective)
  aggpval.twosided <- min(1,(1-log(gamma.min))*optimize(f=agg.pval,interval=c(gamma.min,1),maximum=FALSE,pval=pval.twosided[!is.na(pval.twosided)])$objective)
  k <- ncol(x1)
  if(save.mle==TRUE){
    medwi <- list(modJ=matrix(apply(sapply(res.multisplit,function(x){x[['wi']][['modJ']]}),1,median),k,k),
                  modIpop1=matrix(apply(sapply(res.multisplit,function(x){x[['wi']][['modIpop1']]}),1,median),k,k),
                  modIpop2=matrix(apply(sapply(res.multisplit,function(x){x[['wi']][['modIpop2']]}),1,median),k,k))
  }else{
    medwi <- NULL
  }
  
 result <- list(ms.pval=pval.onesided,
                ss.pval=pval.onesided[1],
                medagg.pval=median(pval.onesided,na.rm=TRUE),
                meinshagg.pval=aggpval.onesided,
                teststat=teststat,weights.nulldistr=weights.nulldistr,
                active.last=res.multisplit[[b.splits]]$active,
                medwi=medwi,
                sig.last=res.multisplit[[b.splits]]$sig,wi.last=res.multisplit[[b.splits]]$wi)
  class(result) <- 'diffnet'
  return(result)
}

library(network)

##' Convert inverse covariance to partial correlation
##' 
##'  
##' @param invcov Inverse covariance matrix
##' @export
##' @return The partial correlation matrix.
##' @examples
##' inv.cov = generate_inv_cov(p=25)
##' p.corr = invcov2parcor(inv.cov)
invcov2parcor <- function(invcov){
	return(-invcov*tcrossprod(sqrt(1/diag(invcov))))
}

##' Convert inverse covariance to partial correlation for several inverse
##' covariance matrices collected in an array.
##' 
##' @param invcov.array Array of inverse covariance matrices, of dimension
##' numNodes by numNodes by numComps.
##' @export
##' @return Array of partial correlation matrices of dimension numNodes by 
##' numNodes by numComps
##' @examples
##' invcov.array = sapply(1:5, function(x) generate_inv_cov(p=25), simplify='array')
##' p.corr = invcov2parcor_array(invcov.array)
invcov2parcor_array <- function(invcov.array) {
	return(sapply(1:dim(invcov.array)[3], 
								function(i) invcov2parcor(invcov.array[,,i]), 
						    simplify='array'))
}

##' Export networks as a CSV table.
##'
##' This function takes the output of \code{\link{het_cv_glasso}} or 
##' \code{\link{mixglasso}} and exports it as a text table in CSV format, where each
##' entry in the table records an edge in one group and its partial correlation.
##' 
##' @param net.clustering A network clustering object as returned by 
##' \code{\link{screen_cv.glasso}} or \code{\link{mixglasso}}.
##' @param file Filename to save the network table under.
##' @param node.names Names for the nodes in the network. If NULL, names from 
##' net.clustering will be used.
##' @param group.names Names for the clusters or groups. If NULL, names from 
##' net.clustering will be used (by default these are integets 1:numClusters).
##' @param p.corrs.thresh Threshold applied to the absolute partial correlations. 
##' Edges that are below the threshold in all of the groups are not exported. 
##' Using a negative value will export all possible edges (including those 
##' with zero partial correlation).
##' @param ... Further parameters passed to \link{write.csv}.
##' @return Function does not return anything.
##' @author Frank Dondelinger
##' @export
##' @examples
##' n = 500
##' p = 10
##' s = 0.9
##' n.comp = 3
##'
##' # Create different mean vectors
##' Mu = matrix(0,p,n.comp)
##'
##' # Define non-zero means in each group (non-overlapping)
##' nonzero.mean = split(sample(1:p),rep(1:n.comp,length=p))
##'
##' # Set non-zero means to fixed value
##' for(k in 1:n.comp){
##' 	Mu[nonzero.mean[[k]],k] = -2/sqrt(ceiling(p/n.comp))
##' }
##'
##' # Generate data
##' sim.result = sim_mix_networks(n, p, n.comp, s, Mu=Mu)
##' mixglasso.result = mixglasso(sim.result$data, n.comp=3)
##' mixglasso.clustering = mixglasso.result$models[[mixglasso.result$bic.opt]]
##' 
##' \dontrun{
##' # Save network in CSV format suitable for Cytoscape import
##' export_network(mixglasso.clustering, file='nethet_network.csv',
##'							 p.corrs.thresh=0.25, quote=FALSE)
##' }
export_network <- function(net.clustering, file='network_table.csv',
													 node.names=rownames(net.clustering$Mu),
													 group.names=sort(unique(net.clustering$comp)),
													 p.corrs.thresh=0.2, ...) {	
	
	if(is.null(node.names)) node.names = 1:dim(net.clustering$Mu)[1]
	
	# Calculate partial correlations
	p.corrs = invcov2parcor_array(net.clustering$SigInv)
	
	edge.table = data.frame()
	
	for(group.i in 1:length(group.names)) {
		group.name = group.names[group.i]
		
		p.corrs.group = p.corrs[,,group.i]
		
		diag(p.corrs.group) = NA # Discard diagonal
		p.corrs.group[lower.tri(p.corrs.group)] = NA # Ignore lower triangular matrix
		
		edges = which(abs(p.corrs.group) > p.corrs.thresh, 
									arr.ind=TRUE, useNames=TRUE)
		
		edge.table.temp = data.frame(Node.1=node.names[edges[,1]], 
																 Node2=node.names[edges[,2]],
																 P.Corr=c(abs(p.corrs.group[edges])), 
																 Edge.Sign=c(sign(p.corrs.group[edges])),
																 Group=rep(group.name, dim(edges)[1]))
		
		edge.table = rbind(edge.table, edge.table.temp)
	}
	
	write.csv(edge.table, file=file, ...)
}

##' Summary function for object of class 'diffnet'
##'
##' 
##' @title Summary function for object of class 'diffnet'
##' @param object object of class 'diffnet'
##' @param ... Other arguments.
##' @return aggregated p-values
##' @author nicolas
##' @export
##' @method summary diffnet
summary.diffnet <- function(object, ...){
	out <- data.frame(medagg.pval=object$medagg.pval,meinshagg.pval=object$meinshagg.pval)
	rownames(out) <- 'aggregated p-values'
	return(out)
}

##' Summary function for object of class 'diffregr'
##'
##' 
##' @title Summary function for object of class 'diffregr'
##' @param object object of class 'diffregr
##' @param ... Other arguments
##' @return aggregated p-values
##' @author nicolas
##' @export
##' @method summary diffregr
summary.diffregr <- function(object, ...){
	out <- data.frame(medagg.pval=object$medagg.pval,meinshagg.pval=object$meinshagg.pval)
	rownames(out) <- 'aggregated p-values'
	return(out)
}

##' Summary function for object of class 'ggmgsa'
##'
##' 
##' @title Summary function for object of class 'ggmgsa'
##' @param object object of class 'ggmgsa'
##' @param ... Other arguments
##' @return aggregated p-values
##' @author nicolas
##' @export
##' @method summary ggmgsa
summary.ggmgsa <- function(object, ...){
	out <- data.frame(medagg.pval=object$medagg.pval,meinshagg.pval=object$meinshagg.pval)
	rownames(out) <- object$gs.names
	return(out)
}

##' Summary function for object of class 'nethetclustering'
##'
##' 
##' @title Summary function for object of class 'nethetclustering'
##' @param object object of class 'nethetclustering'
##' @param ... Other arguments
##' @return Network statistics (a 'nethetsummary' object)
##' @author frankd
##' @export
##' @method summary nethetclustering
summary.nethetclustering <- function(object, ...) {
	
	out = list(mix.prob=object$mix.prob, p=dim(object$Mu)[1], n.comp=dim(object$Mu)[2],
			 loglik=object$loglik, bic=object$bic, mmdl=object$mmdl, compsize=object$compsize)
	
	class(out) = 'nethetsummary'
	return(out)
}

##' Print function for object of class 'nethetsummary'
##'
##' 
##' @title Print function for object of class 'nethetsummmary'
##' @param x object of class 'nethetsummary'
##' @param ... Other arguments
##' @return Function does not return anything.
##' @author frankd
##' @export
##' @method print nethetsummary
##' 
print.nethetsummary <- function(x, ...) {
	
	cat('Heterogeneous network mixture\n',
			'Number of nodes:', x$p, '\n',
			'Number of components:', x$n.comp, '\n',
			'Log Likelihood:', x$loglik, '\n',
			'BIC Score:', x$bic, 'MMDL Score:', x$mmdl, '\n',
		  'Mixture.probabilities:\n', x$mix.prob, '\n',
			'Component size:\n', x$compsize, '\n')
	
}

#' NetHet-package
#'
#' A bioconductor package for high-dimensional exploration of biological network heterogeneity 
#' 
#'
#' Includes:
#'   *Network-based clustering (MixGLasso)
#'   *Differential network (DiffNet)
#'   *Differential regression (DiffRegr)
#'   *Gene-set analysis based on graphical models (GGMGSA)
#'   *Plotting functions for exploring network heterogeneity
#'
#' @references St{\"a}dler, N. and Mukherjee, S. (2013). Two-Sample Testing in High-Dimensional Models.
#' Preprint \url{http://arxiv.org/abs/1210.4584}.
#' @import glasso mvtnorm GeneNet huge CompQuadForm ggm mclust parallel GSA limma multtest ICSNP glmnet 
#' @docType package
#' @name NetHet-package
#' @useDynLib nethet
NULL

## ----style-knitr, eval=TRUE, echo=FALSE, results="asis"--------------------
BiocStyle::latex()

## ----echo=FALSE, include=FALSE, warning=FALSE------------------------------
# Load package
library(nethet)

set.seed(1)

## ----echo=TRUE, include=TRUE-----------------------------------------------
# Specify number of simulated samples and dimensionality
n = 100
p = 25

# Specify number of components of the mixture model and mixture probabilities
n.comp = 4

mix.prob = c(0.1, 0.4, 0.3, 0.2)

# Specify sparsity in [0,1], indicating fraction of off-diagonal zero entries.
s = 0.9


# Generate networks with random means and covariances. Means will be drawn from 
# a standard Gaussian distribution, non-zero covariance values from a 
# Beta(1,1) distribution.
sim.result = sim_mix_networks(n, p, n.comp, s, mix.prob)

## ----echo=TRUE, include=TRUE, fig.width=5, fig.height=3, fig.align='center'----
print(table(sim.result$comp)/n)

component = as.factor(sim.result$comp)

library('ggplot2')
qplot(x=sim.result$data[,1], y=sim.result$data[,2], 
      colour=component) + 
  xlab('Dimension 1') +
  ylab('Dimension 2')
  

## ----echo=TRUE, include=TRUE, fig.width=5, fig.height=3, fig.align='center'----
# Generate new dataset with the same covariances, but different means
sim.result.new = sim_mix_networks(n, p, n.comp, s, mix.prob, Sig=sim.result$Sig)

component = as.factor(sim.result.new$comp)

qplot(x=sim.result.new$data[,1], y=sim.result.new$data[,2], 
      colour=component) + 
  xlab('Dimension 1') +
  ylab('Dimension 2')

## ----echo=TRUE, include=TRUE, fig.width=7, fig.height=3, fig.align='center'----
## Sample size and number of nodes
n <- 40
p <- 10

## Specify sparse inverse covariance matrices,
## with number of edges in common equal to ~ 0.8*p
gen.net <- generate_2networks(p,graph='random',n.nz=rep(p,2),
                              n.nz.common=ceiling(p*0.8))

invcov1 <- gen.net[[1]]
invcov2 <- gen.net[[2]]

plot_2networks(invcov1,invcov2,label.pos=0,label.cex=0.7)

## ----echo=TRUE, include=TRUE-----------------------------------------------
set.seed(10)

n = 100
p = 25

# Generate networks with random means and covariances. 
sim.result = sim_mix_networks(n, p, n.comp, s, mix.prob)

test.data = sim.result$data
test.labels = sim.result$comp

# Reconstruct networks for each component
networks = het_cv_glasso(data=test.data, grouping=test.labels)

## ----echo=TRUE, include=TRUE, fig.width=5, fig.height=3, fig.align='center'----
# Component labels for covariance values
components = as.factor(rep(1:n.comp, each=p^2))

qplot(x=c(networks$Sig), y=c(sim.result$Sig),
			colour=components) + 
  xlab('Reconstructed Covariances') +
  ylab('True Covariances')
  

## ----eval=TRUE, echo=TRUE, include=TRUE, warning=TRUE, results="asis"------
## Set seed
set.seed(1)
 
## Sample size and number of nodes
p <- 30
 
## Specify sparse inverse covariance matrices
gen.net <- generate_2networks(p,graph='random',n.nz=rep(p,2),                               
                             n.nz.common=ceiling(p*0.8))
invcov1 <- gen.net[[1]]
invcov2 <- gen.net[[2]] 

## Get corresponding correlation matrices
cor1 <- cov2cor(solve(invcov1))
cor2 <- cov2cor(solve(invcov2))

## ----echo=TRUE, include=TRUE, warning=TRUE, results="asis"-----------------
## Generate data under null hypothesis
library(mvtnorm) # To generate multivariate Gaussian random samples

## Sample size
n <- 70

x1 <- rmvnorm(n,mean = rep(0,dim(cor1)[1]), sigma = cor1)
x2 <- rmvnorm(n,mean = rep(0,dim(cor1)[1]), sigma = cor1)

## ----echo=TRUE, include=TRUE, warning=TRUE, results="asis"-----------------
## Run diffnet (under null hypothesis)
dn.null <- diffnet_multisplit(x1,x2,b.splits=1,verbose=FALSE)

## ----echo=TRUE, include=TRUE, warning=TRUE, results="asis"-----------------

## Generate data under alternative hypothesis (datasets have different networks)
x1 <- rmvnorm(n,mean = rep(0,dim(cor1)[1]), sigma = cor1)
x2 <- rmvnorm(n,mean = rep(0,dim(cor1)[2]), sigma = cor2)

## Run diffnet (under alternative)
dn.altn <- diffnet_multisplit(x1,x2,b.splits=1,verbose=FALSE)

## ----eval=TRUE, echo=TRUE, include=TRUE, warning=TRUE, fig.width=3.5, fig.height=3.5, fig.align='center'----
## Typically we would choose a larger number of splits
# Use parallel library (only available under Unix) for computational efficiency
if(.Platform$OS.type == "unix") {
	dn.altn <- diffnet_multisplit(x1,x2,b.splits=50,verbose=FALSE,mc.flag=TRUE)
} else {
  dn.altn <- diffnet_multisplit(x1,x2,b.splits=25,verbose=FALSE,mc.flag=FALSE)
}

par(cex=0.7)
plot(dn.altn, cex=0.5) # histogram over 50 p-values
cat('p-value:',dn.altn$medagg.pval,'\n') # median aggregated p-value

## ----echo=TRUE, include=TRUE, warning=TRUE, results="asis", fig.width=5, fig.height=2.5, fig.align='center'----
## Generate new networks
set.seed(1)
p <- 9 # network with p nodes
n <- 40
hub.net <- generate_2networks(p,graph='hub',n.hub=3,n.hub.diff=1)#generate hub networks
invcov1 <- hub.net[[1]]
invcov2 <- hub.net[[2]]
plot_2networks(invcov1,invcov2,label.pos=0,label.cex=0.7,
               main=c('network 1', 'network 2'),cex.main=0.7)

## Generate data
library('mvtnorm')
x1 <- rmvnorm(n,mean = rep(0,p), sigma = cov2cor(solve(invcov1)))
x2 <- rmvnorm(n,mean = rep(0,p), sigma = cov2cor(solve(invcov2)))

## ----eval=TRUE, echo=TRUE, include=TRUE, warning=TRUE, results="asis"------
## Identify groups with 'gene-sets'
gene.names <- paste('G',1:p,sep='')
gsets <- split(gene.names,rep(1:3,each=3))

## ----eval=TRUE, echo=TRUE, include=TRUE, warning=TRUE, results="asis"------
## Run GGM-GSA
fit.ggmgsa <- ggmgsa_multisplit(x1,x2,b.splits=1,gsets,gene.names,verbose=FALSE)

library(xtable)
print(xtable(summary(fit.ggmgsa),digits=6))

## ----echo=TRUE, include=TRUE, warning=TRUE, results="asis"-----------------
## Number of predictors and sample size
p <- 100
n <- 80

## Predictor matrices
x1 <- matrix(rnorm(n*p),n,p)
x2 <- matrix(rnorm(n*p),n,p)

## Active-sets and regression coefficients
act1 <- sample(1:p,5)
act2 <- c(act1[1:3],sample(setdiff(1:p,act1),2))
beta1 <- beta2 <- rep(0,p)
beta1[act1] <- 0.7
beta2[act2] <- 0.7

## ----echo=TRUE, include=TRUE, warning=TRUE, results="asis", fig.align='center', fig.width=3.5,fig.height=3.5----
## Response vectors under null-hypothesis
y1 <- x1%*%as.matrix(beta1)+rnorm(n,sd=1)
y2 <- x2%*%as.matrix(beta1)+rnorm(n,sd=1)

## Differential regression; b.splits=10
fit.null <- diffregr_multisplit(y1,y2,x1,x2,b.splits=10)
par(cex=0.7)
plot(fit.null,cex=0.5) # histogram of p-values from b.split data splits
cat('p-value: ',fit.null$medagg.pval,'\n') # median aggregated p-value

## ----echo=TRUE, include=TRUE, warning=TRUE, results="asis",fig.align='center',fig.width=3.5,fig.height=3.5----
## Response vectors under alternative-hypothesis
y1 <- x1%*%as.matrix(beta1)+rnorm(n,sd=1)
y2 <- x2%*%as.matrix(beta2)+rnorm(n,sd=1)

## Differential regression (asymptotic p-values)
fit.alt <- diffregr_multisplit(y1,y2,x1,x2,b.splits=10)
par(cex=0.7)
plot(fit.alt)
cat('p-value: ',fit.alt$medagg.pval,'\n')

## ----echo=TRUE, include=TRUE, warning=TRUE, results="asis",fig.align='center',fig.width=3.5,fig.height=3.5----
## Differential regression (permutation-based p-values; 100 permutations)
fit.alt.perm <- diffregr_multisplit(y1,y2,x1,x2,b.splits=5,n.perm=100)

## ----echo=TRUE, include=TRUE-----------------------------------------------
# Generate networks with random means and covariances. 
n = 1000
p = 10
s = 0.9
n.comp = 3

# Create different mean vectors
Mu = matrix(0,p,n.comp)

# Define non-zero means in each group (non-overlapping)
nonzero.mean = split(sample(1:p),rep(1:n.comp,length=p))

# Set non-zero means to fixed value
for(k in 1:n.comp){
  Mu[nonzero.mean[[k]],k] = -2/sqrt(ceiling(p/n.comp))
}

# Generate data
sim.result = sim_mix_networks(n, p, n.comp, s, Mu=Mu)

## ----echo=TRUE, include=TRUE-----------------------------------------------
# Run mixglasso
mixglasso.result = mixglasso(sim.result$data, n.comp=3)

# Calculate adjusted rand index to judge how accurate the clustering is
# Values > 0.7 indicate good agreement.
library(mclust, quietly=TRUE)
adj.rand = adjustedRandIndex(mixglasso.result$comp, sim.result$comp)
cat('Adjusted Rand Index', round(adj.rand, digits=2), '\n')

## ----echo=FALSE, include=TRUE, cache=FALSE, results='asis'-----------------
crosstab <- function(class1,class2){
  tab <- matrix(NA,length(levels(class1)),length(levels(class2)))
  colnames(tab) <- levels(class2)
  rownames(tab) <- levels(class1)
  for (i in levels(class1)){
    tab[i,] <- table(class2[which(class1==i)])
  }
  return(tab)
}

# Relabel true groupings to avoid confusion
sim.grouping = c('A', 'B', 'C')[sim.result$comp]

cross.table = crosstab(factor(mixglasso.result$comp), factor(sim.grouping))

# Generate table
library(xtable)
latex.table = xtable(cross.table, caption=
										 	paste('Cross-tabulation of mixglasso clusters (rows) with',
                    'true group assignments (columns).'),
										 label='table:crosstab')

print(latex.table)

## ----echo=TRUE, include=TRUE, fig.width=5, fig.height=3, fig.align='center'----

# Run mixglasso over a range of numbers of components
mixglasso.result = mixglasso(sim.result$data, n.comp=1:6)

# Repeat with lambda=0 and lambda=Inf for comparison
mixglasso.result.0 = mixglasso(sim.result$data, n.comp=1:6, lambda=0)
mixglasso.result.Inf = mixglasso(sim.result$data, n.comp=1:6, lambda=Inf)

# Aggregate BIC results for plotting
BIC.vals = c(mixglasso.result$bic, mixglasso.result.0$bic,
						 mixglasso.result.Inf$bic)

lambda.labels = rep(c('Default', 'Lambda = 0', 'Lambda = Inf'), each=6)

# Plot to verify that minimum BIC value corresponds with true 
library(ggplot2)
plotting.frame <- data.frame(BIC=BIC.vals, Num.Comps=rep(1:6, 3), Lambda=lambda.labels)

p <- ggplot(plotting.frame) + 
	geom_line(aes(x=Num.Comps, y=BIC, colour=Lambda)) + 
	geom_vline(xintercept=3, linetype='dotted')

print(p)

## ----echo=TRUE, include=TRUE, fig.width=3, fig.height=3, fig.align='center'----

# Retrieve best clustering and networks by BIC
mixglasso.clustering = mixglasso.result$models[[mixglasso.result$bic.opt]]

# Plot networks, omitting edges with absolute partial correlation < 0.5 in 
# every group.
# NOTE: Not displayed.
# plot(mixglasso.clustering, p.corrs.thresh=0.5)

## ----echo=TRUE, include=TRUE, fig.width=5, fig.height=3, fig.align='center'----

# Plot edges, omitting those with absolute partial correlation < 0.5 in every 
# group.
g = dot_plot(mixglasso.clustering, p.corrs.thresh=0.5, dot.size.range=c(1,5))

## ----echo=TRUE, include=TRUE, fig.width=7, fig.height=7, fig.align='center'----

# Specify edges
node.pairs = rbind(c(9,10), c(2,5),c(4,9))

# Create scatter plots of specified edges
g = scatter_plot(mixglasso.clustering, data=sim.result$data,
						node.pairs=node.pairs, cex=0.5)

## ----echo=TRUE, include=TRUE, eval=FALSE-----------------------------------
# 
# # Save network in CSV format, omitting edges with absolute partial correlation
# # less than 0.25.
# #export_network(mixglasso.clustering, file='nethet_network.csv',
# #							 p.corrs.thresh=0.25)

## ----echo=TRUE, include=TRUE, eval=FALSE-----------------------------------
# 
# # Save network in CSV format suitable for Cytoscape import
# #export_network(mixglasso.clustering, file='nethet_network.csv',
# #							 p.corrs.thresh=0.25, quote=FALSE)

## ----echo=TRUE, include=TRUE, eval=TRUE------------------------------------
sessionInfo()

###############################################################
##This example illustrates the use of Differential Regression##
###############################################################

##set seed
set.seed(1)

## Number of predictors and sample size
p <- 100
n <- 80

## Predictor matrices
x1 <- matrix(rnorm(n*p),n,p)
x2 <- matrix(rnorm(n*p),n,p)

## Active-sets and regression coefficients
act1 <- sample(1:p,5)
act2 <- c(act1[1:3],sample(setdiff(1:p,act1),2))
beta1 <- beta2 <- rep(0,p)
beta1[act1] <- 0.5
beta2[act2] <- 0.5

## Response vectors under null-hypothesis
y1 <- x1%*%as.matrix(beta1)+rnorm(n,sd=1)
y2 <- x2%*%as.matrix(beta1)+rnorm(n,sd=1)

## Diffregr (asymptotic p-values)
fit.null <- diffregr_multisplit(y1,y2,x1,x2,b.splits=5)
fit.null$ms.pval#multi-split p-values
fit.null$medagg.pval#median aggregated p-values

## Response vectors under alternative-hypothesis
y1 <- x1%*%as.matrix(beta1)+rnorm(n,sd=1)
y2 <- x2%*%as.matrix(beta2)+rnorm(n,sd=1)

## Diffregr (asymptotic p-values)
fit.alt <- diffregr_multisplit(y1,y2,x1,x2,b.splits=5)
fit.alt$ms.pval
fit.alt$medagg.pval

## Diffregr (permutation-based p-values; 100 permutations)
fit.alt.perm <- diffregr_multisplit(y1,y2,x1,x2,b.splits=5,n.perm=100)
fit.alt.perm$ms.pval
fit.alt.perm$medagg.pval

##set seed
set.seed(1)

##number of predictors / sample size
p <- 100
n <- 80

##predictor matrices
x1 <- matrix(rnorm(n*p),n,p)
x2 <- matrix(rnorm(n*p),n,p)

##active-sets and regression coefficients
act1 <- sample(1:p,5)
act2 <- c(act1[1:3],sample(setdiff(1:p,act1),2))
beta1 <- beta2 <- rep(0,p)
beta1[act1] <- 0.5
beta2[act2] <- 0.5

##response vectors 
y1 <- x1%*%as.matrix(beta1)+rnorm(n,sd=1)
y2 <- x2%*%as.matrix(beta2)+rnorm(n,sd=1)

##run diffregr
split1 <- sample(1:n,50)#samples for screening (condition 1)
split2 <- sample(1:n,50)#samples for screening (condition 2)
fit <- diffregr_singlesplit(y1,y2,x1,x2,split1,split2)
fit$pval.onesided#p-value

##########################################################################################
# Differential Regression: Two-sample testing for high-dimensional regression
#-------------------------------------------------------------------------------
# * Intercepts are assumed to be zero in log-likelihood (mu1=mu2).
#   Therefore, center input data (y1,y2,x1,x2).
#
#
#
##########################################################################################


#####################
##Required Packages##
#####################
library(mvtnorm)
library(glmnet)
library(CompQuadForm)


#######################
##-----Screening-----##
#######################

##' Cross-validated Lasso screening (lambda.min-rule)
##'
##' 
##' @title Cross-validation lasso screening (lambda.min-rule)
##' @param x Predictor matrix
##' @param y Response vector
##' @return Active-set
##' @author n.stadler
##' @export
##' @examples
##' screen_cvmin.lasso(matrix(rnorm(5000),50,100),rnorm(50))
screen_cvmin.lasso <- function(x,y){
  fit.cv <- cv.glmnet(x,y)
  beta <- as.numeric(coef(fit.cv,s='lambda.min')[-1])
  p <- length(beta)
  n <- nrow(x)
  beta[-(order(abs(beta),decreasing=TRUE)[1:min(p,n)])] <- 0
  return(which(beta!=0))
}

##' Cross-validated Lasso screening (lambda.1se-rule)
##'
##' 
##' @title Cross-validated Lasso screening (lambda.1se-rule)
##' @param x Predictor matrix
##' @param y Response vector 
##' @return Active-set
##' @author n.stadler
##' @export
##' @examples
##' screen_cv1se.lasso(matrix(rnorm(5000),50,100),rnorm(50))
screen_cv1se.lasso <- function(x,y){
  fit.cv <- cv.glmnet(x,y)
  beta <- as.numeric(coef(fit.cv,s='lambda.1se')[-1])
  p <- length(beta)
  n <- nrow(x)
  d <- min(p,n)
  beta[-order(abs(beta),decreasing=TRUE)[1:d]] <- 0
  return(which(beta!=0))
}

##' Cross-validated Lasso screening and additional truncation.
##'
##' Computes Lasso coefficients (cross-validation optimal lambda). Truncates
##' smallest coefficients to zero, such that there are no more than n/k.trunc
##' non-zero coefficients.
##' 
##' @title Cross-validated Lasso screening and additional truncation.
##' @param x Predictor matrix.
##' @param y Response vector.
##' @param k.trunc Truncation constant="number of samples per predictor" (default=5).
##' @return Active-set.
##' @author n.stadler
##' @export
##' @examples
##' screen_cvtrunc.lasso(matrix(rnorm(5000),50,100),rnorm(50))
screen_cvtrunc.lasso <- function(x,y,k.trunc=5){
  n <- nrow(x)
  fit.cv <- cv.glmnet(x,y)
  beta <- as.numeric(coef(fit.cv,s='lambda.min')[-1])
  p <- length(beta)
  d <- min(floor(n/k.trunc),p,n)
  beta[-order(abs(beta),decreasing=TRUE)[1:d]] <- 0
  return(which(beta!=0))
}

##' Cross-validated Lasso screening and sqrt-truncation. 
##'
##' Computes Lasso coefficients (cross-validation optimal lambda). Truncates
##' smallest coefficients to zero, such that there are no more than sqrt(n)
##' non-zero coefficients.
##' 
##' @title Cross-validated Lasso screening and sqrt-truncation.
##' @param x Predictor matrix.
##' @param y Response vector. 
##' @return Active-set.
##' @author n.stadler
##' @export
##' @examples
##' screen_cvsqrt.lasso(matrix(rnorm(5000),50,100),rnorm(50))
screen_cvsqrt.lasso <- function(x,y){
  n <- nrow(x)
  fit.cv <- cv.glmnet(x,y)
  beta <- as.numeric(coef(fit.cv,s='lambda.min')[-1])
  p <- length(beta)
  d <- min(floor(sqrt(n)),p,n)
  beta[-order(abs(beta),decreasing=TRUE)[1:d]] <- 0
  return(which(beta!=0))
}

##' Cross-validated Lasso screening and upper bound on number of predictors
##'
##' Computes Lasso coefficients (cross-validation optimal lambda). Truncates
##' smalles coefficients to zero such that there are no more than no.predictors
##' non-zero coefficients
##' 
##' @title Cross-validated Lasso screening and upper bound on number of predictors.
##' @param x Predictor matrix.
##' @param y Response vector. 
##' @param no.predictors Upper bound on number of active predictors,
##' @return Active-set.
##' @author n.stadler
##' @export
##' @examples
##' screen_cvfix.lasso(matrix(rnorm(5000),50,100),rnorm(50))
screen_cvfix.lasso <- function(x,y,no.predictors=10){
  n <- nrow(x)
  fit.cv <- cv.glmnet(x,y)
  beta <- as.numeric(coef(fit.cv,s='lambda.min')[-1])
  p <- length(beta)
  d <- min(no.predictors,p,n)
  beta[-order(abs(beta),decreasing=TRUE)[1:d]] <- 0
  return(which(beta!=0))
}

##############################
##--------P-VALUES----------##
##############################

##' Log-likelihood ratio statistics for Differential Regression.
##'
##' 
##' @title Log-likelihood ratio statistics for Differential Regression.
##' @param y1 Response vector condition 1.
##' @param y2 Response vector condition 2.
##' @param y Pooled response vector.
##' @param xx1 Predictor matrix condition 1.
##' @param xx2 Predictor matrix condition 2.
##' @param xx Pooled predictor matrix
##' @param beta1 Regression coefficients condition 1.
##' @param beta2 Regression coefficients condition 2.
##' @param beta Pooled regression coefficients.
##' @return 2 times log-likelihood ratio statistics.
##' @author n.stadler
##' @keywords internal
logratio.diffregr <- function(y1,y2,y,xx1,xx2,xx,beta1,beta2,beta){
  ##Compute 2*log-likelihood ratio
  ##Input:
  ##-y1,y2,y
  ##-xx1,xx2,xx (include only selected features)
  ##-MLE estimates beta1,beta2,beta

  n1 <- length(y1)
  n2 <- length(y2)
  n <- length(y)
  mu1<-mu2<-mu<-0
  sig1 <- (sum(y1^2)/n1);sig2 <- (sum(y2^2)/n2);sig <- (sum(y^2)/n)
  if(length(beta1)!=0){
    mu1<-xx1%*%beta1
    sig1 <- sum((y1-mu1)^2)/n1
  }
  if(length(beta2)!=0){
    mu2<-xx2%*%beta2
    sig2 <- sum((y2-mu2)^2)/n2
  }
  if(length(beta)!=0){
    mu<-xx%*%beta
    sig <- sum((y-mu)^2)/n
  }

  2*(sum(dnorm(y1,mean=mu1,sd=sqrt(sig1),log=TRUE))+sum(dnorm(y2,mean=mu2,sd=sqrt(sig2),log=TRUE))-sum(dnorm(y,mean=mu,sd=sqrt(sig),log=TRUE)))
}

##' Computation M matrix and eigenvalues
##'
##' 
##' @title Computation M matrix and eigenvalues
##' @param Sig no descr
##' @param act no descr
##' @param act1 no descr
##' @param act2 no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
ww.mat.diffregr <- function(Sig,act,act1,act2){
  ##Compute W and Eval(W) (without simplification of W)
  ##
  ##Input:
  ##  Sig: E_0[s(X)s(X)'] (information matrix)
  ##  act: active variables for joint model
  ##  act1,act2: active variables for individual models
  bfg <- rbind(Sig[act1,act,drop=FALSE],Sig[act2,act,drop=FALSE])
  bf <- matrix(0,length(act1)+length(act2),length(act1)+length(act2))
  bf[1:length(act1),1:length(act1)] <- Sig[act1,act1]
  bf[length(act1)+(1:(length(act2))),length(act1)+(1:(length(act2)))] <- Sig[act2,act2]
  bg <- 2*Sig[act,act,drop=FALSE]
  mat <- rbind(cbind(diag(1,length(act1)+length(act2)),bfg%*%solve(bg)),cbind(-t(bfg)%*%solve(bf),diag(-1,length(act))))
  eval <- Re(eigen(mat)$values)
  eval[abs(eval)<10^{-6}] <- 0
  return(list(ww.mat=mat,eval=eval))
}

##' Computation M matrix and eigenvalues
##'
##' 
##' @title Computation M matrix and eigenvalues
##' @param Sig no descr
##' @param act no descr
##' @param act1 no descr
##' @param act2 no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
ww.mat2.diffregr <- function(Sig,act,act1,act2){
  ##Compute W and Eval(W) ('1st order' simplification of W)
  ##
  ##Input:
  ##  Sig: E_0[s(X)s(X)']
  ##  act: active variables for joint model (including sigma^2)
  ##  act1,act2: active variables for individual models (including sigma_1^2, sigma_2^2)
  dimf <- length(act1)+length(act2)
  dimg <- length(act)
  bfg <- rbind(Sig[act1,act,drop=FALSE],Sig[act2,act,drop=FALSE])
  bgf <- t(bfg)
  bf <- matrix(0,length(act1)+length(act2),length(act1)+length(act2))
  bf[1:length(act1),1:length(act1)] <- Sig[act1,act1]
  bf[length(act1)+(1:(length(act2))),length(act1)+(1:(length(act2)))] <- Sig[act2,act2]
  bg <- 2*Sig[act,act]
  if (dimf>=dimg){
    mat <- bgf%*%solve(bf)%*%bfg%*%solve(bg)
    eval<-rep(1,dimf-dimg)
  }
  if (dimf<dimg){
    mat <- bfg%*%solve(bg)%*%bgf%*%solve(bf)
    eval<-rep(-1,dimg-dimf)
  }
  eval.mu <- Re(eigen(mat)$values)
  eval2 <- 1-eval.mu
  eval2[abs(eval2)<10^{-6}] <- 0
  eval <- c(eval,sqrt(eval2),-sqrt(eval2))
  return(list(ww.mat=mat,eval=eval))
}

##' Computation Q matrix
##'
##' 
##' @title Computation Q matrix
##' @param Sig no descr
##' @param a no descr
##' @param b no descr
##' @param s no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
q.matrix.diffregr <- function(Sig,a,b,s){
  ##Compute Q: based on true information matrix (Sig)
  if(length(s)==0){
    return(Sig[a,b,drop=FALSE])
  }else{
    return(Sig[a,b,drop=FALSE]-Sig[a,s,drop=FALSE]%*%solve(Sig[s,s,drop=FALSE])%*%Sig[s,b,drop=FALSE])
  }
}

##' Computation eigenvalues
##'
##' 
##' @title Computation eigenvalues
##' @param Sig no descr
##' @param act no descr
##' @param act1 no descr
##' @param act2 no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
my.ev2.diffregr <- function(Sig,act,act1,act2){
  ##Compute Eval (with 2nd order simplification of W)
  ##
  ##Input:
  ##  Sig: E_0[s(X)s(X)']
  ##  act: active beta's for joint model (without sigma^2)
  ##  act1,act2: active beta's for individual models (without sigma_1^2, sigma_2^2)
  
  #dimension of models
  dimf1 <- length(act1)+1
  dimf2 <- length(act2)+1
  dimf <- dimf1+dimf2
  dimg <- length(act)+1
  #intersection of models
  ss <- intersect(act,intersect(act1,act2))
  if(length(ss)==0){warning('no intersection between models')}
  aa <- setdiff(act1,ss)
  bb <- setdiff(act2,ss)
  cc <- setdiff(act,ss)
  ev.aux <- ev.aux.complex<-numeric(0)
  if (dimf>=dimg){
    if (length(cc)!=0){
      qcc <- q.matrix.diffregr(Sig,cc,cc,ss)
      aux.mat <- matrix(0,length(cc),length(cc))
      if(length(aa)!=0){
        qac <- q.matrix.diffregr(Sig,aa,cc,ss)
        qaa <- q.matrix.diffregr(Sig,aa,aa,ss)
        aux.mat <- aux.mat+(t(qac)%*%solve(qaa)%*%qac)%*%solve(qcc)
      }
      if(length(bb)!=0){
        qbc <- q.matrix.diffregr(Sig,bb,cc,ss)
        qbb <- q.matrix.diffregr(Sig,bb,bb,ss)
        aux.mat <- aux.mat+(t(qbc)%*%solve(qbb)%*%qbc)%*%solve(qcc)
      }
      ev.aux.complex <- eigen(aux.mat)$values
      ev.aux <- Re(ev.aux.complex)
      ev.aux <-  sqrt(1-ev.aux/2)
    }
    eval<-rep(1,dimf-dimg)
    eval <- c(eval,rep(0,2*(length(ss)+1)),ev.aux,-ev.aux)
  }# end if (dimf>=dimg){
  if (dimf<dimg){
    if (length(cc)!=0){
      qcc <- q.matrix.diffregr(Sig,cc,cc,ss)
      if(length(aa)!=0){
        qac <- q.matrix.diffregr(Sig,aa,cc,ss)
        qaa <- q.matrix.diffregr(Sig,aa,aa,ss)
        oaa <- qac%*%solve(qcc)%*%t(qac)
        aux.mat.aa<- oaa%*%solve(qaa)
        aux.mat <- aux.mat.aa
      }
      if(length(bb)!=0){
        qbc <- q.matrix.diffregr(Sig,bb,cc,ss)
        qbb <- q.matrix.diffregr(Sig,bb,bb,ss)
        obb <- qbc%*%solve(qcc)%*%t(qbc)
        aux.mat.bb<- obb%*%solve(qbb)
        aux.mat <- aux.mat.bb
      }
      if((length(aa)!=0)&(length(bb)!=0)){
        oab <- qac%*%solve(qcc)%*%t(qbc)
        aux.mat<- rbind(cbind(aux.mat.aa,oab%*%solve(qbb)),cbind(t(oab)%*%solve(qaa),aux.mat.bb))
      }
    }
    if ((length(aa)!=0)|(length(bb)!=0)){##if (length(aa)==0)&(length(bb)==0) 'MI included in MJ; therefore -X^2(dim(MJ)-dim(MI)) distributed'
      ev.aux.complex <- eigen(aux.mat)$values
      ev.aux <- Re(ev.aux.complex)
      ev.aux <-  sqrt(1-ev.aux/2)
    }
    eval<-rep(-1,dimg-dimf)
    eval <- c(eval,rep(-1,(length(ss)+1)),rep(1,(length(ss)+1)),rep(0,2*(length(ss)+1)),ev.aux,-ev.aux)
  }
  return(list(eval=eval,ev.aux.complex=ev.aux.complex))
}

##' Computation beta matrix
##'
##' 
##' @title Computation beta matrix
##' @param ind1 no descr
##' @param ind2 no descr
##' @param beta1 no descr
##' @param beta2 no descr
##' @param beta no descr
##' @param sig1 no descr
##' @param sig2 no descr
##' @param sig no descr
##' @param Sig no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
beta.mat.diffregr<-function(ind1,ind2,beta1,beta2,beta,sig1,sig2,sig,Sig){
  ##compute Beta-Matrix
  ##Input:
  ##-beta1,beta2
  ## beta (for expectation)
  ##-sig1,sig2
  ## sig (for expectation)
  ##-Sig (variance of predictors=var(x))

  beta.mat <- Sig*(sig+as.numeric(t(beta-beta1)%*%Sig%*%(beta-beta2)))/(sig1*sig2)
  return(beta.mat[ind1,ind2,drop=FALSE])
}

##' Computation Q matrix
##'
##' 
##' @title Computation Q matrix
##' @param beta.a no descr
##' @param beta.b no descr
##' @param beta no descr
##' @param sig.a no descr
##' @param sig.b no descr
##' @param sig no descr
##' @param Sig no descr
##' @param act.a no descr
##' @param act.b no descr
##' @param ss no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
q.matrix.diffregr3 <- function(beta.a,beta.b,beta,sig.a,sig.b,sig,Sig,act.a,act.b,ss){

 ##Estimate Q
    b.ab<-beta.mat.diffregr(act.a,act.b,beta.a,beta.b,beta,sig.a,sig.b,sig,Sig)
    aa<-seq(1,length(act.a))[!(act.a%in%ss)]
    bb<-seq(1,length(act.b))[!(act.b%in%ss)]
    s.a<-seq(1,length(act.a))[(act.a%in%ss)]
    s.b<-seq(1,length(act.b))[(act.b%in%ss)]

    if(length(ss)==0){
        return(b.ab[aa,bb,drop=FALSE])
    }else{
        return(b.ab[aa,bb,drop=FALSE]-(b.ab[aa,s.b,drop=FALSE]%*%solve(b.ab[s.a,s.b,drop=FALSE])%*%b.ab[s.a,bb,drop=FALSE]))
    }
}

##' Computation Q matrix
##'
##' 
##' @title Computation Q matrix
##' @param b.mat no descr
##' @param act.a no descr
##' @param act.b no descr
##' @param ss no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
q.matrix.diffregr4 <- function(b.mat,act.a,act.b,ss){

 ##Estimate Q
    aa<-seq(1,length(act.a))[!(act.a%in%ss)]
    bb<-seq(1,length(act.b))[!(act.b%in%ss)]
    s.a<-seq(1,length(act.a))[(act.a%in%ss)]
    s.b<-seq(1,length(act.b))[(act.b%in%ss)]

    if(length(ss)==0){
        return(b.mat[aa,bb,drop=FALSE])
    }else{
        return(b.mat[aa,bb,drop=FALSE]-(b.mat[aa,s.b,drop=FALSE]%*%solve(b.mat[s.a,s.b,drop=FALSE])%*%b.mat[s.a,bb,drop=FALSE]))
    }
}

##' Estimate weights 
##'
##' estimate W-matrix (using plug-in estimates of Beta-matrix); calculate eigenvalues(W-matrix)
##' 
##' @title Estimate weights
##' @param y1 no descr
##' @param y2 no descr
##' @param x1 no descr
##' @param x2 no descr
##' @param beta1 no descr
##' @param beta2 no descr
##' @param beta no descr
##' @param act1 no descr
##' @param act2 no descr
##' @param act no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
est2.ww.mat.diffregr <- function(y1,y2,x1,x2,beta1,beta2,beta,act1,act2,act){

  n1 <- length(y1)
  n2 <- length(y2)
  y <- c(y1,y2)
  x <- rbind(x1,x2)
  n <- length(y)

  ##get mle's for sigma
  mu1<-mu2<-mu<-0
  sig1 <- (sum(y1^2)/n1);sig2 <- (sum(y2^2)/n2);sig <- (sum(y^2)/n)
  if(length(beta1)!=0){
    mu1<-x1[,act1,drop=FALSE]%*%beta1
    sig1 <- sum((y1-mu1)^2)/n1
  }
  if(length(beta2)!=0){
    mu2<-x2[,act2,drop=FALSE]%*%beta2
    sig2 <- sum((y2-mu2)^2)/n2
  }
  if(length(beta)!=0){
    mu<-x[,act,drop=FALSE]%*%beta
    sig <- sum((y-mu)^2)/n
  }

  ##expand beta's with zeros
  exp.beta <- exp.beta1 <- exp.beta2 <- rep(0,ncol(x1))
  exp.beta[act] <- beta
  exp.beta1[act1] <- beta1
  exp.beta2[act2] <- beta2

  ##compute beta.mat
  bact.pop1<- beta.mat.diffregr(act,act,exp.beta,exp.beta,exp.beta1,sig,sig,sig1,var(x1))
  bact.pop2<- beta.mat.diffregr(act,act,exp.beta,exp.beta,exp.beta2,sig,sig,sig2,var(x2))
  bact1<- beta.mat.diffregr(act1,act1,exp.beta1,exp.beta1,exp.beta1,sig1,sig1,sig1,var(x1))
  bact2<- beta.mat.diffregr(act2,act2,exp.beta2,exp.beta2,exp.beta2,sig2,sig2,sig2,var(x2))
  bact1.act<- beta.mat.diffregr(act1,act,exp.beta1,exp.beta,exp.beta1,sig1,sig,sig1,var(x1))
  bact2.act<- beta.mat.diffregr(act2,act,exp.beta2,exp.beta,exp.beta2,sig2,sig,sig2,var(x2))
  
  bfg <- rbind(bact1.act,bact2.act)
  bgf <- t(bfg)
  bf <- matrix(0,length(act1)+length(act2),length(act1)+length(act2))
  bf[1:length(act1),1:length(act1)] <- bact1
  bf[length(act1)+(1:length(act2)),length(act1)+(1:length(act2))] <- bact2
  bg <- bact.pop1+bact.pop2
  mat <- rbind(cbind(diag(1,length(act1)+length(act2)),bfg%*%solve(bg)),cbind(-t(bfg)%*%solve(bf),diag(-1,length(act))))
  
  eval.complex<-eigen(mat)$values
  eval <- Re(eval.complex)
  
  return(list(eval=c(1,0,0,eval),eval.complex=eval.complex))## eigenvalues 1,0,0 correspond to sig1, sig2 and sig12
}

##' Estimate weights
##'
##' 
##' @title Estimate weights
##' @param y1 no descr
##' @param y2 no descr
##' @param x1 no descr
##' @param x2 no descr
##' @param beta1 no descr
##' @param beta2 no descr
##' @param beta no descr
##' @param act1 no descr
##' @param act2 no descr
##' @param act no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
est2.ww.mat2.diffregr <- function(y1,y2,x1,x2,beta1,beta2,beta,act1,act2,act){
  ##Estimate W and Eval(W) ('1st order' simplification of W)
  ##
  ##Input:
  ##  data: y1,y2,x1,x2
  ##  beta: estimate for joint model
  ##  beta1,beta2: estimates for individual model
  ##  act: active beta's for joint model
  ##  act1,act2: active beta's for individual models
  ##
  ##8!8! does not include weights from estimating error variances... (TO DO)

  n1 <- length(y1)
  n2 <- length(y2)
  y <- c(y1,y2)
  x <- rbind(x1,x2)
  n <- length(y)

  ##get mle's for sigma
  mu1<-mu2<-mu<-0
  sig1 <- (sum(y1^2)/n1);sig2 <- (sum(y2^2)/n2);sig <- (sum(y^2)/n)
  if(length(beta1)!=0){
    mu1<-x1[,act1,drop=FALSE]%*%beta1
    sig1 <- sum((y1-mu1)^2)/n1
  }
  if(length(beta2)!=0){
    mu2<-x2[,act2,drop=FALSE]%*%beta2
    sig2 <- sum((y2-mu2)^2)/n2
  }
  if(length(beta)!=0){
    mu<-x[,act,drop=FALSE]%*%beta
    sig <- sum((y-mu)^2)/n
  }

  ##expand beta's with zeros
  exp.beta <- exp.beta1 <- exp.beta2 <- rep(0,ncol(x1))
  exp.beta[act] <- beta
  exp.beta1[act1] <- beta1
  exp.beta2[act2] <- beta2

  ##dimension of models
  dimf1 <- length(act1)
  dimf2 <- length(act2)
  dimf <- dimf1+dimf2
  dimg <- length(act)

  ##compute beta.mat
  bact.pop1<- beta.mat.diffregr(act,act,exp.beta,exp.beta,exp.beta1,sig,sig,sig1,var(x1))
  bact.pop2<- beta.mat.diffregr(act,act,exp.beta,exp.beta,exp.beta2,sig,sig,sig2,var(x2))
  bact1<- beta.mat.diffregr(act1,act1,exp.beta1,exp.beta1,exp.beta1,sig1,sig1,sig1,var(x1))
  bact2<- beta.mat.diffregr(act2,act2,exp.beta2,exp.beta2,exp.beta2,sig2,sig2,sig2,var(x2))
  bact1.act<- beta.mat.diffregr(act1,act,exp.beta1,exp.beta,exp.beta1,sig1,sig,sig1,var(x1))
  bact2.act<- beta.mat.diffregr(act2,act,exp.beta2,exp.beta,exp.beta2,sig2,sig,sig2,var(x2))
  
  bfg <- rbind(bact1.act,bact2.act)
  bgf <- t(bfg)
  bf <- matrix(0,dimf,dimf)
  bf[1:dimf1,1:dimf1] <- bact1
  bf[dimf1+(1:dimf2),dimf1+(1:dimf2)] <- bact2
  bg <- bact.pop1+bact.pop2
  
  if (dimf>=dimg){
    mat <- bgf%*%solve(bf)%*%bfg%*%solve(bg)
    eval<-rep(1,dimf-dimg)
  }
  if (dimf<dimg){
    mat <- bfg%*%solve(bg)%*%bgf%*%solve(bf)
    eval<-rep(-1,dimg-dimf)
  }
  eval.mu.complex<-eigen(mat)$values
  eval.mu <- Re(eval.mu.complex)
  eval <- c(eval,sqrt(1-eval.mu),-sqrt(1-eval.mu))
  
  return(list(eval=eval,eval.mu.complex=eval.mu.complex))
}

##' Compute weights of sum-w-chi2 (2nd order simplification)
##'
##' 
##' *expansion of W in two directions ("dimf>dimg direction" & "dimf>dimg direction") 
##' *simplified computation of weights is obtained by assuming H0 and that X_u~X_v holds
##' @param y1 no descr
##' @param y2 no descr
##' @param x1 no descr
##' @param x2 no descr
##' @param beta1 no descr
##' @param beta2 no descr
##' @param beta no descr
##' @param act1 no descr
##' @param act2 no descr
##' @param act no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
est2.my.ev2.diffregr <- function(y1,y2,x1,x2,beta1,beta2,beta,act1,act2,act){

  ##Estimate Evals ('2nd order' simplification of W)
  ##
  ##Input:
  ##  data: y1,y2,x1,x2
  ##  beta: estimate for joint model
  ##  beta1,beta2: estimates for individual model
  ##  act: active beta's for joint model
  ##  act1,act2: active beta's for individual models

  n1 <- length(y1)
  n2 <- length(y2)
  y <- c(y1,y2)
  x <- rbind(x1,x2)
  n <- length(y)

  ##get mle's for sigma
  mu1<-mu2<-mu<-0
  sig1 <- (sum(y1^2)/n1);sig2 <- (sum(y2^2)/n2);sig <- (sum(y^2)/n)
  if(length(beta1)!=0){
    mu1<-x1[,act1,drop=FALSE]%*%beta1
    sig1 <- sum((y1-mu1)^2)/n1
  }
  if(length(beta2)!=0){
    mu2<-x2[,act2,drop=FALSE]%*%beta2
    sig2 <- sum((y2-mu2)^2)/n2
  }
  if(length(beta)!=0){
    mu<-x[,act,drop=FALSE]%*%beta
    sig <- sum((y-mu)^2)/n
  }

  ##expand beta's with zeros
  exp.beta <- exp.beta1 <- exp.beta2 <- rep(0,ncol(x1))
  exp.beta[act] <- beta
  exp.beta1[act1] <- beta1
  exp.beta2[act2] <- beta2

  ##dimension of models
  dimf1 <- length(act1)+1
  dimf2 <- length(act2)+1
  dimf <- dimf1+dimf2
  dimg <- length(act)+1
  
  ##intersection of models
  ss <- intersect(act,intersect(act1,act2))
  if(length(ss)==0){cat('warning! no intersection between models','\n')}
  aa <- setdiff(act1,ss)
  bb <- setdiff(act2,ss)
  cc <- setdiff(act,ss)

  ev.aux <- ev.aux.complex<-numeric(0)
  if (dimf>=dimg){
    if (length(cc)!=0){
      qcc <- q.matrix.diffregr3(exp.beta,exp.beta,exp.beta1,sig,sig,sig1,var(x1),act,act,ss)+q.matrix.diffregr3(exp.beta,exp.beta,exp.beta2,sig,sig,sig2,var(x2),act,act,ss)
      aux.mat <- matrix(0,length(cc),length(cc))
      if(length(aa)!=0){
        qac <- q.matrix.diffregr3(exp.beta1,exp.beta,exp.beta1,sig1,sig,sig1,var(x1),act1,act,ss)
        qaa <- q.matrix.diffregr3(exp.beta1,exp.beta1,exp.beta1,sig1,sig1,sig1,var(x1),act1,act1,ss)
        aux.mat <- aux.mat+(t(qac)%*%solve(qaa)%*%qac)%*%solve(qcc)
      }
      if(length(bb)!=0){
        qbc <- q.matrix.diffregr3(exp.beta2,exp.beta,exp.beta2,sig2,sig,sig2,var(x2),act2,act,ss)
        qbb <- q.matrix.diffregr3(exp.beta2,exp.beta2,exp.beta2,sig2,sig2,sig2,var(x2),act2,act2,ss)
        aux.mat <- aux.mat+(t(qbc)%*%solve(qbb)%*%qbc)%*%solve(qcc)
      }
      ev.aux.complex <- eigen(aux.mat)$values
      ev.aux <- Re(ev.aux.complex)
      ev.aux <-  sqrt(1-ev.aux)
    }
    eval<-rep(1,dimf-dimg)
    eval <- c(eval,rep(0,2*(length(ss)+1)),ev.aux,-ev.aux)
  }# end if (dimf>=dimg){
  if (dimf<dimg){
    if (length(cc)!=0){
      qcc <- q.matrix.diffregr3(exp.beta,exp.beta,exp.beta1,sig,sig,sig1,var(x1),act,act,ss)+q.matrix.diffregr3(exp.beta,exp.beta,exp.beta2,sig,sig,sig2,var(x2),act,act,ss)
      aux.mat <- matrix(0,length(cc),length(cc))
      if(length(aa)!=0){
        qac <- q.matrix.diffregr3(exp.beta1,exp.beta,exp.beta1,sig1,sig,sig1,var(x1),act1,act,ss)
        qaa <- q.matrix.diffregr3(exp.beta1,exp.beta1,exp.beta1,sig1,sig1,sig1,var(x1),act1,act1,ss)
        oaa <- qac%*%solve(qcc)%*%t(qac)
        aux.mat.aa<- oaa%*%solve(qaa)
        aux.mat <- aux.mat.aa
      }
      if(length(bb)!=0){
        qbc <- q.matrix.diffregr3(exp.beta2,exp.beta,exp.beta2,sig2,sig,sig2,var(x2),act2,act,ss)
        qbb <- q.matrix.diffregr3(exp.beta2,exp.beta2,exp.beta2,sig2,sig2,sig2,var(x2),act2,act2,ss)
        obb <- qbc%*%solve(qcc)%*%t(qbc)
        aux.mat.bb<- obb%*%solve(qbb)
        aux.mat <- aux.mat.bb
      }
      if((length(aa)!=0)&(length(bb)!=0)){
        oab <- qac%*%solve(qcc)%*%t(qbc)
        aux.mat<- rbind(cbind(aux.mat.aa,oab%*%solve(qbb)),cbind(t(oab)%*%solve(qaa),aux.mat.bb))
      }
    }
    if ((length(aa)!=0)|(length(bb)!=0)){ ##if (length(aa)==0)&(length(bb)==0) 'MI included in MJ; therefore -X^2(dim(MJ)-dim(MI)) distributed'
      ev.aux.complex <- eigen(aux.mat)$values
      ev.aux <- Re(ev.aux.complex)
      ev.aux <-  sqrt(1-ev.aux)
    }
    eval<-rep(-1,dimg-dimf)
    eval <- c(eval,rep(-1,(length(ss)+1)),rep(1,(length(ss)+1)),rep(0,2*(length(ss)+1)),ev.aux,-ev.aux)
  }# end if (dimf<dimg){
  return(list(eval=eval,ev.aux.complex=ev.aux.complex))
}

##' Compute weights of sum-of-weighted-chi2s
##'
##' *'2nd order simplification':
##'   1) Factor out (1-vi)^(d1+d2) "expansion in dimf>dimg direction (old terminology)"
##'   2) Factor out (1-mu)^d0 
##' *simplified computation of weights is obtained without further invoking H0, or assuming X_u~X_v
##' 
##' @title Compute weights of sum-of-weighted-chi2s
##' @param y1 Response vector sample 1.
##' @param y2 Response vector sample 2.
##' @param x1 Predictor matrix sample 1.
##' @param x2 Predictor matrix sample 2.
##' @param beta1 MLE (regression coefficients) sample 1.
##' @param beta2 MLE (regression coefficients) sample 2.
##' @param beta Pooled MLE (regression coefficients).
##' @param act1 Active-set sample 1
##' @param act2 Active-set sample 2
##' @param act Pooled active-set
##' @return Eigenvalues of M, respectively the weights.
##' @author n.stadler
##' @keywords internal
est2.my.ev3.diffregr <- function(y1,y2,x1,x2,beta1,beta2,beta,act1,act2,act){

  show.warn <- FALSE

  n1 <- length(y1)
  n2 <- length(y2)
  y <- c(y1,y2)
  x <- rbind(x1,x2)
  n <- length(y)

  ##get mle's for sigma
  mu1<-mu2<-mu<-0
  sig1 <- (sum(y1^2)/n1);sig2 <- (sum(y2^2)/n2);sig <- (sum(y^2)/n)
  if(length(beta1)!=0){
    mu1<-x1[,act1,drop=FALSE]%*%beta1
    sig1 <- sum((y1-mu1)^2)/n1
  }
  if(length(beta2)!=0){
    mu2<-x2[,act2,drop=FALSE]%*%beta2
    sig2 <- sum((y2-mu2)^2)/n2
  }
  if(length(beta)!=0){
    mu<-x[,act,drop=FALSE]%*%beta
    sig <- sum((y-mu)^2)/n
  }

  ##expand beta's with zeros
  exp.beta <- exp.beta1 <- exp.beta2 <- rep(0,ncol(x1))
  exp.beta[act] <- beta
  exp.beta1[act1] <- beta1
  exp.beta2[act2] <- beta2

  ##dimension of models
  dimf1 <- length(act1)+1
  dimf2 <- length(act2)+1
  dimf <- dimf1+dimf2
  dimg <- length(act)+1
  
  ##intersection of models
  ss <- intersect(act,intersect(act1,act2))
  if((length(ss)==0)&show.warn){cat('warning! no intersection between models','\n')}
  aa <- setdiff(act1,ss)
  bb <- setdiff(act2,ss)
  cc <- setdiff(act,ss)

  ev.aux <- ev.aux.complex<-numeric(0)
  no.zero.ev.aux <- 0
  #if (dimf>=dimg){
  if (length(cc)!=0){
    qcc1 <- q.matrix.diffregr3(exp.beta,exp.beta,exp.beta1,sig,sig,sig1,var(x1),act,act,ss)
    qcc2 <- q.matrix.diffregr3(exp.beta,exp.beta,exp.beta2,sig,sig,sig2,var(x2),act,act,ss)
    bmat <- beta.mat.diffregr(act,act,exp.beta,exp.beta,exp.beta1,sig,sig,sig1,var(x1))+beta.mat.diffregr(act,act,exp.beta,exp.beta,exp.beta2,sig,sig,sig2,var(x2))
    qcc12 <- q.matrix.diffregr4(bmat,act,act,ss)
    aux.mat <- diag(1,length(cc))-qcc1%*%solve(qcc12)-qcc2%*%solve(qcc12)
    if(length(aa)!=0){
      qac <- q.matrix.diffregr3(exp.beta1,exp.beta,exp.beta1,sig1,sig,sig1,var(x1),act1,act,ss)
      qaa <- q.matrix.diffregr3(exp.beta1,exp.beta1,exp.beta1,sig1,sig1,sig1,var(x1),act1,act1,ss)
      aux.mat <- aux.mat+(t(qac)%*%solve(qaa)%*%qac)%*%solve(qcc12)
    }
    if(length(bb)!=0){
      qbc <- q.matrix.diffregr3(exp.beta2,exp.beta,exp.beta2,sig2,sig,sig2,var(x2),act2,act,ss)
      qbb <- q.matrix.diffregr3(exp.beta2,exp.beta2,exp.beta2,sig2,sig2,sig2,var(x2),act2,act2,ss)
      aux.mat <- aux.mat+(t(qbc)%*%solve(qbb)%*%qbc)%*%solve(qcc12)
    }
    ev.aux.complex <- eigen(aux.mat)$values
    ev.aux <- Re(ev.aux.complex)
    zero.ev.aux <- abs(ev.aux)<10^{-10}
    no.zero.ev.aux <- sum(zero.ev.aux)
    ev.aux <- ev.aux[!zero.ev.aux]
    ev.aux <-  sqrt(1-ev.aux)
  }
  eval<-c(rep(1,dimf-dimg+no.zero.ev.aux),rep(-1,no.zero.ev.aux))
  eval <- c(eval,rep(0,2*(length(ss)+1)),ev.aux,-ev.aux)
  #}# end if (dimf>=dimg){
  
  return(list(eval=eval,ev.aux.complex=ev.aux.complex))
}

##' Computation "split-asym"/"split-perm" p-values.
##'
##' 
##' @title Computation "split-asym" p-values.
##' @param y1 Response vector condition 1.
##' @param y2 Response vector condition 2.
##' @param x1 Predictor matrix condition 1.
##' @param x2 Predictor matrix condition 2.
##' @param beta1 Regression coefficients condition 1.
##' @param beta2 Regression coefficients condition 2.
##' @param beta Pooled regression coefficients.
##' @param act1 Active-set condition 1.
##' @param act2 Active-set condition 2.
##' @param act Pooled active-set.
##' @param compute.evals Method for computation of weights.
##' @param method.compquadform Method to compute distribution function of w-sum-of-chi2.
##' @param acc See ?davies.
##' @param epsabs See ?imhof.
##' @param epsrel See ?imhof.
##' @param show.warn Show warnings?
##' @param n.perm Number of permutations.
##' @return P-value, test statistic, estimated weights.
##' @author n.stadler
diffregr_pval <- function(y1,y2,x1,x2,beta1,beta2,beta,act1,act2,act,compute.evals,method.compquadform,acc,epsabs,epsrel,show.warn,n.perm){

  if(is.null(n.perm)){
    ##########################
    ##compute test-statistic##
    ##########################
    teststat <- logratio.diffregr(y1,y2,c(y1,y2),x1[,act1,drop=FALSE],x2[,act2,drop=FALSE],rbind(x1,x2)[,act,drop=FALSE],beta1,beta2,beta)
    #################################
    ##compute weights of sum-w-chi2##
    #################################
    weights.nulldistr <- eval(as.name(compute.evals))(y1,y2,x1,x2,beta1,beta2,beta,act1,act2,act)$eval
    weights.nulldistr <- weights.nulldistr[weights.nulldistr!=0]
    if (any(is.na(weights.nulldistr))){
      cat('warning: weight with value NA; pval=NA','\n')
      pval.onesided <- pval.twosided <- NA
    }else{
      if(method.compquadform=='davies'){
        pval.onesided <- davies(teststat,lambda=weights.nulldistr,acc=acc)$Qq;if(show.warn){cat('ifault(davies):',davies(teststat,lambda=weights.nulldistr,acc=acc)$ifault,'\n')}
      }
      if(method.compquadform=='imhof'){
        pval.onesided <-imhof(teststat, lambda=weights.nulldistr,epsabs = epsabs, epsrel = epsrel, limit = 10000)$Qq
        if(pval.onesided<0){pval.onesided <- 0}
        if(pval.onesided>1){pval.onesided <- 1}
      }
      pval.twosided <- 2*min(pval.onesided,1-pval.onesided)
    }
    return(list(pval.onesided=pval.onesided,pval.twosided=pval.twosided,weights.nulldistr=weights.nulldistr,teststat=teststat))
  }else{
    
    return(perm.diffregr_pval(y1,y2,x1,x2,act1,act2,act,n.perm))
  }
}

##' Auxiliary function for computation of "split-perm" p-value.
##'
##' 
##' @title Auxiliary function for computation of "split-perm" p-value.
##' @param y1 Response vector condition 1.
##' @param y2 Response vector condition 2.
##' @param y12 Pooled response vector.
##' @param x1 Predictor matrix condition 1.
##' @param x2 Predictor matrix condition 2.
##' @param x12 Pooled predictor matrix
##' @return Test statistic (log-likelihood-ratio statistic).
##' @author n.stadler
##' @keywords internal
perm.diffregr_teststat <- function(y1,y2,y12,x1,x2,x12){
  p1 <- ncol(x1)
  p2 <- ncol(x2)
  p12 <- ncol(x12)
  
  if(p1!=0){
    beta1 <- as.numeric(coef(lm(y1~x1-1)))
  }else{
   beta1<-numeric(0)
  }
  if(p2!=0){
    beta2 <- as.numeric(coef(lm(y2~x2-1)))
  }else{
   beta2<-numeric(0)
  }
  if(p12!=0){
    beta <- as.numeric(coef(lm(y12~x12-1)))
  }else{
   beta<-numeric(0)
  }
  return(logratio.diffregr(y1,y2,y12,x1,x2,x12,beta1,beta2,beta))
}

##' Computation "split-perm" p-value.
##'
##' 
##' @title Computation "split-perm" p-value.
##' @param y1 Response vector condition 1.
##' @param y2 Response vector condition 2.
##' @param x1 Predictor matrix condition 1.
##' @param x2 Predictor matrix condition 2.
##' @param act1 Active-set condition 1.
##' @param act2 Active-set condition 2.
##' @param act Pooled active-set.
##' @param n.perm Number of permutations.
##' @return Permutation based p-value.
##' @author n.stadler
##' @keywords internal
perm.diffregr_pval <- function(y1,y2,x1,x2,act1,act2,act,n.perm){
  n1 <- nrow(x1);n2 <- nrow(x2)
  x12 <- rbind(x1,x2)
  x12.act1 <- x12[,act1,drop=FALSE]
  x12.act2 <- x12[,act2,drop=FALSE]
  x12.act <- x12[,act,drop=FALSE]
  y12 <- c(y1,y2)

  tobs <- perm.diffregr_teststat(y1,y2,y12,x12.act1[1:n1,,drop=FALSE],x12.act2[(n1+1):(n1+n2),,drop=FALSE],x12.act)

  tperm <- sapply(1:n.perm,
                  function(i){
                    my.perm <- sample(c(rep(1,n1),rep(2,n2)))
                    x1.p <- x12.act1[my.perm==1,,drop=FALSE]
                    x2.p <- x12.act2[my.perm==2,,drop=FALSE]
                    y1.p <- y12[my.perm==1]
                    y2.p <- y12[my.perm==2]
                    perm.diffregr_teststat(y1.p,y2.p,y12,x1.p,x2.p,x12.act)
                  }
                  )
  pval <- (1+sum(tperm>=tobs))/n.perm
  return(list(pval.onesided=pval,pval.twosided=pval,weights.nulldistr=NULL,teststat=tobs))
}
              
##' Differential Regression (single-split version).
##'
##' Intercepts in regression models are assumed to be zero (mu1=mu2=0).
##' You might need to center the input data prior to running
##' Differential Regression.
##' 
##' @title Differential Regression (single-split version).
##' @param y1 Response vector condition 1.
##' @param y2 Response vector condition 2.
##' @param x1 Predictor matrix condition 1.
##' @param x2 Predictor matrix condition 2.
##' @param split1 Samples condition 1 used in screening-step.
##' @param split2 Samples condition 2 used in screening-step.
##' @param screen.meth Screening method (default='screen_cvtrunc.lasso').
##' @param compute.evals Method to estimate the weights in the weighted-sum-of-chi2s distribution.
##'                      The default and (currently) the only available option 
##'                      is the method 'est2.my.ev3.diffregr'.
##' @param method.compquadform Algorithm for computing distribution function
##'                            of weighted-sum-of-chi2 (default='imhof').
##' @param acc See ?davies (default=1e-4).
##' @param epsabs See ?imhof (default=1e-10).
##' @param epsrel See ?imhof (default=1e-10).
##' @param show.warn Show warnings (default=FALSE)? 
##' @param n.perm Number of permutation for "split-perm" p-value (default=NULL).
##' @param ... Other arguments specific to screen.meth.
##' @return List consisting of
##' \item{pval.onesided}{"One-sided" p-value.}
##' \item{pval.twosided}{"Two-sided" p-value. Ignore all "*.twosided results.}
##' \item{teststat}{2 times Log-likelihood-ratio statistics}
##' \item{weights.nulldistr}{Estimated weights of weighted-sum-of-chi2s.}
##' \item{active}{List of active-sets obtained in screening step.}
##' \item{beta}{Regression coefficients (MLE) obtaind in cleaning-step.}
##' @author n.stadler
##' @export
##' @example tests/Examples/diffregr_ss_ex.R
diffregr_singlesplit<- function(y1,y2,x1,x2,split1,split2,screen.meth='screen_cvtrunc.lasso',
                                compute.evals='est2.my.ev3.diffregr',method.compquadform='imhof',acc=1e-04,
                                epsabs=1e-10,epsrel=1e-10,
                                show.warn=FALSE,n.perm=NULL,...){
  
  n1 <- nrow(x1)
  n2 <- nrow(x2)
 
  est.beta <- active <- list()#save est.beta & active variables

  ###############
  ##Joint Model##
  ###############
  xx.train <- rbind(x1[split1,],x2[split2,])
  yy.train <- c(y1[split1],y2[split2])
  xx.valid <- rbind(x1[-split1,],x2[-split2,])
  yy.valid <- c(y1[-split1],y2[-split2])
  active[['modJ']] <- eval(as.name(screen.meth))(xx.train,yy.train,...)
  if(length(active[['modJ']])!=0){
    est.beta[['modJ']] <- as.numeric(coef(lm(yy.valid~xx.valid[,active[['modJ']]]-1)))
  }else{
    est.beta[['modJ']]<-numeric(0)
  }
  ####################  
  ##Individual Model##
  ####################
  for (j in c('1','2')){
    split.train <- eval(as.name(paste('split',j,sep='')))
    xx.train <- eval(as.name(paste('x',j,sep='')))[split.train,]
    yy.train <- eval(as.name(paste('y',j,sep='')))[split.train]
    xx.valid <- eval(as.name(paste('x',j,sep='')))[-split.train,]
    yy.valid <- eval(as.name(paste('y',j,sep='')))[-split.train]
    active[[paste('modIpop',j,sep='')]] <- eval(as.name(screen.meth))(xx.train,yy.train,...)

    if(length(active[[paste('modIpop',j,sep='')]])!=0){
      est.beta[[paste('modIpop',j,sep='')]] <- as.numeric(coef(lm(yy.valid~xx.valid[,active[[paste('modIpop',j,sep='')]]]-1)))
    }else{
      est.beta[[paste('modIpop',j,sep='')]]<-numeric(0)
    }
  }
  ###############
  ##Some Checks##
  ###############
  l.act <- lapply(active,length)
  n1.valid <- nrow(x1[-split1,])
  n2.valid <- nrow(x2[-split2,])
  if (any(l.act==0)){ if(show.warn){cat('warning: at least one active-set is empty','\n')}}
  if (all(l.act<c(n1.valid+n2.valid,n1.valid,n2.valid))){
    res.pval <- diffregr_pval(y1=y1[-split1],y2=y2[-split2],
                              x1=x1[-split1,,drop=FALSE],x2=x2[-split2,,drop=FALSE],
                              beta1=est.beta[['modIpop1']],beta2=est.beta[['modIpop2']],beta=est.beta[['modJ']],
                              active[['modIpop1']],active[['modIpop2']],active[['modJ']],
                              compute.evals,method.compquadform,acc,epsabs,epsrel,show.warn,n.perm)
  }else{
    if(show.warn){cat('warning: dim(model) > n: pval=NA','\n')}
    res.pval <- list(pval.onesided=NA,pval.twosided=NA,weights.nulldistr=NA,teststat=NA)
  }
  
  return(list(pval.onesided=res.pval$pval.onesided,pval.twosided=res.pval$pval.twosided,
              teststat=res.pval$teststat,weights.nulldistr=res.pval$weights.nulldistr,
              active=active,beta=est.beta))
}

##' Differential Regression (multi-split version).
##'
##' Intercepts in regression models are assumed to be zero (mu1=mu2=0).
##' You might need to center the input data prior to running
##' Differential Regression.
##'
##' 
##' @title Differential Regression (multi-split version).
##' @param y1 Response vector condition 1.
##' @param y2 Response vector condition 2.
##' @param x1 Predictor matrix condition 1.
##' @param x2 Predictor matrix condition 2.
##' @param b.splits Number of splits (default=50).
##' @param frac.split Fraction train-data (screening) / test-data (cleaning) (default=0.5).
##' @param screen.meth Screening method (default='screen_cvtrunc.lasso').
##' @param gamma.min Tuning parameter in p-value aggregation of Meinshausen et al (2009) (default=0.05).
##' @param compute.evals Method to estimate the weights in the weighted-sum-of-chi2s distribution.
##'                      The default and (currently) the only available option 
##'                      is the method 'est2.my.ev3.diffregr'.
##' @param method.compquadform Algorithm for computing distribution function
##'                            of weighted-sum-of-chi2 (default='imhof').
##' @param acc See ?davies (default=1e-4).
##' @param epsabs See ?imhof (default=1e-10).
##' @param epsrel See ?imhof (default=1e-10).
##' @param show.warn Show warnings (default=FALSE)?
##' @param n.perm Number of permutation for "split-perm" p-value. Default=NULL, which means
##'               that the asymptotic approximation is used.
##' @param mc.flag If \code{TRUE} use parallel execution for each b.splits via function 
##'                \code{mclapply} of package \code{parallel}.
##' @param mc.set.seed See mclapply. Default=TRUE
##' @param mc.preschedule See mclapply. Default=TRUE
##' @param mc.cores Number of cores to use in parallel execution. Defaults to
##'                 mc.cores option if set, or 2 otherwise.
##' @param ... Other arguments specific to screen.meth.
##' @return List consisting of
##' \item{ms.pval}{p-values for all b.splits}
##' \item{ss.pval}{single-split p-value}
##' \item{medagg.pval}{median aggregated p-value}
##' \item{meinshagg.pval}{meinshausen aggregated p-value (meinshausen et al 2009)}
##' \item{teststat}{test statistics for b.splits}
##' \item{weights.nulldistr}{estimated weights}
##' \item{active.last}{active-sets obtained in last screening-step}
##' \item{beta.last}{constrained mle (regression coefficients) obtained in last cleaning-step}
##' @author n.stadler
##' @export
##' @example tests/Examples/diffregr_ex.R
diffregr_multisplit<- function(y1,y2,x1,x2,b.splits=50,frac.split=1/2,screen.meth='screen_cvtrunc.lasso',
                               gamma.min=0.05,compute.evals='est2.my.ev3.diffregr',
                               method.compquadform='imhof',acc=1e-04,epsabs=1e-10,epsrel=1e-10,
                               show.warn=FALSE,n.perm=NULL,
                               mc.flag=FALSE,mc.set.seed=TRUE, mc.preschedule = TRUE,mc.cores=getOption("mc.cores", 2L),...){

  n1 <- nrow(x1)
  n2 <- nrow(x2)

  if(mc.flag && .Platform$OS.type == "unix"){
      res.multisplit <- mclapply(seq(b.splits),
                                 FUN=function(i){
                                     split1 <- sample(1:n1,floor((n1-1)*frac.split),replace=FALSE)
                                     split2 <- sample(1:n2,floor((n2-1)*frac.split),replace=FALSE)
                                     res.singlesplit <- diffregr_singlesplit(y1,y2,x1,x2,split1,split2,screen.meth,
                                                                             compute.evals,method.compquadform,
                                                                             acc,epsabs,epsrel,show.warn,n.perm,...)
                                 },mc.set.seed=mc.set.seed, mc.preschedule = mc.preschedule, mc.cores=mc.cores)                                 
  }else{
  	  if(mc.flag) warning('Windows does not support parallelisation via mclapply, using sequential lapply instead.')
      res.multisplit <- lapply(seq(b.splits),
                               function(i){
                                   split1 <- sample(1:n1,floor((n1-1)*frac.split),replace=FALSE)
                                   split2 <- sample(1:n2,floor((n2-1)*frac.split),replace=FALSE)
                                   res.singlesplit <- diffregr_singlesplit(y1,y2,x1,x2,split1,split2,screen.meth,
                                                                           compute.evals,method.compquadform,
                                                                           acc,epsabs,epsrel,show.warn,n.perm,...)                      
                               })
  }
  
  pval.onesided <- sapply(res.multisplit,function(x){x[['pval.onesided']]},simplify='array')
  pval.twosided <- sapply(res.multisplit,function(x){x[['pval.twosided']]},simplify='array')
  teststat <- sapply(res.multisplit,function(x){x[['teststat']]},simplify='array')
  weights.nulldistr <- sapply(res.multisplit,function(x){x[['weights.nulldistr']]},simplify='array')
  aggpval.onesided <- min(1,(1-log(gamma.min))*optimize(f=agg.pval,interval=c(gamma.min,1),maximum=FALSE,pval=pval.onesided[!is.na(pval.onesided)])$objective)
  aggpval.twosided <- min(1,(1-log(gamma.min))*optimize(f=agg.pval,interval=c(gamma.min,1),maximum=FALSE,pval=pval.twosided[!is.na(pval.twosided)])$objective)

  result <- list(ms.pval=pval.onesided,
                 ss.pval=pval.onesided[1],
                 medagg.pval=median(pval.onesided,na.rm=TRUE),
                 meinshagg.pval=aggpval.onesided,
                 teststat=teststat,weights.nulldistr=weights.nulldistr,
                 active.last=res.multisplit[[b.splits]]$active,
                 beta.last=res.multisplit[[b.splits]]$beta)
  class(result) <- 'diffregr'
  return(result)
}
    

##' Old single-split function for diffregr
##'
##' 
##' @title old single-split function for diffregr
##' @param y1 no descr
##' @param y2 no descr
##' @param x1 no descr
##' @param x2 no descr
##' @param n.screen.pop1 no descr
##' @param n.screen.pop2 no descr
##' @param screen.meth no descr
##' @param compute.evals no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
twosample_single_regr <- function(y1,y2,x1,x2,n.screen.pop1=100,n.screen.pop2=100,screen.meth='screen_cvmin.lasso',compute.evals='est2.my.ev3.diffregr'){

  ##Single-Split Pvalues
  ##
  ##Input:
  ##
  ## -Data:y1,y2,x1,x2
  ## -n.screen.pop1, n.screen.pop2: no samples for screening
  ## -screen.meth={lasso.cvmin,lasso.cv1se}
  ## -compute.evals: 'est2.my.ev2.diffregr'

  n1 <- nrow(x1)
  n2 <- nrow(x2)

  ##split data
  split1 <- sample(1:n1,n.screen.pop1,replace=FALSE)
  split2 <- sample(1:n2,n.screen.pop2,replace=FALSE)

  est.beta <- active <- list()#save est.beta & active variables

  ##model joint
  xx.train <- rbind(x1[split1,],x2[split2,])
  yy.train <- c(y1[split1],y2[split2])
  xx.valid <- rbind(x1[-split1,],x2[-split2,])
  yy.valid <- c(y1[-split1],y2[-split2])
  active[['modJ']] <- eval(as.name(screen.meth))(xx.train,yy.train)
  if(length(active[['modJ']])!=0){
    est.beta[['modJ']] <- as.numeric(coef(lm(yy.valid~xx.valid[,active[['modJ']]]-1)))
  }else{
    est.beta[['modJ']]<-numeric(0)
  }
    
  ##model individual
  for (j in c('1','2')){
    split.train <- eval(as.name(paste('split',j,sep='')))
    xx.train <- eval(as.name(paste('x',j,sep='')))[split.train,]
    yy.train <- eval(as.name(paste('y',j,sep='')))[split.train]
    xx.valid <- eval(as.name(paste('x',j,sep='')))[-split.train,]
    yy.valid <- eval(as.name(paste('y',j,sep='')))[-split.train]
    active[[paste('modIpop',j,sep='')]] <- eval(as.name(screen.meth))(xx.train,yy.train)

    if(length(active[[paste('modIpop',j,sep='')]])!=0){
      est.beta[[paste('modIpop',j,sep='')]] <- as.numeric(coef(lm(yy.valid~xx.valid[,active[[paste('modIpop',j,sep='')]]]-1)))
    }else{
      est.beta[[paste('modIpop',j,sep='')]]<-numeric(0)
    }
  }
  
  l.act <- lapply(active,length)
  n1.valid <- nrow(x1[-split1,])
  n2.valid <- nrow(x2[-split2,])
  if (any(l.act==0)){ warning('at least one active-set is empty')}
    
  if (all(l.act<c(n1.valid+n2.valid,n1.valid,n2.valid))){
      
    teststat <- logratio.diffregr(y1[-split1],y2[-split2],c(y1[-split1],y2[-split2]),
                         x1[-split1,active[['modIpop1']],drop=FALSE],x2[-split2,active[['modIpop2']],drop=FALSE],
                         rbind(x1[-split1,active[['modJ']],drop=FALSE],x2[-split2,active[['modJ']],drop=FALSE]),
                         est.beta[['modIpop1']],est.beta[['modIpop2']],est.beta[['modJ']])
      
    ev.nulldistr <- eval(as.name(compute.evals))(y1[-split1],y2[-split2],x1[-split1,],x2[-split2,],
                                                 est.beta[['modIpop1']],est.beta[['modIpop2']],est.beta[['modJ']],
                                                 active[['modIpop1']],active[['modIpop2']],active[['modJ']])$eval
    if (any(is.na(ev.nulldistr))){
      warning('Eigenval is NA: pval=NA')
    }else{
      sspval.onesided <- davies(teststat,lambda=ev.nulldistr)$Qq
      sspval.twosided <- 2*min(sspval.onesided,1-sspval.onesided)
    }
  }else{warning('dim(model) > n-1: pval=NA')}

    
  return(list(sspval.onesided=sspval.onesided,
              sspval.twosided=sspval.twosided,
              LR=teststat,active=active,beta=est.beta))
}

###########################################
##This an example of how to use MixGLasso##
###########################################

##generate data
set.seed(1)
n <- 1000
n.comp <- 3
p <- 10

# Create different mean vectors
Mu <- matrix(0,p,n.comp)

nonzero.mean <- split(sample(1:p),rep(1:n.comp,length=p))
for(k in 1:n.comp){
  Mu[nonzero.mean[[k]],k] <- -2/sqrt(ceiling(p/n.comp))
}

sim <- sim_mix_networks(n, p, n.comp, Mu=Mu)

##run mixglasso
set.seed(1)
fit1 <-  mixglasso(sim$data,n.comp=1:6)
fit1$bic
set.seed(1)
fit2 <-  mixglasso(sim$data,n.comp=6)
fit2$bic
set.seed(1)
fit3 <-  mixglasso(sim$data,n.comp=1:6,lambda=0)
set.seed(1)
fit4 <-  mixglasso(sim$data,n.comp=1:6,lambda=Inf)
#set.seed(1)
#fit5 <-  bwprun_mixglasso(sim$data,n.comp=1,n.comp.max=5,selection.crit='bic')
#plot(fit5$selcrit,ylab='bic',xlab='Num.Comps',type='b')

##compare bic
library('ggplot2')
plotting.frame <- 
  data.frame(BIC= c(fit1$bic, fit3$bic, fit4$bic), 
             Num.Comps=rep(1:6, 3), 
             Lambda=rep(c('Default', 
                          'Lambda = 0',
                          'Lambda = Inf'),
                        each=6))

p <- ggplot(plotting.frame) + 
	geom_line(aes(x=Num.Comps, y=BIC, colour=Lambda))

print(p)

############################################################
##This example illustrates the use of Differential Network##
############################################################


##set seed
set.seed(1)

##sample size and number of nodes
n <- 40
p <- 10

##specifiy sparse inverse covariance matrices
gen.net <- generate_2networks(p,graph='random',n.nz=rep(p,2),
                              n.nz.common=ceiling(p*0.8))
invcov1 <- gen.net[[1]]
invcov2 <- gen.net[[2]]
plot_2networks(invcov1,invcov2,label.pos=0,label.cex=0.7)

##get corresponding correlation matrices
cor1 <- cov2cor(solve(invcov1))
cor2 <- cov2cor(solve(invcov2))

##generate data under null hypothesis (both datasets have the same underlying
## network)
library('mvtnorm')
x1 <- rmvnorm(n,mean = rep(0,p), sigma = cor1)
x2 <- rmvnorm(n,mean = rep(0,p), sigma = cor1)

##run diffnet (under null hypothesis)
dn.null <- diffnet_multisplit(x1,x2,b.splits=1,verbose=FALSE)
dn.null$ss.pval#single-split p-value

##generate data under alternative hypothesis (datasets have different networks)
x1 <- rmvnorm(n,mean = rep(0,p), sigma = cor1)
x2 <- rmvnorm(n,mean = rep(0,p), sigma = cor2)

##run diffnet (under alternative hypothesis)
dn.altn <- diffnet_multisplit(x1,x2,b.splits=1,verbose=FALSE)
dn.altn$ss.pval#single-split p-value
dn.altn$medagg.pval#median aggregated p-value

##typically we would choose a larger number of splits
# dn.altn <- diffnet_multisplit(x1,x2,b.splits=10,verbose=FALSE)
# dn.altn$ms.pval#multi-split p-values
# dn.altn$medagg.pval#median aggregated p-value
# plot(dn.altn)#histogram of single-split p-values

#######################################################
##This example illustrates the use of GGMGSA         ##
#######################################################


## Generate networks
set.seed(1)
p <- 9#network with p nodes
n <- 40
hub.net <- generate_2networks(p,graph='hub',n.hub=3,n.hub.diff=1)#generate hub networks
invcov1 <- hub.net[[1]]
invcov2 <- hub.net[[2]]
plot_2networks(invcov1,invcov2,label.pos=0,label.cex=0.7)

## Generate data
library('mvtnorm')
x1 <- rmvnorm(n,mean = rep(0,p), sigma = cov2cor(solve(invcov1)))
x2 <- rmvnorm(n,mean = rep(0,p), sigma = cov2cor(solve(invcov2)))

## Run DiffNet
# fit.dn <- diffnet_multisplit(x1,x2,b.splits=2,verbose=FALSE)
# fit.dn$medagg.pval

## Identify hubs with 'gene-sets'
gene.names <- paste('G',1:p,sep='')
gsets <- split(gene.names,rep(1:3,each=3))

## Run GGM-GSA
fit.ggmgsa <- ggmgsa_multisplit(x1,x2,b.splits=2,gsets,gene.names,verbose=FALSE)
summary(fit.ggmgsa)
fit.ggmgsa$medagg.pval#median aggregated p-values
p.adjust(apply(fit.ggmgsa$pval,1,median),method='fdr')#or: first median aggregation,
                                                      #second fdr-correction

######################################################################
# Multivariate gene-set testing based on graphical models (GGM-GSA)
# -------------------------------------------------------------------
#
#
######################################################################


#####################
##Required packages##
#####################

library(GSA)
library('limma')
library('multtest')
library(ICSNP)
library(parallel)
library(mvtnorm)

######################
##P-value adjustment##
######################

##' P-value adjustment
##'
##' 
##' @title P-value adjustment
##' @param p Vector of p-values.
##' @param method Method for p-value adjustment (default='fdr').
##' @return Vector of adjusted p-values.
##' @author n.stadler
##' @keywords internal
my.p.adjust <- function(p,method='fdr'){
  if(method=='fdr'){
    return(p.adjust(p,method='fdr'))
  }
  if(method=='none'){
    return(p.adjust(p,method='none'))
  }
   if(method=='bonferroni'){
    return(p.adjust(p,method='bonferroni'))
  }
  #if(method=='qvalue'){
  #  return(qvalue(p)$qvalues)
  #}
}

##################################################
##Classical gene-set testing (Irizarry approach)##
##################################################

##' T-test (equal variances)
##'
##' 
##' @title T-test
##' @param x1 no descr
##' @param x2 no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
my.ttest <- function(x1,x2){
  ##equal variances
  s <- sqrt((var(x1)*(length(x1)-1)+var(x2)*(length(x2)-1))/(length(x1)+length(x2)-2))
  tstat <- (mean(x1)-mean(x2))/(s*sqrt(1/length(x1)+1/length(x2)))
  return(tstat)
}

##' T-test (unequal variances)
##'
##' 
##' @title T-test
##' @param x1 no descr
##' @param x2 no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
my.ttest2 <- function(x1,x2){
  ##unequal variances
  s <- sqrt((var(x1)/length(x1))+(var(x2)/length(x2)))
  tstat <- (mean(x1)-mean(x2))/s
  return(tstat)
}

##' Irizarry aggregate score (shift)
##'
##' 
##' @title Irizarry aggregate score (shift)
##' @param ttstat no descr
##' @param geneset no descr
##' @param gene.name no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
agg.score.iriz.shift <- function(ttstat,geneset,gene.name){
  genes.gs <- which(gene.name%in%geneset)
  ttstat <- ttstat[genes.gs]
  aggscore <- sqrt(length(genes.gs))*mean(ttstat)
  return(aggscore)
}

##' Irizarry aggregate score (scale)
##'
##' 
##' @title Irizarry aggregate score (scale)
##' @param ttstat no descr
##' @param geneset no descr
##' @param gene.name no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
agg.score.iriz.scale <- function(ttstat,geneset,gene.name){
  genes.gs <- which(gene.name%in%geneset)
  ttstat <- ttstat[genes.gs]
  aggscore <- (sum((ttstat-mean(ttstat))^2)-(length(genes.gs)-1))/(2*(length(genes.gs)-1))
  return(aggscore)
}

##' Irizarry approach (shift only)
##'
##' 
##' @title Irizarry approach (shift only)
##' @param x1 no descr
##' @param x2 no descr
##' @param gene.sets no descr
##' @param gene.names no descr
##' @param gs.names no descr
##' @param method.p.adjust no descr
##' @param alternative no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
gsea.iriz.shift <- function(x1,x2,gene.sets,gene.names,gs.names=NULL,method.p.adjust='fdr',alternative='two-sided'){
  
  no.genes <- ncol(x1)

  my.tstat <- sapply(1:no.genes,function(i){my.ttest2(x1[,i],x2[,i])})
  agg.scores <- sapply(gene.sets,function(x){agg.score.iriz.shift(my.tstat,x,gene.names)})
  if(alternative=='two-sided'){
    pvals <- my.p.adjust(2*(1-pnorm(abs(agg.scores))),method=method.p.adjust)
  }
  if(alternative=='one-sided'){
    pvals <- my.p.adjust(1-pnorm(agg.scores),method=method.p.adjust)
  }

  if(is.null(gs.names)){
    return(list(tstat=my.tstat,
                agg.scores.sort=agg.scores[order(pvals)],pvals.sort=pvals[order(pvals)],
                agg.scores=agg.scores,pvals=pvals))
  }else{
    return(list(tstat=my.tstat,
                agg.scores.sort=agg.scores[order(pvals)],pvals.sort=pvals[order(pvals)],
                gs.names.sort=gs.names[order(pvals)],
                agg.scores=agg.scores,pvals=pvals))
  }
}

##' Irizarry approach (scale only)
##'
##' 
##' @title Irizarry approach (scale only)
##' @param x1 no descr
##' @param x2 no descr
##' @param gene.sets no descr
##' @param gene.names no descr
##' @param gs.names no descr
##' @param method.p.adjust no descr
##' @param alternative no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
gsea.iriz.scale <- function(x1,x2,gene.sets,gene.names,gs.names=NULL,method.p.adjust='fdr',alternative='two-sided'){
  
  no.genes <- ncol(x1)

  my.tstat <- sapply(1:no.genes,function(i){my.ttest2(x1[,i],x2[,i])})
  agg.scores <- sapply(gene.sets,function(x){agg.score.iriz.scale(my.tstat,x,gene.names)})
  if(alternative=='two-sided'){
    pvals <- my.p.adjust(2*(1-pnorm(abs(agg.scores))),method=method.p.adjust)
  }
  if(alternative=='one-sided'){
    pvals <- my.p.adjust(1-pnorm(agg.scores),method=method.p.adjust)
  }

  if(is.null(gs.names)){
    return(list(tstat=my.tstat,
                agg.scores.sort=agg.scores[order(pvals)],pvals.sort=pvals[order(pvals)],
                agg.scores=agg.scores,pvals=pvals))
  }else{
    return(list(tstat=my.tstat,
                agg.scores.sort=agg.scores[order(pvals)],pvals.sort=pvals[order(pvals)],
                gs.names.sort=gs.names[order(pvals)],
                agg.scores=agg.scores,pvals=pvals))
  }
}

##' Irizarry approach for gene-set testing
##'
##' Implements the approach described in
##' "Gene set enrichment analysis made simple" by Irizarry et al (2011).
##' It tests for shift and/or change in scale of the distribution.
##' 
##' @title Irizarry approach for gene-set testing
##' @param x1 Expression matrix (condition 1)
##' @param x2 Expression matrix (condition 2)
##' @param gene.sets List of gene-sets
##' @param gene.names Gene names
##' @param gs.names Gene-set names
##' @param method.p.adjust Method for p-value adjustment (default='fdr')
##' @param alternative Default='two-sided' (uses two-sided p-values).
##' @return List consisting of
##' \item{pval.shift}{p-values measuring shift}
##' \item{pval.scale}{p-values measuring scale}
##' \item{pval.combined}{combined p-values (minimum of pval.shift and pval.scale)}
##' @author n.stadler
##' @export
##' @examples
##' n <- 100
##' p <- 20
##' x1 <- matrix(rnorm(n*p),n,p)
##' x2 <- matrix(rnorm(n*p),n,p)
##' gene.names <- paste('G',1:p,sep='')
##' gsets <- split(gene.names,rep(1:4,each=5))
##' fit <- gsea.iriz(x1,x2,gsets,gene.names)
##' fit$pvals.combined
##'
##' x2[,1:3] <- x2[,1:3]+0.5#variables 1-3 of first gene-set are upregulated
##' fit <- gsea.iriz(x1,x2,gsets,gene.names)
##' fit$pvals.combined
gsea.iriz <- function(x1,x2,gene.sets,gene.names,gs.names=NULL,method.p.adjust='fdr',alternative='two-sided'){

  fit.shift <- gsea.iriz.shift(x1,x2,gene.sets,gene.names,gs.names,method.p.adjust,alternative)
  fit.scale <- gsea.iriz.scale(x1,x2,gene.sets,gene.names,gs.names,method.p.adjust,alternative)
  pvals.shift <- fit.shift$pvals
  agg.scores.shift <- fit.shift$agg.scores
  pvals.scale <- fit.scale$pvals
  agg.scores.scale <- fit.scale$agg.scores

  return(list(pvals.shift=pvals.shift,agg.scores.shift=agg.scores.shift,
              pvals.scale=pvals.scale,agg.scores.scale=agg.scores.scale,
              pvals.combined=pmin(pvals.shift,pvals.scale)))
}


#######################
##Covariance T2-tests##
#######################

##' Classical likelihood-ratio test (equality of covariance matrices)
##'
##' 
##' @title Classical likelihood-ratio test
##' @param x1 no descr
##' @param x2 no descr
##' @param include.mean no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
t2cov.lr <- function(x1,x2,include.mean=FALSE){
  k <- ncol(x1)
  if(include.mean){
    lr.mle <-logratio(x1,x2,rbind(x1,x2),var(x1),var(x2),var(rbind(x1,x2)),mean(x1),mean(x2),mean(rbind(x1,x2)))$twiceLR
    twosamp.df <- k+k*(k+1)/2
    pval.mle <- 1-pchisq(lr.mle, df=twosamp.df)
  }else{
    lr.mle <-logratio(x1,x2,rbind(x1,x2),var(x1),var(x2),var(rbind(x1,x2)),rep(0,k),rep(0,k),rep(0,k))$twiceLR
    twosamp.df <- k*(k+1)/2
    pval.mle <- 1-pchisq(lr.mle, df=twosamp.df)
  }
  list(pval=pval.mle)
}

##' Diagonal-restricted likelihood-ratio test
##'
##' 
##' @title Diagonal-restricted likelihood-ratio test
##' @param x1 no descr
##' @param x2 no descr
##' @param include.mean no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
t2diagcov.lr <- function(x1,x2,include.mean=FALSE){
  k <- ncol(x1)
  if(include.mean){
    lr.mle <-logratio(x1,x2,rbind(x1,x2),diag(diag(var(x1))),diag(diag(var(x2))),diag(diag(var(rbind(x1,x2)))),
                      mean(x1),mean(x2),mean(rbind(x1,x2)))$twiceLR
    twosamp.df <- 2*k
    pval.mle <- 1-pchisq(lr.mle, df=twosamp.df)
  }else{
    lr.mle <-logratio(x1,x2,rbind(x1,x2),diag(diag(var(x1))),diag(diag(var(x2))),diag(diag(var(rbind(x1,x2)))),
                      rep(0,k),rep(0,k),rep(0,k))$twiceLR
    twosamp.df <- k
    pval.mle <- 1-pchisq(lr.mle, df=twosamp.df)
  }
  list(pval=pval.mle)
}

##' GSA using T2cov-test
##'
##' 
##' @title GSA using T2cov-test
##' @param x1 expression matrix (condition 1)
##' @param x2 expression matrix (condition 2)
##' @param gene.sets list of gene-sets
##' @param gene.names gene names
##' @param gs.names gene-set names
##' @param method method for testing equality of covariance matrices
##' @param method.p.adjust method for p-value adjustment (default: 'fdr')
##' @return list of results
##' @author n.stadler
##' @keywords internal
gsea.t2cov <- function(x1,x2,gene.sets,gene.names,gs.names=NULL,method='t2cov.lr',method.p.adjust='fdr'){
  
  
  pvals<- sapply(gene.sets,
                 function(y){
                   ind.genes <- which(gene.names%in%y)
                   eval(as.name(method))(x1[,ind.genes],x2[,ind.genes])$pval
                 })
  pvals.corrected <- my.p.adjust(pvals,method=method.p.adjust)
  if(is.null(gs.names)){
    return(list(pvals.sort=pvals.corrected[order(pvals.corrected)],pvals=pvals.corrected))
  }else{
    return(list(pvals.sort=pvals.corrected[order(pvals.corrected)],
                gs.names.sort=gs.names[order(pvals.corrected)],
                pvals=pvals.corrected))
  }
  
}

#################
##Mean T2-tests##
#################

##' HotellingsT2
##'
##' 
##' @title HotellingsT2
##' @param x1 no descr
##' @param x2 no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
test.t2 <- function(x1,x2){
  fit.t2 <- HotellingsT2(x1,x2, mu = NULL, test = "f")
  list(pval=fit.t2$p.value)
}

##' High-Dim Two-Sample Test (Srivastava, 2006)
##'
##' 
##' @title High-Dim Two-Sample Test (Srivastava, 2006)
##' @param x1 no descr
##' @param x2 no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
test.sd <- function(x1,x2){
  k <- ncol(x1)
  N1 <- nrow(x1)
  N2 <- nrow(x2)
  n <- N1+N2-2
  mu1 <- colMeans(x1)
  mu2 <- colMeans(x2)
  pooled.cov <-(var(x1)*(N1-1)+var(x2)*(N2-1))/n
  d.pooled.cov <- diag(pooled.cov)
  cormat <- diag(1/sqrt(d.pooled.cov))%*%pooled.cov%*%diag(1/sqrt(d.pooled.cov))
  c.const <- 1+tr(cormat%*%cormat)/(k^{3/2})
  teststat <- ((N1*N2)/(N1+N2))*crossprod(mu1-mu2,diag(1/d.pooled.cov))%*%(mu1-mu2)-(n*k/(n-2))
  teststat <- teststat/sqrt(2*(tr(cormat%*%cormat)-(k^2)/n)*c.const)
  
  list(teststat=teststat,pval=1-pnorm(teststat))
}

##' GSA based on HighdimT2
##'
##' 
##' @title GSA based on HighdimT2
##' @param x1 no descr
##' @param x2 no descr
##' @param gene.sets no descr
##' @param gene.names no descr
##' @param gs.names no descr
##' @param method no descr
##' @param method.p.adjust no descr
##' @return no descr
##' @author n.stadler
##' @keywords internal
gsea.highdimT2 <- function(x1,x2,gene.sets,gene.names,gs.names=NULL,method='test.sd',method.p.adjust='fdr'){
  
  
  pvals<- sapply(gene.sets,
                 function(y){
                   ind.genes <- which(gene.names%in%y)
                   eval(as.name(method))(x1[,ind.genes],x2[,ind.genes])$pval
                 })
  pvals.corrected <- my.p.adjust(pvals,method=method.p.adjust)
  if(is.null(gs.names)){
    return(list(pvals.sort=pvals.corrected[order(pvals.corrected)],pvals=pvals.corrected))
  }else{
    return(list(pvals.sort=pvals.corrected[order(pvals.corrected)],
                gs.names.sort=gs.names[order(pvals.corrected)],
                pvals=pvals.corrected))
  }
  
}

###########################################################
##------------GGMGSA---------------------------------------
###########################################################

##' Filter "non-normal" genes
##'
##' Discarding genes which have Shapiro-Wilk p-value (corrected for multiplicity)
##' smaller than sign.level in either of the two conditions. We used sign.level=0.001
##' in the GGMGSA paper.
##' 
##' @title Filter "non-normal" genes
##' @param x1 expression matrix (condition 1)
##' @param x2 expression matrix (condition 2)
##' @param sign.level sign.level in Shapiro-Wilk tests (default: sign.level=0.001)
##' @return list consisting of
##' \item{x1.filt}{expression matrix (condition 1) after filtering}
##' \item{x2.filt}{expression matrix (condition 2) after filtering}
##' @author n.stadler
##' @keywords internal
shapiro_screen <- function(x1,x2,sign.level=0.001){
  p <- ncol(x1)
  pval1 <- sapply(1:p,function(j){shapiro.test(x1[,j])$p.value})
  pval1.adj <- p.adjust(pval1,method='bonferroni')
  pval2 <- sapply(1:p,function(j){shapiro.test(x2[,j])$p.value})
  pval2.adj <- p.adjust(pval2,method='bonferroni')
  return(list(x1.filt=x1[,-which(pmin(pval1.adj,pval2.adj)<sign.level)],x2.filt=x2[,-which(pmin(pval1.adj,pval2.adj)<sign.level)]))
}

##' Meinshausen p-value aggregation.
##'
##' Inf-quantile formula for p-value aggregation presented in Meinshausen et al 2009.
##' 
##' @title Meinshausen p-value aggregation
##' @param pval Vector of p-values.
##' @param gamma.min See inf-quantile formula of Meinshausen et al 2009 (default=0.05).
##' @return Aggregated p-value.
##' @author n.stadler
##' @export
##' @examples
##' pval=runif(50)
##' aggpval(pval)
aggpval <- function(pval,gamma.min=0.05){
  
  min(1,(1-log(gamma.min))*optimize(
                                    f=function(gamma){
                                      min(quantile(pval[!is.na(pval)]/gamma,probs=gamma),1)
                                    }
                                    ,interval=c(gamma.min,1),maximum=FALSE)$objective)
}

##' Single-split GGMGSA
##'
##' 
##' @title Single-split GGMGSA
##' @param x1 centered (scaled) data for condition 1
##' @param x2 centered (scaled) data for condition 2
##' @param gene.sets List of gene-sets.
##' @param gene.names Gene names. Each column in x1 (and x2) corresponds
##'                   to a gene.   
##' @param method.p.adjust Method for p-value adjustment (default='fdr').
##' @param verbose If TRUE, show output progess.
##' @param ... Other arguments (see diffnet_singlesplit).
##' @return List of results.
##' @author n.stadler
ggmgsa_singlesplit <- function(x1,x2,gene.sets,gene.names,method.p.adjust='fdr',verbose=TRUE,...){
  n1 <- nrow(x1)
  n2 <- nrow(x2)
  split1 <- sample(1:n1,round(n1*0.5),replace=FALSE)
  split2 <- sample(1:n2,round(n2*0.5),replace=FALSE)
  res<- sapply(seq(length(gene.sets)),
                 function(i){
                   y <- gene.sets[[i]]
                   if(verbose){cat('  gene set:',i,'\n')}
                   ind.genes <- which(gene.names%in%y)
                   fit <- diffnet_singlesplit(x1[,ind.genes],x2[,ind.genes],split1,split2,...)
                   dfu <- length(fit$active[['modIpop1']])
                   dfv<- length(fit$active[['modIpop2']])
                   dfuv<- length(fit$active[['modJ']])
                   edge.intersect <- length(intersect(fit$active[['modIpop1']],fit$active[['modIpop2']]))/(length(union(fit$active[['modIpop1']],fit$active[['modIpop2']]))-length(ind.genes))
                   teststat.aic <- fit$teststat-2*(dfu+dfv-dfuv)
                   teststat.bic <- fit$teststat-log(0.5*(n1+n2))*(dfu+dfv-dfuv)
                   c(fit$pval.onesided,fit$teststat,teststat.bic,teststat.aic,edge.intersect,dfu,dfv,dfuv)
                 })
  rownames(res) <- c('pvals','teststat','teststat.bic','teststat.aic','rel.edgeinter','dfu','dfv','dfuv')
  pvals.corrected <- my.p.adjust(res['pvals',],method=method.p.adjust)
  return(list(pvals=pvals.corrected,teststat=res['teststat',],teststat.bic=res['teststat.bic',],teststat.aic=res['teststat.aic',],rel.edgeinter=res['rel.edgeinter',],dfu=res['dfu',],dfv=res['dfv',],dfuv=res['dfuv',]))
}

##' Multi-split GGMGSA (parallelized computation)
##'
##' Computation can be parallelized over many data splits.
##' 
##' @title Multi-split GGMGSA (parallelized computation)
##' @param x1 Expression matrix for condition 1 (mean zero is required).
##' @param x2 Expression matrix for condition 2 (mean zero is required).
##' @param b.splits Number of random data splits (default=50).
##' @param gene.sets List of gene-sets.
##' @param gene.names Gene names. Each column in x1 (and x2) corresponds
##'                   to a gene. 
##' @param gs.names Gene-set names (default=NULL).
##' @param method.p.adjust Method for p-value adjustment (default='fdr').
##' @param order.adj.agg Order of aggregation and adjustment of p-values.
##'                      Options: 'agg-adj' (default), 'adj-agg'.
##' @param mc.flag If \code{TRUE} use parallel execution for each b.splits via function 
##'                \code{mclapply} of package \code{parallel}.
##' @param mc.set.seed See mclapply. Default=TRUE
##' @param mc.preschedule See mclapply. Default=TRUE
##' @param mc.cores Number of cores to use in parallel execution. Defaults to
##'                 mc.cores option if set, or 2 otherwise.
##' @param verbose If TRUE, show output progess.
##' @param ... Other arguments (see diffnet_singlesplit).
##' @return List consisting of
##' \item{medagg.pval}{Median aggregated p-values}
##' \item{meinshagg.pval}{Meinshausen aggregated p-values}
##' \item{pval}{matrix of p-values before correction and adjustement, dim(pval)=(number of gene-sets)x(number of splits)}
##' \item{teststatmed}{median aggregated test-statistic}
##' \item{teststatmed.bic}{median aggregated bic-corrected test-statistic}
##' \item{teststatmed.aic}{median aggregated aic-corrected test-statistic}
##' \item{teststat}{matrix of test-statistics, dim(teststat)=(number of gene-sets)x(number of splits)}
##' \item{rel.edgeinter}{normalized intersection of edges in condition 1 and 2}
##' \item{df1}{degrees of freedom of GGM obtained from condition 1}
##' \item{df2}{degrees of freedom of GGM obtained from condition 2}
##' \item{df12}{degrees of freedom of GGM obtained from pooled data (condition 1 and 2)}
##' @author n.stadler
##' @export
##' @example tests/Examples/ggmgsa_ex.R
ggmgsa_multisplit<- function(x1,x2,b.splits=50,gene.sets,gene.names,gs.names=NULL,
                                 method.p.adjust='fdr',order.adj.agg='agg-adj',
                                 mc.flag=FALSE,mc.set.seed=TRUE,mc.preschedule=TRUE,mc.cores=getOption("mc.cores", 2L),verbose=TRUE,...){

    if(is.null(gs.names)){
        gs.names <- paste('gs',1:length(gene.sets),sep='')
    }

    if(mc.flag && .Platform$OS.type == "unix"){
        res <- mclapply(seq(b.splits),
                        function(i){
                            if(verbose){cat('\n split: ',i,'\n\n')}
                            res <- ggmgsa_singlesplit(x1,x2,gene.sets=gene.sets,gene.names=gene.names,
                                                      method.p.adjust='none',verbose=verbose,...)

                            if(verbose){cat('  pvals: ',res$pvals,'\n')}
                            mat <- cbind(res$pvals,res$teststat,res$teststat.bic,res$teststat.aic,res$rel.edgeinter,res$dfu,res$dfv,res$dfuv)
                            colnames(mat) <- c('pvals','teststat','teststat.bic','teststat.aic','rel.edgeinter','dfu','dfv','dfuv')
                            return(mat)
                        }, mc.set.seed=mc.set.seed, mc.preschedule = mc.preschedule,mc.cores=mc.cores)
    }else{
    	  if(mc.flag) warning('Windows does not support parallelisation via mclapply, using sequential lapply instead.')
        res <- lapply(seq(b.splits),
                      function(i){
                          if(verbose){cat('\n split: ',i,'\n\n')}
                          res <- ggmgsa_singlesplit(x1,x2,gene.sets=gene.sets,gene.names=gene.names,
                                                    method.p.adjust='none',verbose=verbose,...)
                          if(verbose){cat('  p-values: ',res$pvals,'\n\n')}
                          mat <- cbind(res$pvals,res$teststat,res$teststat.bic,res$teststat.aic,res$rel.edgeinter,res$dfu,res$dfv,res$dfuv)
                          colnames(mat) <- c('pvals','teststat','teststat.bic','teststat.aic','rel.edgeinter','dfu','dfv','dfuv')
                          return(mat)
                      }
                      )
    }
    
    res.pval <- sapply(seq(b.splits),function(i){res[[i]][,'pvals']})
    res.teststat <- sapply(seq(b.splits),function(i){res[[i]][,'teststat']})
    res.teststat.bic <- sapply(seq(b.splits),function(i){res[[i]][,'teststat.bic']})
    res.teststat.aic <- sapply(seq(b.splits),function(i){res[[i]][,'teststat.aic']})
    res.rel.edgeinter <- sapply(seq(b.splits),function(i){res[[i]][,'rel.edgeinter']})
    res.dfu <- sapply(seq(b.splits),function(i){res[[i]][,'dfu']})
    res.dfv <- sapply(seq(b.splits),function(i){res[[i]][,'dfv']})
    res.dfuv <- sapply(seq(b.splits),function(i){res[[i]][,'dfuv']})

    if(order.adj.agg=='agg-adj'){
        pvalagg <- my.p.adjust(apply(res.pval,1,aggpval),method=method.p.adjust)
        pvalmed <- my.p.adjust(apply(res.pval,1,median,na.rm=TRUE),method=method.p.adjust)
    }
    if(order.adj.agg=='adj-agg'){
        pvalagg <- apply(apply(res.pval,2,my.p.adjust,method=method.p.adjust),1,aggpval)
        pvalmed <- apply(apply(res.pval,2,my.p.adjust,method=method.p.adjust),1,median)
    }

    out <- list(medagg.pval=pvalmed,meinshagg.pval=pvalagg,
                pval=res.pval,
                teststatmed=apply(res.teststat,1,median,na.rm=TRUE),
                teststatmed.bic=apply(res.teststat.bic,1,median,na.rm=TRUE),
                teststatmed.aic=apply(res.teststat.aic,1,median,na.rm=TRUE),
                teststat=res.teststat,teststat.aic=res.teststat.aic,
                rel.edgeinter=res.rel.edgeinter,df1=res.dfu,df2=res.dfv,df12=res.dfuv,
                gs.names=gs.names)
    class(out) <- 'ggmgsa'
    return(out)
  
}

##Packages
library(mvtnorm)
library(glasso)
library(huge)


#' Cross-validated glasso on heterogeneous dataset with grouping
#' 
#'
#' Run glasso on a heterogeneous dataset to obtain networks (inverse covariance 
#' matrices) of the variables in the dataset for each pre-specified group of 
#' samples.
#'
#' This function runs the graphical lasso with 
#' cross-validation to determine the best parameter lambda for each group of 
#' samples. Note that this function defaults to using package huge (rather than
#' package glasso) unless otherwise specified, as it tends to be more 
#' numerically stable.
#'
#'  
#' @param data The heterogenous network data. Needs to be 
#' a num.samples by dim.samples matrix or dataframe.
#' @param grouping The grouping of samples; a vector of length num.samples,
#' with num.groups unique elements.
#' @param mc.flag Whether to use parallel processing via package mclapply to
#' distribute the glasso estimation over different groups.
#' @param use.package 'glasso' for glasso package, or 'huge' for huge package 
#' (default)
#' @param normalise If TRUE, normalise the columns of the data matrix before 
#' running glasso.
#' @param verbose If TRUE, output progress.
#' @param ... Further parameters to be passed to \code{screen_cv.glasso}.
#' @export
#' @return Returns a list with named elements 'Sig', 'SigInv', 'Mu', 'Sigma.diag', 
#' 'group.names' and 'var.names. 
#' The variables Sig and SigInv are arrays of size dim.samples by dim.samples 
#' by num.groups, where the first two dimensions contain the (inverse)
#' covariance matrix for the network obtained by running glasso on group k. Variables 
#' Mu and Sigma.diag contain the mean and variance of the input data,
#' and group.names and var.names contains the names for the groups and
#' variables in the data (if specified as colnames of the input data matrix).
#' @examples
#' n = 100
#' p = 25
#'
#' # Generate networks with random means and covariances. 
#' sim.result = sim_mix_networks(n, p, n.comp=3)
#'
#' test.data = sim.result$data
#' test.labels = sim.result$comp
#'
#' # Reconstruct networks for each component
#' networks = het_cv_glasso(data=test.data, grouping=test.labels) 
het_cv_glasso <- function(data, grouping=rep(1, dim(data)[1]), mc.flag=FALSE,
                          use.package='huge', normalise=FALSE, verbose=FALSE, ...) {
  
  group.names = sort(unique(grouping))
  
  mu = matrix(0, dim(data)[2], length(group.names))
  Sigma.diag = matrix(0, dim(data)[2], length(group.names))
  
  data.list = list()
  
  # Scale data if necessary and split into list
  for(group.i in 1:length(group.names)) {
    group.name = group.names[group.i]
    group.data = data[grouping==group.name,]
    
    scaled.group.data = scale(group.data)
    
    data.list[[group.i]] =  if(normalise) scaled.group.data
                            else group.data 
      
    mu[,group.i] = attributes(scaled.group.data)[['scaled:center']]
    Sigma.diag[,group.i] = attributes(scaled.group.data)[['scaled:scale']]
    
    rownames(mu) = colnames(group.data)
    colnames(mu) = group.names
    rownames(Sigma.diag) = colnames(group.data)
    colnames(Sigma.diag) = group.names
  }
  
  # Run glasso on each group
  if(mc.flag && .Platform$OS.type == "unix") {
    results = mclapply(data.list, screen_cv.glasso, use.package=use.package, 
                       verbose=verbose, include.mean=TRUE, trunc.method='none', ...)
  } else {
  	if(mc.flag) warning('Windows does not support parallelisation via mclapply, using sequential lapply instead.')
    results = lapply(data.list, screen_cv.glasso, use.package=use.package, 
                     verbose=verbose, include.mean=TRUE, trunc.method='none', ...)
  }
  
  
  # Collect results
  results.all = list()
  
  wi = array(0, dim=c(dim(data)[2], dim(data)[2], length(group.names)))
  w = array(0, dim=c(dim(data)[2], dim(data)[2], length(group.names)))
      
  for(group.i in 1:length(group.names)) {
    wi[,,group.i] = results[[group.i]]$wi
    w[,,group.i] = solve(wi[,,group.i])
  }
      
  results.all$SigInv = wi
  results.all$Sig = w
  results.all$Mu = mu
  results.all$Sigma.diag = Sigma.diag
  results.all$group.names = group.names
  results.all$proteins = colnames(data)
  results.all$comp = grouping # For consistency with mixglasso
  
  class(results.all) = 'nethetclustering'
  
  return(results.all)    
  
}

# Code for plotting networks
library(ggplot2)
library(network)
library(mclust)
library(mvtnorm)


# Suppress check warning
if(getRversion() >= '2.15.1')  
	utils::globalVariables(c('P.Corr', 'Edge', 'Mean', 'Type', 'Node.1', 
													 'Node.2', 'Group', 'Edge.Name', 'Corr'))


#' Build up dataframe for plotting dot plot with ggplot2
#' 
#' Internal function
#' 
#' @param net.clustering Clustering
#' @param cluster.names Cluster names
#' @param node.names Node names
#' @return A data frame for plotting the dotPlot with ggplot2 is returned. 
#' Column P.Corr contains the partial correlations of each edge as a numeric, 
#' column Mean contains the minimum mean expression of the two proteins
#' (e.g. if the edge is e(p1, p2), then the column contains min(mean(p1), 
#' mean(p2))), column Edge contains the name of the edge as a character
#' string of the form "p1-p2" and column Type contains the cluster name of the 
#' cluster that the edge belongs to as a character string.
#' @keywords internal
#' 
buildDotPlotDataFrame <- function(net.clustering, cluster.names, node.names) {
	results.frame = data.frame()
	
	for(cluster.i in 1:length(cluster.names)) {
		cluster.name = as.character(cluster.names[cluster.i])
		
		# Calculate partial correlations
		p.corrs = invcov2parcor(net.clustering$SigInv[,,cluster.i])
		
		Mu = net.clustering$Mu[,cluster.i]
		
		mean.edges = c()
		p.corr.edges = c()
		edge.names = c()
		
		# Record pcorr and means for all edges
		for(node.i in 1:(length(node.names) - 1)) {
			for(node.j in (node.i+1):length(node.names)) {
				
				if(abs(p.corrs[node.i, node.j]) >= 0) {
					mean.edges = c(mean.edges, min(Mu[node.i], Mu[node.j]))
					
					p.corr.edges = c(p.corr.edges, p.corrs[node.i, node.j])
					
					edge.name = paste(node.names[node.i], 
														node.names[node.j], sep='-')
					edge.names = c(edge.names, edge.name)
				}
			}
		}
		
		results.frame = rbind(results.frame, 
													data.frame(P.Corr=p.corr.edges,
																		 Mean=mean.edges,
																		 Edge=edge.names,
																		 Type=cluster.name))
	}
	
	return(results.frame)
	
}


#' Create a plot showing the edges with the highest partial correlation in any cluster.
#' 
#' This function takes the output of \code{\link{het_cv_glasso}} or 
#' \code{\link{mixglasso}} and creates a plot of the highest scoring edges along the
#' y axis, where, the edge in each cluster is represented by a circle whose area
#' is proportional to the smallest mean of the two nodes that make up the edge,
#' and the position along the y axis shows the partial correlation of the edge.
#'
#' @param net.clustering A network clustering object as returned by 
#' \code{\link{het_cv_glasso}} or \code{\link{mixglasso}}.
#' @param p.corrs.thresh Cutoff for the partial correlations; only edges with absolute 
#' partial correlation > p.corrs.thresh (in any cluster) will be displayed. 
#' @param hard.limit Additional hard limit on the number of edges to display. If 
#' p.corrs.thresh results in more edges than hard.limit, only hard.limit edges with the
#' highest partial correlation are returned. 
#' @param display If TRUE, print the plot to the current output device.
#' @param node.names Names for the nodes in the network.
#' @param group.names Names for the clusters or groups.
#' @param dot.size.range Graphical parameter for scaling the size of the circles (dots)
#' representing an edge in each cluster.
#' @export
#' @import ggplot2
#' @return Returns a ggplot2 object. If display=TRUE, additionally displays the 
#' plot.
#' @examples
#' n = 500
#' p = 10
#' s = 0.9
#' n.comp = 3
#'
#' # Create different mean vectors
#' Mu = matrix(0,p,n.comp)
#'
#' # Define non-zero means in each group (non-overlapping)
#' nonzero.mean = split(sample(1:p),rep(1:n.comp,length=p))
#'
#' # Set non-zero means to fixed value
#' for(k in 1:n.comp){
#' 	Mu[nonzero.mean[[k]],k] = -2/sqrt(ceiling(p/n.comp))
#' }
#'
#' # Generate data
#' sim.result = sim_mix_networks(n, p, n.comp, s, Mu=Mu)
#' mixglasso.result = mixglasso(sim.result$data, n.comp=3)
#' mixglasso.clustering = mixglasso.result$models[[mixglasso.result$bic.opt]]
#' 
#' dot_plot(mixglasso.clustering, p.corrs.thresh=0.5)
dot_plot <- function(net.clustering, p.corrs.thresh=0.25, hard.limit=50,
										display=TRUE, node.names=rownames(net.clustering$Mu),
										group.names=sort(unique(net.clustering$comp)),
										dot.size.range=c(3,12)) {
	
	if(is.null(node.names)) node.names = 1:dim(net.clustering$Mu)[1]
	
  cluster.assignments = net.clustering$comp
	
  results.frame = buildDotPlotDataFrame(net.clustering, group.names, 
  																			node.names)
  
  # Determine which edges to include
	biggest.pcorr = results.frame[abs(results.frame$P.Corr) > p.corrs.thresh,]  
  
  # If any edges above threshold
  if(dim(biggest.pcorr)[1] > 0) {
  	
  	interesting.edges = data.frame()
  	
  	# Find edges with maximum pcorr
  	for(edge in unique(biggest.pcorr$Edge)) {
  		edge.pcorr = biggest.pcorr[biggest.pcorr$Edge==edge,];
  		edge.max = edge.pcorr[which.max(abs(edge.pcorr$P.Corr)),];
  		interesting.edges = rbind(interesting.edges, edge.max)
  	}
  	  
    # Sorted by max p.corr
    interesting.edges = 
  	  interesting.edges[sort(abs(interesting.edges$P.Corr), index.return=TRUE)$ix,]
  
    # Enforce hard limit
    if(dim(interesting.edges)[1] > hard.limit) {
  	  interesting.edges = interesting.edges[1:hard.limit,]
    }
	
	  results.frame.reduced = results.frame[results.frame$Edge %in% interesting.edges$Edge,]
	  results.frame.reduced$Abs.P.Corr = abs(results.frame.reduced$P.Corr)
	
    # Ensure right order
	  results.frame.reduced$Edge = factor(results.frame.reduced$Edge, levels=interesting.edges$Edge)
  	
	  g <-  ggplot(results.frame.reduced, aes(x=P.Corr, y=Edge, size=Mean, colour=Type)) +
		  geom_point() +
  		scale_size(range=dot.size.range) +
		  theme(axis.title.y=element_blank())
	
	  if(display) print(g)
	
	  return(g)
	  
  } else {
  	warning('No edges with partial correlation above threshold.')
  	return(NULL)
  }
}


#' Create a scatterplot showing correlation between specific nodes in the network
#' for each pre-specified group.
#' 
#' This function takes the output of \code{\link{het_cv_glasso}} or 
#' \code{\link{mixglasso}} and creates a plot showing the correlation between specified
#' node pairs in the network for all groups. The subplots for each node pair are 
#' arranged in a numPairs by numGroups grid. Partial correlations associated 
#' with each node pair are also displayed.
#'
#' @param net.clustering A network clustering object as returned by 
#' \code{\link{het_cv_glasso}} or \code{\link{mixglasso}}.
#' @param data Observed data for the nodes, a numObs by numNodes matrix. Note 
#' that nodes need to be in the same ordering as in node.names.
#' @param node.pairs A matrix of size numPairs by 2, where each row contains a 
#' pair of nodes to display. If node.names is specified, names in node.pairs
#' must correspond to elements of node.names.
#' @param display If TRUE, print the plot to the current output device.
#' @param node.names Names for the nodes in the network. If NULL, names from 
#' net.clustering will be used.
#' @param group.names Names for the clusters or groups. If NULL, names from 
#' net.clustering will be used (by default these are integets 1:numClusters).
#' @param cex Scale factor for text and symbols in plot.
#' @export
#' @import ggplot2
#' @return Returns a ggplot2 object. If display=TRUE, additionally displays the 
#' plot.
#' @examples
#' n = 500
#' p = 10
#' s = 0.9
#' n.comp = 3
#'
#' # Create different mean vectors
#' Mu = matrix(0,p,n.comp)
#'
#' # Define non-zero means in each group (non-overlapping)
#' nonzero.mean = split(sample(1:p),rep(1:n.comp,length=p))
#'
#' # Set non-zero means to fixed value
#' for(k in 1:n.comp){
#' 	Mu[nonzero.mean[[k]],k] = -2/sqrt(ceiling(p/n.comp))
#' }
#'
#' # Generate data
#' sim.result = sim_mix_networks(n, p, n.comp, s, Mu=Mu)
#' mixglasso.result = mixglasso(sim.result$data, n.comp=3)
#' mixglasso.clustering = mixglasso.result$models[[mixglasso.result$bic.opt]]
#' 
#' # Specify edges
#' node.pairs = rbind(c(1,3), c(6,9),c(7,8))
#' 
#' # Create scatter plots of specified edges
#' scatter_plot(mixglasso.clustering, data=sim.result$data,
#'						 node.pairs=node.pairs)
scatter_plot <- function(net.clustering, data, node.pairs, display=TRUE, 
												node.names=rownames(net.clustering$Mu),
												group.names=sort(unique(net.clustering$comp)),
												cex=1) {
	
	if(class(net.clustering) != 'nethetclustering') 
		 stop('net.clustering needs to be an object of class nethetclustering')
	
	if(is.null(node.names)) node.names = 1:dim(net.clustering$Mu)[1]
	
	# Prepare data frame for plotting
	plotting.frame = data.frame()
	corr.frame = data.frame()
	
	num.samples = dim(data)[1]
	
	# Obtain partial correlations from graphical lasso output
	p.corrs = invcov2parcor_array(net.clustering$SigInv)
	
	# Cycle over node pairs/edges
	for(edge.i in 1:dim(node.pairs)[1]) {
		node.1 = node.pairs[edge.i, 1]
		node.2 = node.pairs[edge.i, 2]
		edge.name = paste(node.1, node.2, sep='\n')
					
		# Partial correlation
		p.corrs.values = p.corrs[node.names == node.1, 
												   node.names == node.2,]
		
		names(p.corrs.values) = group.names
		
		# Spearman correlation
		corrs = sapply(group.names, 
									 function(x) cor(data[net.clustering$comp == x,node.names==node.1],
		 														   data[net.clustering$comp == x,node.names==node.2]))
		
		corr.frame = rbind(corr.frame,
											 data.frame(Edge.Name=rep(edge.name, length(group.names)), 
											 					 Group=factor(group.names),
											 					 Corr=round(corrs, digits=2),
											 					 P.Corr=round(p.corrs.values, digits=2)))
		
		temp.frame = data.frame(Edge.Name=rep(edge.name, num.samples), 
														Node.1=data[,node.1], 
														Node.2=data[,node.2], 
														Group=factor(net.clustering$comp))
		
		plotting.frame = rbind(temp.frame, plotting.frame)
	}
	
	p <- ggplot(plotting.frame, aes(Node.1, Node.2)) +
		geom_point(aes(colour=Group), size=2*cex) +		
		geom_abline(linetype=3) + 
		facet_grid(Edge.Name ~ Group) +
		geom_text(data=corr.frame, aes(x=Inf, y=Inf, label=paste('PCorr: ', P.Corr, '\n', 'Corr: ', Corr, sep='')), size=6*cex, family='Times', hjust=1, vjust=1) +
		theme(strip.text.y = element_text(size=cex*15), strip.text.x = element_text(size=cex*20),
					axis.text=element_text(size=cex*18), axis.title=element_text(size=cex*24,face="bold"))
	
	if(display) print(p)
	
	return(p)
}

#' Plot networks
#' 
#' This function takes the output of \code{\link{screen_cv.glasso}} or 
#' \code{\link{mixglasso}} and creates a network plot using the network library.
#' 
#'
#' @param x A network clustering object as returned by 
#' \code{\link{screen_cv.glasso}} or \code{\link{mixglasso}}.
#' @param node.names Names for the nodes in the network. If NULL, names from 
#' net.clustering will be used.
#' @param group.names Names for the clusters or groups. If NULL, names from 
#' net.clustering will be used (by default these are integets 1:numClusters).
#' @param p.corrs.thresh Threshold applied to the absolute partial correlations. 
#' Edges that are below the threshold in all of the groups are not displayed.
#' @param print.pdf If TRUE, save the output as a PDF file.
#' @param pdf.filename If \code{print.pdf} is TRUE, specifies the file name of
#' the output PDF file.
#' @param ... Further arguments
#' @return Returns NULL and prints out the networks (or saves them to pdf if
#' \code{print.pdf} is TRUE. The networks are displayed as a series of nComps+1
#' plots, where in the first plot edge widths are shown according to 
#' the maximum partial correlation of the edge over all groups. The following plots
#' show the edges for each group. Positive partial correlation edges are shown in
#' black, negative ones in blue. If an edge is below the threshold on the absolute
#' partial correlation, it is displayed in gray or light blue respectively.
#' @method plot nethetclustering
#' @export 
#' @import network
plot.nethetclustering <- function(x, 
																	node.names=rownames(net.clustering$Mu),
																	group.names=sort(unique(net.clustering$comp)),
																	p.corrs.thresh=0.2, print.pdf=FALSE, 
																	pdf.filename='networks', ...){
  
	net.clustering = x
	
	if(is.null(node.names)) node.names = 1:dim(net.clustering$Mu)[1]
	
	p.corrs = invcov2parcor_array(net.clustering$SigInv)
	
	k = ncol(net.clustering$Mu)
	
	#combine parcors for all k states by taking max parcor values:
	PCmax = apply(abs(p.corrs), 1:2, max, na.rm=TRUE)
	diag(PCmax) = NA
	
	#plot network:
	if(print.pdf){
		pdf(paste(pdf.filename,'.pdf',sep=''), width=14, height=14, onefile=TRUE)
	}
	
	#create and plot network object for combined states:
	NWbin = PCmax>p.corrs.thresh
	diag(NWbin) = FALSE
	Network = network(NWbin)
	edgeWeight = ((PCmax-p.corrs.thresh)/max(PCmax-p.corrs.thresh,na.rm=TRUE)+0.1)*15
	
	Coord = plot.network(Network, 
											main="combined states",
											#coord=as.matrix(L[,2:3]),
											vertex.col="orange", 
											vertex.cex=1.2,#0.5-min(mean),
											vertex.rot=360/16, 
											vertex.border=grey(0.8),
											displaylabels=TRUE,
											boxed.labels=FALSE,
											label=node.names,
											label.pos=5,
											label.cex=0.7,
											edge.lwd=edgeWeight, 
											usearrows=FALSE, 
											pad=0.02)
	
	edge.sum = 0
	
	#now the same for each of the k states:
	for(i in 1:k) { 
		PC = abs(p.corrs[,,i])
	  NWbin = abs(PC)>0 & PCmax>p.corrs.thresh
	  diag(NWbin) = FALSE
	  Network = network(NWbin) 
	  #edgeWeight<-((PC-p.corrs.thresh)/max(PCmax-p.corrs.thresh,na.rm=TRUE)+0.1)*15
	  #cat('\n', group.names[i], '\n')
	  PC.temp = PC
	  diag(PC.temp) = 0
	  PC.temp[lower.tri(PC.temp)] = 0
	  indices = which(PC.temp >= p.corrs.thresh, arr.ind=TRUE)
	  #cat(paste(node.names[indices[,1]], node.names[indices[,2]], sep='-'), sep='\n')
	  edge.sum = edge.sum + sum(PC.temp >= p.corrs.thresh)
	 
 	  weights.temp = 6*(PC/max(PCmax,na.rm=TRUE))
	  #edgeWeight = (weights.temp + 0.1)*(15/6)
	  edgeWeight = (((1/(1+exp(-weights.temp))) - 0.5)*1.8 + 0.1)*12
	 
	  edge.col = matrix('black', dim(PC)[1], dim(PC)[2])
	  edge.col[p.corrs[,,i]<0] = 'blue'
	  edge.col[p.corrs[,,i] < p.corrs.thresh & p.corrs[,,i] > 0] = 'grey'
	  edge.col[p.corrs[,,i] > -p.corrs.thresh & p.corrs[,,i] < 0] = 'skyblue'
	 
	  plot.network(Network, 
	 						 main=group.names[i],
	 						 coord=Coord,
	 						 vertex.col="orange", 
	 						 vertex.cex=1.2,#0.5+mean[,i]-min(mean),
	 						 vertex.rot=360/16, 
	 						 vertex.border=grey(0.8),
	 						 displaylabels=TRUE,
	 						 boxed.labels=FALSE,
	 						 label=node.names,
	 						 label.pos=5,
	 						 label.cex=0.7,
	 						 #label.cex=0.3*(2^mean[,i]),
	 						 edge.lwd=edgeWeight, 
	 						 usearrows=FALSE, 
	 						 pad=0.02,
	 						 edge.col=edge.col)	 
	 
	  legend('bottomleft', legend=c('Positive PCorr',
	 										 'Negative PCorr',
	 										 'Pos PCorr < Threshold',
	 										 'Neg PCorr > -Threshold'), 
	 			 col=c('black', 'blue', 'grey', 'skyblue'),
	 			 lty=1, cex=0.75,
	 			 lwd=3,
	 			 xpd=TRUE)
	 
	}
	if(print.pdf){
		dev.off() #close PDF
	}
	
}



##' Plot two networks (GGMs)
##'
##' 
##' @title Plot two networks (GGMs)
##' @param invcov1 Inverse covariance matrix of GGM1.
##' @param invcov2 Inverse covariance matrix of GGM2.
##' @param node.label Names of nodes.
##' @param main Vector (two elements) with network names.
##' @param ... Other arguments (see plot.network).
##' @return Figure with two panels (for each network).
##' @author nicolas
##' @export
##' @examples
##' n <- 70
##' p <- 30
##' 
##' ## Specifiy sparse inverse covariance matrices,
##' ## with number of edges in common equal to ~ 0.8*p
##' gen.net <- generate_2networks(p,graph='random',n.nz=rep(p,2),
##'                               n.nz.common=ceiling(p*0.8))
##' 
##' invcov1 <- gen.net[[1]]
##' invcov2 <- gen.net[[2]]
##' 
##' plot_2networks(invcov1,invcov2,label.pos=0,label.cex=0.7)
plot_2networks <- function(invcov1,invcov2,
													 node.label=paste('X',1:nrow(invcov1),sep=''),
													 main=c('',''),...){
	par.orig <- par(no.readonly=TRUE)
	par(mfrow=c(1,2),mar=c(1,1,1,1))
	adj1 <- invcov1!=0
	adj2 <- invcov2!=0
	adj <- list(adj1,adj2)
	nonzeros <- sapply(adj,sum)
	
	if(nonzeros[1]!=nonzeros[2]){
		coord <- plot.network(network(adj[[which.max(nonzeros)]]),
													main=main[which.max(nonzeros)],
													displaylabels=TRUE,
													label=node.label,
													usearrows=FALSE,...)
		plot.network(network(adj[[which.min(nonzeros)]]),
								 coord=coord,
								 main=main[which.min(nonzeros)],
								 displaylabels=TRUE,
								 label=node.label,
								 usearrows=FALSE,...)
	}
	
	if(nonzeros[1]==nonzeros[2]){
		coord <- plot.network(network(adj[[1]]),
													main=main[1],
													displaylabels=TRUE,
													label=node.label,
													usearrows=FALSE,...)
		plot.network(network(adj[[2]]),
								 coord=coord,
								 main=main[2],
								 displaylabels=TRUE,
								 label=node.label,
								 usearrows=FALSE,...)
	}
	par(par.orig)
}

##' Plotting function for object of class 'diffnet' 
##'
##' 
##' @title Plotting function for object of class 'diffnet' 
##' @param x object of class 'diffnet'
##' @param ... Further arguments.
##' @return Histogram over multi-split p-values.
##' @author nicolas
##' @method plot diffnet
##' @export 
plot.diffnet <- function(x,...){
	#if(is.null(x$medwi)){
	hh <- hist(x$ms.pval,
						 main='histogram single-split p-values',xlab='p-values',ylab='frequency',...)
	abline(v=x$medagg.pval,lty=2,col='red')
	abline(v=x$meinshagg.pval,lty=2,col='green')
	legend(x=min(hh$mids),y=max(hh$counts),lty=c(2,2),col=c('red','green'),legend=c('median aggregated','meinshausen aggregated'))
	#}else{
	#    medwi <- x$medwi
	#    k <- ncol(x$medwi[[1]])
	#    par(mfrow=c(2,2))
	#    image(x=1:k,y=1:k,abs(medwi$modIpop1),xlab='',ylab='',main='median invcov1')
	#    image(x=1:k,y=1:k,abs(medwi$modIpop2),xlab='',ylab='',main='median invcov2')
	#    hist(x$ms.pval,breaks=10,
	#         main='histogram single-split p-values',xlab='p-values',ylab='frequency')
	#    abline(v=x$medagg.pval,lty=2,col='red')
	#}
}

##' Plotting function for object of class 'diffregr' 
##'
##' 
##' @title Plotting function for object of class 'diffregr' 
##' @param x object of class 'diffregr'
##' @param ... Further arguments.
##' @return Histogram over multi-split p-values.
##' @author nicolas
##' @method plot diffregr
##' @export
plot.diffregr <- function(x,...){
	hh <- hist(x$ms.pval,
						 main='histogram single-split p-values',xlab='p-values',ylab='frequency',...)
	abline(v=x$medagg.pval,lty=2,col='red')
	abline(v=x$meinshagg.pval,lty=2,col='green')
	legend(x=min(hh$mids),y=max(hh$counts),lty=c(2,2),col=c('red','green'),legend=c('median aggregated','meinshausen aggregated'))
	
}

##' Plotting function for object of class 'ggmgsa' 
##'
##' 
##' @title Plotting function for object of class 'ggmgmsa' 
##' @param x object of class 'ggmgsa'
##' @param ... Further arguments.
##' @return Boxplot of single-split p-values.
##' @author nicolas
##' @method plot ggmgsa
##' @export 
plot.ggmgsa <- function(x,...){
	boxplot(t(x$pval),names=x$gs.names,xlab='gene-sets',ylab='single-split p-values (uncorrected)',...)
}

#####################
##Required Packages##
#####################
library(mvtnorm)

##' Simulate from mixture model with multi-variate Gaussian or t-distributed 
##' components.
##'
##' 
##' @title Simulate from mixture model.
##' @param n sample size
##' @param n.comp number of mixture components ("comps")
##' @param mix.prob mixing probablities (need to sum to 1)
##' @param Mu matrix of component-specific mean vectors 
##' @param Sig array of component-specific covariance matrices
##' @param dist 'norm' for Gaussian components, 't' for t-distributed components
##' @param df degrees of freedom of the t-distribution (not used for Gaussian 
##' distribution), default=2
##' @return  a list consisting of:
##' \item{S}{component assignments}
##' \item{X}{observed data matrix}
##' @author n.stadler
##' @export
##' @import mvtnorm
##' @examples
##' n.comp = 4
##' p = 5 # dimensionality
##' Mu = matrix(rep(0, p), p, n.comp)
##' Sigma = array(diag(p), c(p, p, n.comp))
##' mix.prob = rep(0.25, n.comp)
##' 
##' sim_mix(100, n.comp, mix.prob, Mu, Sigma)

sim_mix <- function(n,n.comp,mix.prob,Mu,Sig, dist='norm', df=2){
	
  # Only one Mu specified
  if(is.vector(Mu) || dim(Mu)[2] == 1) 
    Mu = matrix(Mu, length(Mu), n.comp)
  
  # Only one Sig specified
  if(length(dim(Sig)) == 2 || dim(Sig)[3] == 1) 
    Sig = array(Sig, c(dim(Sig)[1:2], n.comp))
     
  # Incorrect dimension of Mu or Sig
  if(is.vector(Mu) || dim(Mu)[2] != n.comp ||
     length(dim(Sig)) != 3 || dim(Sig)[3] != n.comp) 
    stop(paste('The last dimension of Mu and Sig needs to be equal to n.comp',
               '(one mean vector and covariance matrix per component).'))
  
  # Incorrect number of mixture probabilities
  if(length(mix.prob) != n.comp) 
    stop('Length of mix.prob needs to be equal to n.comp.')
  
  # Misspecified mixture probabilities
  if(any(mix.prob > 1 | mix.prob < 0) || sum(mix.prob) != 1) 
    stop('Misspecified mixture probabilities.')
  
  K <- n.comp
	p <- dim(Mu)[1]
	x <- matrix(0,nrow=n,ncol=p)
	s <- sample(1:K,n,replace=TRUE,prob=mix.prob)
	
  # Generate from each component
  for (k in 1:K){    
		x[s==k,] <-  if(dist == 'norm') rmvnorm(sum(s==k),mean=Mu[,k],
																						sigma=Sig[,,k])
                 else if(dist == 't') rmvt(sum(s==k),delta=Mu[,k],
                 													sigma=Sig[,,k], df=df, type='shifted')
                 else stop(paste('Invalid dist argument:', dist))
	}
	
  return(list(S=s,X=x))
}


##' Generate an inverse covariance matrix with a given sparsity and 
##' dimensionality
##'
##' This function generates an inverse covariance matrix, with at most 
##' (1-sparsity)*p(p-1)
##' non-zero off-diagonal entries, where the non-zero entries are sampled from a 
##' beta distribution.
##' 
##' @title generate_inv_cov
##' @param p Dimensionality of the matrix.
##' @param sparsity Determined the proportion of non-zero off-diagonal entries.
##' @return A p by p positive definite inverse covariance matrix.
##' @export
##' @examples
##' generate_inv_cov(p=162)
generate_inv_cov <- function(p=162, sparsity=0.7) {
	num.edges = p*(p-1)/2
	
	edge.prop = 1 - sparsity
	s = round(num.edges*edge.prop) # Number of non-zero entries in each matrix
	
	return(getinvcov(p, s=s))
}

##' Generate an inverse covariance matrix with a given sparsity and 
##' dimensionality
##'
##' @param p Dimensionality
##' @param s Sparsity
##' @param a.diff binomial parameter
##' @param b.diff binomial parameter
##' @param magn.diag Magnitude
##' @param emin e min
##' @return Inverse covariance matrix
##' Internal function
##' @keywords internal
getinvcov<- function(p,s, a.diff=5,b.diff=5,magn.diag=0,emin=0.1){
	#####8!!! act1 are the indices of the upper-diagonal non-zero entries of a 
	#####pxp matrix 
	
	ind.upper.tri <- which(upper.tri(matrix(NA,p,p)))
	act1 <- sample(ind.upper.tri,size=s,replace=FALSE)
  
	B1 <- matrix(0,p,p)
	B1[act1] <- rbeta(length(act1),a.diff,b.diff)
	diag(B1) <- 0
	B1 <- (t(B1)+B1)
	
	
	####compute (positive definite) concentration matrices
	SigInv <- list()
	
	siginv <- B1
	e.min <- min(eigen(siginv)$values)
	siginv <- siginv+diag(abs(e.min)+emin+magn.diag,p)
	SigInv <- siginv
	
	return(SigInv)
}



##' Generate inverse covariances, means, mixing probabilities, and simulate 
##' data from resulting mixture model.
##'
##' This function generates n.comp mean vectors from a standard Gaussian and
##' n.comp covariance matrices, with at most (1-sparsity)*p(p-1)/2
##' non-zero off-diagonal entries, where the non-zero entries are sampled from a 
##' beta distribution. Then it uses \code{\link{sim_mix}} to simulate from a 
##' mixture model with these means and covariance matrices.
##' 
##' Means Mu and covariance matrices Sig can also be supplied by the user.
##' 
##' @title sim_mix_networks
##' @param n Number of data points to simulate.
##' @param p Dimensionality of the data.
##' @param n.comp Number of components of the mixture model.
##' @param sparsity Determines the proportion of non-zero off-diagonal entries.
##' @param mix.prob Mixture probabilities for the components; defaults to 
##' uniform distribution.
##' @param Mu Means for the mixture components, a p by n.comp matrix. If NULL, 
##' sampled from a standard Gaussian.
##' @param Sig Covariances for the mixture components, a p by p by n.comp 
##' array. If NULL,
##' generated using \code{\link{generate_inv_cov}}.
##' @param ... Further arguments passed to \code{\link{sim_mix}}.
##' @return A list with components:
##' \code{Mu} Means of the mixture components.
##' \code{Sig} Covariances of the mixture components.
##' \code{data} Simulated data, a n by p matrix.
##' \code{S} Component assignments, a vector of length n.
##' @export
##' @examples
##' # Generate dataset with 100 samples of dimensionality 30, and 4 components
##' test.data = sim_mix_networks(n=100, p=30, n.comp=4)
sim_mix_networks <- function(n, p, n.comp, sparsity=0.7, 
														 mix.prob=rep(1/n.comp, n.comp),
														 Mu=NULL, Sig=NULL, ...) {
	
	if(is.null(Mu)) {
	  Mu = sapply(1:n.comp, function(n.comp) rnorm(p,0,3))
	}
	
	if(is.null(Sig)) {
	  Sig = sapply(1:n.comp, 
												function(n.comp) solve(generate_inv_cov(p, sparsity)), 
												simplify='array')
	}
	
	data = sim_mix(n, n.comp, mix.prob, Mu, Sig, ...)
	
	return(list(Mu=Mu, Sig=Sig, data=data$X, comp=data$S))
}

##' Generate two sparse inverse covariance matrices with overlap
##'
##' 
##' @title Generate sparse invcov with overlap
##' @param p number of nodes
##' @param graph 'random' or 'hub'
##' @param n.hub number of hubs (only for graph='hub')
##' @param n.hub.diff number of different hubs
##' @param n.nz number of edges per graph (only for graph='random')
##' @param n.nz.common number of edges incommon between graphs (only for 
##' graph='random')
##' @param magn.nz.diff default=0.9
##' @param magn.nz.common default=0.9
##' @param magn.diag default=0
##' @param emin default=0.1 (see ?huge.generator)
##' @param verbose If verbose=FALSE then tracing output is disabled.
##' @export
##' @return Two sparse inverse covariance matrices with overlap
##' @examples
##' n <- 70
##' p <- 30
##' 
##' ## Specifiy sparse inverse covariance matrices,
##' ## with number of edges in common equal to ~ 0.8*p
##' gen.net <- generate_2networks(p,graph='random',n.nz=rep(p,2),
##'                               n.nz.common=ceiling(p*0.8))
##' 
##' invcov1 <- gen.net[[1]]
##' invcov2 <- gen.net[[2]]
##' 
##' plot_2networks(invcov1,invcov2,label.pos=0,label.cex=0.7)
generate_2networks<- function(p,graph='random',
															n.nz=rep(p,2),n.nz.common=p,
															n.hub=2,n.hub.diff=1,
															magn.nz.diff=0.8,
															magn.nz.common=0.9,magn.diag=0,emin=0.1,
															verbose=FALSE){
	
	if(graph=='random'){
		
		if(min(n.nz)<n.nz.common){stop('min(n.nz) < n.nz.common')}
		
		ind.upper.tri <- which(upper.tri(matrix(NA,p,p)))
		B.list <- list()
		B1 <- matrix(0,p,p)
		nz1 <- sample(ind.upper.tri,size=n.nz[1],replace=FALSE)
		nz12 <- sample(nz1,size=n.nz.common,replace=FALSE)
		remain.zero <- setdiff(ind.upper.tri,nz1)
		B1[nz12] <- magn.nz.common
		B1[setdiff(nz1,nz12)] <- magn.nz.diff
		B.list[[1]] <- B1+t(B1)
		
		B2 <- matrix(0,p,p)
		nz2 <- c(nz12,sample(remain.zero,size=n.nz[2]-n.nz.common,replace=FALSE))
		B2[nz2] <- magn.nz.common
		B2[setdiff(nz2,nz12)] <- magn.nz.diff
		B.list[[2]] <- B2+t(B2)
		
	}
	
	if(graph=='hub'){
		
		if(n.hub<n.hub.diff){
			stop("n.hub less than n.hub.diff: choose smaller n.hub.diff")}
		
		##generate hub network (see library(huge); huge.generator)
		theta.hub <- matrix(0,p,p)
		g.large = p%%n.hub#number of large hubs
		g.small = n.hub - g.large#number of small hubs
		n.small = floor(p/n.hub)#size small hub
		if(n.small<=1){
			stop('hub with less than 2 nodes: choose a smaller n.hub')
		}
		n.large = n.small + 1#size large hub
		g.list = c(rep(n.small, g.small), rep(n.large, g.large))
		g.ind = rep(c(1:n.hub), g.list)
		for (i in 1:n.hub) {
			tmp = which(g.ind == i)
			theta.hub[tmp[1], tmp] = 1
			theta.hub[tmp, tmp[1]] = 1
			rm(tmp)
			gc()
		}
		
		B.list <- list()
		if(n.hub.diff==0){
			B.list[[1]] <- B.list[[2]] <- theta.hub*magn.nz.common
		}else{
			B1 <- B2 <- matrix(0,p,p)
			tmp <- which(g.ind%in%1:n.hub.diff)
			
			B1[-tmp,-tmp] <- theta.hub[-tmp,-tmp]*magn.nz.common
			B.list[[1]] <- B1
			
			B2[-tmp,-tmp] <- theta.hub[-tmp,-tmp]*magn.nz.common
			B2[tmp,tmp] <- theta.hub[tmp,tmp]*magn.nz.diff
			B.list[[2]] <- B2
		}
		
	}
	
	####compute (positive definite) concentration matrices
	SigInv <- list()
	for (k in 1:2){
		siginv <- B.list[[k]]
		e.min <- min(eigen(siginv)$values)
		siginv <- siginv+diag(abs(e.min)+emin+magn.diag,p)
		SigInv[[k]] <- siginv
		if(verbose){
			cat('ev.min: ',min(eigen(siginv)$values),'\n')
			cat('condnum: ',
					max(abs(eigen(siginv)$values))/min(abs(eigen(siginv)$values)),'\n')
		}
	}
	return(list(invcov1=SigInv[[1]],invcov2=SigInv[[2]]))
}

context('Running cv.glasso on heterogeneous dataset with fixed grouping')

# Generate test data (all components from same distribution)
n.comps = 4
p = 5
Mu = matrix(rep(0, p), p, n.comps)
Sigma = array(diag(p), c(p, p, n.comps))

# Only do this is not on Bioconductor (takes too long)
if(identical(Sys.getenv("BBS_HOME"), "")) {
  test.data = sim_mix(500, n.comps, rep(0.25, n.comps), Mu, Sigma)

  one.group.result = het_cv_glasso(test.data$X[test.data$S==1,])

  test_that("No errors when running with one group.",
          expect_true(!is.null(one.group.result)))

  full.result = het_cv_glasso(test.data$X, test.data$S)
 
  test_that("Correct means", 
           expect_lt(sum(abs(full.result$Mu-Mu)), 0.1*p*n.comps))
  test_that("Correct variances", 
         expect_lt(sum(
           abs(full.result$Sigma.diag-sapply(1:n.comps, 
                                             function(x) diag(Sigma[,,x])))), 0.1*p*n.comps))
 
  test_that("Correct covariances", 
           expect_lt(sum(abs(full.result$Sig-Sigma)), 0.1*p*p*n.comps))
}

context('Running mixglasso on heterogeneous dataset with unknown grouping')

# Generate heterogeneous dataset
Mu = sapply(1:n.comps, function(n.comp) rnorm(p,0,5))
test.data = sim_mix_networks(1000, p, n.comps, Mu=Mu)

# Try single n.comp
mixglasso.single = mixglasso(test.data$data, n.comps)

test_that("Rand index on inferred components with mixglasso is > 0.7", 
					expect_gt(adjustedRandIndex(test.data$comp, mixglasso.single$comp), 0.7))

# Try multiple n.comp
mixglasso.mult = mixglasso(test.data$data, 2:6)

test_that("Correct BIC is selected", 
					expect_equal(mixglasso.mult$n.comp[mixglasso.mult$bic.opt], n.comps))

context('Simulating data from Gaussian mixture model')

# Generate test data (all components from same distribution)
n.comps = 4
p = 5 # dimensionality
Mu = matrix(rep(0, p), p, n.comps)
Sigma = array(diag(p), c(p, p, n.comps))
mix.prob = rep(0.25, n.comps)

test_that("No errors when using one Mu.",
          expect_true({sim_mix(100, n.comps, mix.prob, Mu[,1], Sigma); TRUE}))

test_that("No errors when using one Sigma.",
          expect_true({sim_mix(100, n.comps, mix.prob, Mu, Sigma[,,1]); TRUE}))

test_that("Errors when using wrong number of Sigma or Mu.", {
           expect_error(sim_mix(100, n.comps, mix.prob, Mu, Sigma[,,1:3]))
           expect_error(sim_mix(100, n.comps, mix.prob, Mu[,1:3], Sigma))}) 

test_that("Errors with misspecified mix.prob", {
          expect_error(sim_mix(100, n.comps, rep(0.25, 3), Mu, Sigma))
          expect_error(sim_mix(100, n.comps, c(mix.prob[1:3], 1.25) , Mu, Sigma))
          expect_error(sim_mix(100, n.comps, c(mix.prob[1:3], -0.25) , Mu, Sigma))
          expect_error(sim_mix(100, n.comps, c(mix.prob[1:3], 0.26) , Mu, Sigma))})

test.data = sim_mix(1e4, n.comps, mix.prob, Mu, Sigma)
emp.mean = sapply(1:4, function(x) colMeans(test.data$X[test.data$S==x,]))
emp.cov = sapply(1:4, function(x) var(test.data$X[test.data$S==x,]), simplify='array')

test_that("Correct Mean and Sigma", {
          expect_lt(sum(abs(emp.mean-Mu)), 0.1*p*n.comps)
          expect_lt(sum(abs(emp.cov-Sigma)), 0.1*p*n.comps*n.comps)})

emp.mix = sapply(1:4, function(x) sum(test.data$S==x)/length(test.data$S))

test_that("Correct mixture components",
          expect_lt(sum(abs(emp.mix-mix.prob)), 0.1))

test_that("t-distribution throws no error",
          expect_true({sim_mix(100, n.comps, mix.prob, Mu[,1], Sigma, 
                                      dist='t', df=2);TRUE}))

context('Simulating means, covariances, and simulating data from Gaussian mixture model')

test.covariance = generate_inv_cov(10, 0.8)

test_that("Inverting inverse covariance",
          expect_true({solve(test.covariance); TRUE}))

test_that("Sparsity of inverse covariance",
          expect_equal((sum(test.covariance != 0) - 10)/(10*9), 0.2))

test.data = sim_mix_networks(1e4, p, n.comps)
emp.mean = sapply(1:4, function(x) colMeans(test.data$data[test.data$comp==x,]))
emp.cov = sapply(1:4, function(x) var(test.data$data[test.data$comp==x,]), simplify='array')

test_that("Correct Mean and Sigma", {
  expect_lt(sum(abs(emp.mean-test.data$Mu)), 0.2*p*n.comps)
  expect_lt(sum(abs(emp.cov-test.data$Sig)), 0.2*p*n.comps*n.comps)})

##set seed
set.seed(1)

##sample size and number of nodes
n <- 40
p <- 10

##specifiy sparse inverse covariance matrices
gen.net <- generate_2networks(p,graph='random',n.nz=rep(p,2),
                              n.nz.common=ceiling(p*0.8))
invcov1 <- gen.net[[1]]
invcov2 <- gen.net[[2]]
plot_2networks(invcov1,invcov2,label.pos=0,label.cex=0.7)

##get corresponding correlation matrices
cor1 <- cov2cor(solve(invcov1))
cor2 <- cov2cor(solve(invcov2))

##generate data under alternative hypothesis
library('mvtnorm')
x1 <- rmvnorm(n,mean = rep(0,p), sigma = cor1)
x2 <- rmvnorm(n,mean = rep(0,p), sigma = cor2)

##run diffnet
split1 <- sample(1:n,20)#samples for screening (condition 1)
split2 <- sample(1:n,20)#samples for screening (condition 2)
dn <- diffnet_singlesplit(x1,x2,split1,split2)
dn$pval.onesided#p-value

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Comments:
% * set opts_chunk$set(eval=FALSE) for 'knitting' 
%   without running R code
%

\documentclass{article}

%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{nethet}

\newcommand{\X}{\mathbf{X}}
\newcommand{\argmin}{\mathop{\arg \min}\limits}
\newcommand{\argmax}{\mathop{\arg \max}\limits}


\usepackage{amsmath}
\usepackage{amsfonts}

<<style-knitr, eval=TRUE, echo=FALSE, results="asis" >>=
BiocStyle::latex()
@

\bioctitle[nethet]{A Bioconductor package for investigation of network heterogeneity from high-dimensional data
  % Heterogeneous Clustering and Network Inference with the MixGLasso
}
\author{Nicolas St\"{a}dler\footnote{staedler.n@gmail.com}, Frank Dondelinger\footnote{fdondelinger.work@gmail.com}}

\begin{document}

\maketitle


<<echo=FALSE, include=FALSE, warning=FALSE>>=
# Load package
library(nethet)

set.seed(1)
@

\section{Introduction}

Data analysis in systems biology and medicine often requires analysing 
data whose dynamics can be described as a network of observed and unobserved 
variables. A simple example is a protein signalling network in a cell. 

Simplifying the process greatly, signalling proteins known as kinases can be 
unphosphorylated (inactive) or phosphorylated (active). Cell signalling 
uses the phosphorylation machinery to pass messages from the exterior of the 
cell to the interior where they will be acted upon. This message passing is 
achieved via a relay of kinases and other proteins (the signalling pathway), 
which can be thought of as a network.

Numerous software packages exist for reconstructing networks from observational
data (e.g. \cite{ARACNE}, \cite{bnlearn}, \cite{RTN}, \cite{EDISON}). 
However, most of these packages assume that there is a single underlying network. 
Package \texttt{nethet} was designed with the intent of handling heterogeneous 
datasets arising from a collection of (possibly related) networks. 

Take for example protein measurements of breast cancer tumor cells. It
is known that there exist several subtypes of breast cancer with different 
molecular profiles \cite{tcga_breast}. We might be interested in whether 
the signalling pathways (networks) reconstructed from two subtypes are 
statistically different. If they are not, then we might want to identify new 
subtypes that present different molecular profiles, and reconstruct the 
networks for each identified subtype. The \texttt{nethet} package contains 
functionalities to tackle all of these tasks.

To the best of our knowledge, \texttt{nethet} is currently the only 
implementation of statistical solid methodology enabling the analysis of 
network heterogeneity from high-dimensional data. Package \texttt{nethet} 
combines several implementations of recent statistical innovations useful for 
estimation and comparison of networks in a heterogeneous, high-dimensional 
setting. In particular, we provide code for formal two-sample testing in 
Gaussian graphical models (differential network and GGM-GSA; 
\cite{staedler_diffnet}, \cite{ggmgsa2014}) and make a novel network-based 
clustering algorithm available (mixed graphical lasso, 
\cite{staedler_mixglasso}).

\section{Statistical setup}

We consider independent samples $X_i\in \mathbb{R}^p$
($i=1,\ldots,n$), measuring $p$ molecular variables. We assume that
the collected data can be divided into $K$ different groups. Let
$S_i\in\{1,\ldots,K\}$ be the group assignment of sample $i$, denote
with $n_k$ the group specific sample size and write $\X_k$ for the
$n_k\times p$ data matrix consisting of all samples belonging to group
$k$.

To describe networks we use Gaussian graphical models (\emph{GGMs},
\cite{rue2005}). These models use an undirected graph (or network) to
describe probabilistic relationships between variables.  For each
group $k$, we assume that $\X_k$ is sampled from a multivariate
Gaussian distribution with (unknown) mean $\mu_k$ and (unknown) $p
\times p$ concentration matrix $\Omega_k=\Sigma_k^{-1}$. The matrix
$\Omega_k$ defines the group-specific graph $G_k$ via
\begin{eqnarray*}
&(j,j') \in
E(G_k) \Leftrightarrow \Omega_{k;jj'}\neq 0,\\
&j,j' \in \{ 1, \ldots, p \}\;\textrm{and}\; j \neq j',
\end{eqnarray*}
where $E(G)$ denotes the edge set of graph $G$. 

%The different groups are characterized by group-specific mean vectors
%$\mu_k$ and networks $G_k$ ($k=1,\ldots,K$). These are unknown at the
%outset and have to be estimated from the data. 
 Learning of networks $G_k$ is a so-called high-dimensional statistical problem.
% because of the large
%number of parameters relative to the sample sizes available at the
%group level. 
We employ regularization to learn sparse, parsimonious networks and
thereby control over-fitting. In particular, we use the popular
\emph{graphical Lasso} \cite{friedman2007sic,huge2012}. Frequently 
the group assignments $S_i$, as well as the number of groups
$K$, are unknown at the outset and have to be inferred simultaneously
with the group-specific mean vectors and networks. The method
\emph{mixglasso}, implemented in this package, is a novel tool for
high-dimensional, network-based clustering. It is based on a finite 
mixture of GGMs and employs an adaptive and automatic penalization scheme 
\cite{staedler_mixglasso}.


Network inference is subject to statistical uncertainty and observed
differences between estimated networks may be due to noise in the data
and variability in estimation rather than any true difference in
underlying network topology. Testing hypotheses of the form
\begin{eqnarray*}
  \label{eq:hypothesis}
  \mathbf{H_0}: G_k=G_{k'}, \quad k,k'\in \{1,\ldots,K\},\; k\neq k'
\end{eqnarray*}
is challenging. %and involves non-nested model comparison
%\citep{vuong1989}. 
We build upon a recent approach called \emph{differential network}
\cite{staedler_diffnet,ggmgsa2014} which allows formal two-sample
testing in high-dimensional GGMs.

\section{Package functionalities}

The package consists of the following main parts: 

\begin{itemize}
 \item Simulation functions for creating synthetic data from the underlying 
       Gaussian mixture (network) model.
 \item Network inference using the \texttt{het\_cv\_glasso} function for 
       reconstructing heterogeneous networks from data with the graphical 
       Lasso \cite{glasso} when the group structure is known.
     \item High-dimensional hypothesis testing capabilities, including
       the \texttt{diffnet} functions implementing a statistical test
       for whether the networks underlying a pair of dataset are
       different, the \texttt{ggmgsa} functions allowing for
       differential gene set testing and the \texttt{diffregr}
       functions testing whether two high-dimensional regression
       models are statistically different \cite{staedler_diffnet,ggmgsa2014}.
 \item The \texttt{mixglasso} functions implementing a 
       network-based clustering and reconstruction algorithm also based on the 
       graphical Lasso, for unknown group structure \cite{staedler_mixglasso}.
 \item Plotting and export functions for displaying and saving the results of
       the analysis in a sensible way.
\end{itemize}

\section{Simulate data}
\label{sec:simulation}

In order to demonstrate the functionalities of the package, we will
first simulate data from a Gaussian mixture model with a known
covariance structure. The \textit{nethet} package includes code for
generating random covariance matrices with a given sparsity, and for
simulating from a Gaussian mixture model with given means and
covariances. The function \texttt{sim\_mix\_networks} provides a
convenient wrapper for both:

<<echo=TRUE, include=TRUE>>=
# Specify number of simulated samples and dimensionality
n = 100
p = 25

# Specify number of components of the mixture model and mixture probabilities
n.comp = 4

mix.prob = c(0.1, 0.4, 0.3, 0.2)

# Specify sparsity in [0,1], indicating fraction of off-diagonal zero entries.
s = 0.9


# Generate networks with random means and covariances. Means will be drawn from 
# a standard Gaussian distribution, non-zero covariance values from a 
# Beta(1,1) distribution.
sim.result = sim_mix_networks(n, p, n.comp, s, mix.prob)
@

The data is contained in \texttt{sim.result\$data}, and the components
that each data point belongs to are contained in
\texttt{sim.result\$comp}. Let's check that the mixture probabilities
are correct and then plot the first two dimensions of the data. Note
that we do not expect these to be well-separated in any way.


<<echo=TRUE, include=TRUE, fig.width=5, fig.height=3, fig.align='center'>>=
print(table(sim.result$comp)/n)

component = as.factor(sim.result$comp)

library('ggplot2')
qplot(x=sim.result$data[,1], y=sim.result$data[,2], 
      colour=component) + 
  xlab('Dimension 1') +
  ylab('Dimension 2')
  
@

The means and covariances of the data are contained in
\texttt{sim.result\$Mu} and \texttt{sim.result\$Sig}. If desired, they
can also be specified when calling \texttt{sim\_mix\_networks}.

<<echo=TRUE, include=TRUE, fig.width=5, fig.height=3, fig.align='center'>>=
# Generate new dataset with the same covariances, but different means
sim.result.new = sim_mix_networks(n, p, n.comp, s, mix.prob, Sig=sim.result$Sig)

component = as.factor(sim.result.new$comp)

qplot(x=sim.result.new$data[,1], y=sim.result.new$data[,2], 
      colour=component) + 
  xlab('Dimension 1') +
  ylab('Dimension 2')
@

When the covariance matrices for the components are not specified in
advance, the \texttt{sim\_mix\_networks} function implicitly assumes
that they are generated independently of each other. In order to test
the \texttt{diffnet} functions, we also want to be able to generate
simulated data from pairs of networks that present some common
edges. The \texttt{generate\_2networks} function is used to generate
pairs of networks with an arbitrary overlap.

<<echo=TRUE, include=TRUE, fig.width=7, fig.height=3, fig.align='center'>>=
## Sample size and number of nodes
n <- 40
p <- 10

## Specify sparse inverse covariance matrices,
## with number of edges in common equal to ~ 0.8*p
gen.net <- generate_2networks(p,graph='random',n.nz=rep(p,2),
                              n.nz.common=ceiling(p*0.8))

invcov1 <- gen.net[[1]]
invcov2 <- gen.net[[2]]

plot_2networks(invcov1,invcov2,label.pos=0,label.cex=0.7)
@

\section{Network estimation with known group labels}

If it is known a priori to which component each sample belongs, then
the problem of reconstructing the network reduces to a simple
application of the graphical Lasso to each component. For convenience,
we have included a wrapper function \texttt{het\_cv\_glasso} in
\texttt{nethet} that applies the graphical Lasso \cite{glasso} to each 
component in a heterogeneous dataset with specified component labels. The 
penalisation hyperparameter is tuned individually for each component using
cross-validation.

To demonstrate \texttt{het\_cv\_glasso}, we will generate some data in
the same way as in the previous section:

<<echo=TRUE, include=TRUE>>=
set.seed(10)

n = 100
p = 25

# Generate networks with random means and covariances. 
sim.result = sim_mix_networks(n, p, n.comp, s, mix.prob)

test.data = sim.result$data
test.labels = sim.result$comp

# Reconstruct networks for each component
networks = het_cv_glasso(data=test.data, grouping=test.labels)
@

One way of checking if the reconstructed networks are sensible is
plotting the covariance matrices used for generating the networks
against the reconstructed covariance matrices.

<<echo=TRUE, include=TRUE, fig.width=5, fig.height=3, fig.align='center'>>=
# Component labels for covariance values
components = as.factor(rep(1:n.comp, each=p^2))

qplot(x=c(networks$Sig), y=c(sim.result$Sig),
			colour=components) + 
  xlab('Reconstructed Covariances') +
  ylab('True Covariances')
  
@

\section{High-dimensional two-sample testing}\label{sec:diffnet}

We have demonstrated how to use our package to estimate networks from
heterogeneous data. Often, we would like to perform a statistical
comparison between networks. \emph{Differential network} allows formal
hypothesis testing regarding network differences. It is based on a
novel and very general methodology for high-dimensional two-sample
testing. Other useful tools based on this technology are
\emph{GGM-GSA} (``multivariate gene-set testing based on GGMs'') and
\emph{differential regression} which allows formal two-sample testing
in the high-dimensional regression model. For details on this
methodology we refer the reader to \cite{staedler_diffnet,ggmgsa2014}.

\subsection{Differential network}
Let us consider datasets generated from GGMs $G_1$ and $G_2$
respectively. We would like to know whether networks inferred from
these datasets differ in a statistical significant manner, that is we
would like to test the hypothesis
\begin{eqnarray*}
  \mathbf{H}_0:\quad G_1=G_2.
\end{eqnarray*}
The function \texttt{diffnet\_multisplit} uses repeated sample
splitting to address this task. The main steps are:
\begin{enumerate}
\item Both datasets are randomly split into two halves: the ``\emph{in}-'' and ``\emph{out}-sample''.
\item Networks are inferred using only the \emph{in}-sample (``screening step'').
\item Based on the \emph{out}-sample, a p-value is computed which
  compares the networks obtained in step 2 (``cleaning step'').
\item Steps 1-3 are repeated many times (e.g. 50 times); the resulting
  p-values are aggregated and the final aggregated p-value is reported.
\end{enumerate}

We now illustrate the use of \texttt{diffnet\_multisplit} with an
example. We consider GGMs (i.e. inverse covariance matrices)
previously generated in Section \ref{sec:simulation}.

<<eval=TRUE, echo=TRUE, include=TRUE, warning=TRUE, results="asis">>=
## Set seed
set.seed(1)
 
## Sample size and number of nodes
p <- 30
 
## Specify sparse inverse covariance matrices
gen.net <- generate_2networks(p,graph='random',n.nz=rep(p,2),                               
                             n.nz.common=ceiling(p*0.8))
invcov1 <- gen.net[[1]]
invcov2 <- gen.net[[2]] 

## Get corresponding correlation matrices
cor1 <- cov2cor(solve(invcov1))
cor2 <- cov2cor(solve(invcov2))
@ 

We start with generating data under the ``null-scenario'' where both datasets have
the same underlying network.

<<echo=TRUE, include=TRUE, warning=TRUE, results="asis">>=
## Generate data under null hypothesis
library(mvtnorm) # To generate multivariate Gaussian random samples

## Sample size
n <- 70

x1 <- rmvnorm(n,mean = rep(0,dim(cor1)[1]), sigma = cor1)
x2 <- rmvnorm(n,mean = rep(0,dim(cor1)[1]), sigma = cor1)
@ 

Then, we run a differential network analysis:
<<echo=TRUE, include=TRUE, warning=TRUE, results="asis">>=
## Run diffnet (under null hypothesis)
dn.null <- diffnet_multisplit(x1,x2,b.splits=1,verbose=FALSE)
@ 

We obtain the p-value \Sexpr{dn.null$ms.pval}, which is stored in 
\texttt{dn.null\$ms.pval}.

The same analysis can be performed for data generated under the
alternative hypothesis. 
<<echo=TRUE, include=TRUE, warning=TRUE, results="asis">>=

## Generate data under alternative hypothesis (datasets have different networks)
x1 <- rmvnorm(n,mean = rep(0,dim(cor1)[1]), sigma = cor1)
x2 <- rmvnorm(n,mean = rep(0,dim(cor1)[2]), sigma = cor2)

## Run diffnet (under alternative)
dn.altn <- diffnet_multisplit(x1,x2,b.splits=1,verbose=FALSE)
@ 

The resulting p-value is \Sexpr{dn.altn$ms.pval} which indicates a highly 
significant network difference.

The variable \texttt{b.splits} specifies the number of data splits
used in the differential network procedure. The p-values in the
previous examples were obtained using only a single data split
(\texttt{b.splits=1}). P-values heavily depend on the random split of
the data. This amounts to a "p-value lottery". To get stable and
reproducible results we therefore would typically choose a larger
number for the variable \texttt{b.split} and report the aggregated
p-value.
<<eval=TRUE, echo=TRUE, include=TRUE, warning=TRUE, fig.width=3.5, fig.height=3.5, fig.align='center'>>=
## Typically we would choose a larger number of splits
# Use parallel library (only available under Unix) for computational efficiency
if(.Platform$OS.type == "unix") {
	dn.altn <- diffnet_multisplit(x1,x2,b.splits=50,verbose=FALSE,mc.flag=TRUE)
} else {
  dn.altn <- diffnet_multisplit(x1,x2,b.splits=25,verbose=FALSE,mc.flag=FALSE)
}

par(cex=0.7)
plot(dn.altn, cex=0.5) # histogram over 50 p-values
cat('p-value:',dn.altn$medagg.pval,'\n') # median aggregated p-value
@


\subsection{Multivariate gene-set testing based on GGMs}
In the case where molecular variables can be grouped into various sets
of biologically related features (e.g. gene-sets or pathways),
\texttt{ggmgsa\_multisplit} can be used to perform differential
network analyses iteratively for all gene-sets. This allows us to
identify gene-sets which show a significant network difference. For
illustration we consider data generated from the following networks.

<<echo=TRUE, include=TRUE, warning=TRUE, results="asis", fig.width=5, fig.height=2.5, fig.align='center'>>=
## Generate new networks
set.seed(1)
p <- 9 # network with p nodes
n <- 40
hub.net <- generate_2networks(p,graph='hub',n.hub=3,n.hub.diff=1)#generate hub networks
invcov1 <- hub.net[[1]]
invcov2 <- hub.net[[2]]
plot_2networks(invcov1,invcov2,label.pos=0,label.cex=0.7,
               main=c('network 1', 'network 2'),cex.main=0.7)

## Generate data
library('mvtnorm')
x1 <- rmvnorm(n,mean = rep(0,p), sigma = cov2cor(solve(invcov1)))
x2 <- rmvnorm(n,mean = rep(0,p), sigma = cov2cor(solve(invcov2)))
@ 

%## Run DiffNet
%# fit.dn <- diffnet_multisplit(x1,x2,b.splits=2,verbose=FALSE)
%# fit.dn$medagg.pval

The nodes can be grouped into three gene-sets where only the first has
a different underlying network.
<<eval=TRUE, echo=TRUE, include=TRUE, warning=TRUE, results="asis">>=
## Identify groups with 'gene-sets'
gene.names <- paste('G',1:p,sep='')
gsets <- split(gene.names,rep(1:3,each=3))
@ 

We run GGM-GSA with a single data split (\texttt{b.splits=1}) and note
that only the p-value for the first gene-set has small magnitude. Again, we
would typically use a larger number of data splits in order to obtain
stable p-values.
<<eval=TRUE, echo=TRUE, include=TRUE, warning=TRUE, results="asis">>=
## Run GGM-GSA
fit.ggmgsa <- ggmgsa_multisplit(x1,x2,b.splits=1,gsets,gene.names,verbose=FALSE)

library(xtable)
print(xtable(summary(fit.ggmgsa),digits=6))
@

\subsection{Differential regression}

In addition to differential network, this \textbf{R}-package also
provides an implementation of differential regression. In particular,
the function \texttt{diffregr\_multisplit} allows formal two-sample
testing in the high-dimensional regression model. It is also based on
sample splitting and is very similar to the previously introduced
\texttt{diffnet\_multisplit}.

Consider the following sparse regression models.
<<echo=TRUE, include=TRUE, warning=TRUE, results="asis">>=
## Number of predictors and sample size
p <- 100
n <- 80

## Predictor matrices
x1 <- matrix(rnorm(n*p),n,p)
x2 <- matrix(rnorm(n*p),n,p)

## Active-sets and regression coefficients
act1 <- sample(1:p,5)
act2 <- c(act1[1:3],sample(setdiff(1:p,act1),2))
beta1 <- beta2 <- rep(0,p)
beta1[act1] <- 0.7
beta2[act2] <- 0.7
@ 

We generate data under the null-hypothesis and run differential regression. The histogram shows the distribution of the p-values obtained form ten data splits.
<<echo=TRUE, include=TRUE, warning=TRUE, results="asis", fig.align='center', fig.width=3.5,fig.height=3.5>>=
## Response vectors under null-hypothesis
y1 <- x1%*%as.matrix(beta1)+rnorm(n,sd=1)
y2 <- x2%*%as.matrix(beta1)+rnorm(n,sd=1)

## Differential regression; b.splits=10
fit.null <- diffregr_multisplit(y1,y2,x1,x2,b.splits=10)
par(cex=0.7)
plot(fit.null,cex=0.5) # histogram of p-values from b.split data splits
cat('p-value: ',fit.null$medagg.pval,'\n') # median aggregated p-value
@ 

The following example illustrates differential regression in scenario with different regression models. 
<<echo=TRUE, include=TRUE, warning=TRUE, results="asis",fig.align='center',fig.width=3.5,fig.height=3.5>>=
## Response vectors under alternative-hypothesis
y1 <- x1%*%as.matrix(beta1)+rnorm(n,sd=1)
y2 <- x2%*%as.matrix(beta2)+rnorm(n,sd=1)

## Differential regression (asymptotic p-values)
fit.alt <- diffregr_multisplit(y1,y2,x1,x2,b.splits=10)
par(cex=0.7)
plot(fit.alt)
cat('p-value: ',fit.alt$medagg.pval,'\n')
@ 

For differential regression we have the option to compute
permutation-based p-values by choosing a number of permutations
\texttt{n.perm}.
<<echo=TRUE, include=TRUE, warning=TRUE, results="asis",fig.align='center',fig.width=3.5,fig.height=3.5>>=
## Differential regression (permutation-based p-values; 100 permutations)
fit.alt.perm <- diffregr_multisplit(y1,y2,x1,x2,b.splits=5,n.perm=100)
@ 
The default option (\texttt{n.perm=NULL}) uses
an asymptotic approximation to calculate p-values.


\section{Network estimation and model-based clustering with unknown group labels}
% Network reconstruction with unknown components using \texttt{mixglasso}


Often we do not know a priori which component each sample belongs to. For example 
in the case of samples corresponding to protein measurements in breast cancer 
patients, the particular subtype of breast cancer that a patient suffers from
may be unknown. In these cases, our package allows for network-based clustering
of the samples using the mixture graphical Lasso (mixglasso), which jointly 
clusters the samples and reconstructs the networks for each group or cluster.

To demonstrate the \texttt{mixglasso} function, let us first generate some data 
in the same way as before, but with means defined to ensure separability 
of the groups:

<<echo=TRUE, include=TRUE>>=
# Generate networks with random means and covariances. 
n = 1000
p = 10
s = 0.9
n.comp = 3

# Create different mean vectors
Mu = matrix(0,p,n.comp)

# Define non-zero means in each group (non-overlapping)
nonzero.mean = split(sample(1:p),rep(1:n.comp,length=p))

# Set non-zero means to fixed value
for(k in 1:n.comp){
  Mu[nonzero.mean[[k]],k] = -2/sqrt(ceiling(p/n.comp))
}

# Generate data
sim.result = sim_mix_networks(n, p, n.comp, s, Mu=Mu)
@

Now we will run mixglasso on this dataset to retrieve the original clustering
and reconstruct the underlying networks.

<<echo=TRUE, include=TRUE>>=
# Run mixglasso
mixglasso.result = mixglasso(sim.result$data, n.comp=3)

# Calculate adjusted rand index to judge how accurate the clustering is
# Values > 0.7 indicate good agreement.
library(mclust, quietly=TRUE)
adj.rand = adjustedRandIndex(mixglasso.result$comp, sim.result$comp)
cat('Adjusted Rand Index', round(adj.rand, digits=2), '\n')
@

Table \ref{table:crosstab} shows the cross-tabulation of the number of samples
in predicted versus true groups.

<<echo=FALSE, include=TRUE, cache=FALSE, results='asis'>>=
crosstab <- function(class1,class2){
  tab <- matrix(NA,length(levels(class1)),length(levels(class2)))
  colnames(tab) <- levels(class2)
  rownames(tab) <- levels(class1)
  for (i in levels(class1)){
    tab[i,] <- table(class2[which(class1==i)])
  }
  return(tab)
}

# Relabel true groupings to avoid confusion
sim.grouping = c('A', 'B', 'C')[sim.result$comp]

cross.table = crosstab(factor(mixglasso.result$comp), factor(sim.grouping))

# Generate table
library(xtable)
latex.table = xtable(cross.table, caption=
										 	paste('Cross-tabulation of mixglasso clusters (rows) with',
                    'true group assignments (columns).'),
										 label='table:crosstab')

print(latex.table)
@


What if we don't know the true number groups? Luckily, \texttt{mixglasso} supports
model comparison using BIC \cite{BIC} and minimum description length \cite{MMDL}. 
In the following example we will use BIC to find the correct number of components:

<<echo=TRUE, include=TRUE, fig.width=5, fig.height=3, fig.align='center'>>=

# Run mixglasso over a range of numbers of components
mixglasso.result = mixglasso(sim.result$data, n.comp=1:6)

# Repeat with lambda=0 and lambda=Inf for comparison
mixglasso.result.0 = mixglasso(sim.result$data, n.comp=1:6, lambda=0)
mixglasso.result.Inf = mixglasso(sim.result$data, n.comp=1:6, lambda=Inf)

# Aggregate BIC results for plotting
BIC.vals = c(mixglasso.result$bic, mixglasso.result.0$bic,
						 mixglasso.result.Inf$bic)

lambda.labels = rep(c('Default', 'Lambda = 0', 'Lambda = Inf'), each=6)

# Plot to verify that minimum BIC value corresponds with true 
library(ggplot2)
plotting.frame <- data.frame(BIC=BIC.vals, Num.Comps=rep(1:6, 3), Lambda=lambda.labels)

p <- ggplot(plotting.frame) + 
	geom_line(aes(x=Num.Comps, y=BIC, colour=Lambda)) + 
	geom_vline(xintercept=3, linetype='dotted')

print(p)
@

We note that mixglasso involves a penalization parameter $\lambda$
which trades off goodness-of-fit and model complexity. We recommend to
use the default which employs an adaptive and automatic penalization
scheme \cite{staedler_mixglasso}. Note that in this simplified
example, $\lambda=0$ (no penalization) performs well because %the true
%inverse covariance matrices are not particularly sparse and 
$n >> p$. $\lambda=\infty$ constrains inverse covariance matrices to
be diagonal, hence the inferior performance.

\section{Plotting and exporting results}

Our package includes several functions for plotting and exporting the networks
and results that have been obtained. 

\subsection{Plotting results}

The output of \texttt{het\_cv\_glmnet} and \texttt{mixglasso} can be plotted either
in network form or as individual edges in the networks. For the network plots,
we use the \texttt{network} package \cite{network_package}. This is the default 
plotting when \texttt{plot} is invoked on an object of class 
\texttt{nethetclustering}, and produces one global plot showing edges that 
occur in any group, as well as one plot for each group. For this example we 
will use the networks and clustering obtained using \texttt{mixglasso} in the 
previous section.

<<echo=TRUE, include=TRUE, fig.width=3, fig.height=3, fig.align='center'>>=

# Retrieve best clustering and networks by BIC
mixglasso.clustering = mixglasso.result$models[[mixglasso.result$bic.opt]]

# Plot networks, omitting edges with absolute partial correlation < 0.5 in 
# every group.
# NOTE: Not displayed.
# plot(mixglasso.clustering, p.corrs.thresh=0.5)
@

Usually we are only interested in specific edges, and perhaps we wish to compare
them among groups. Function \texttt{dot\_plot} generates a plot with edges above
a certain threshold along the y-axis, and one circle for each group showing the
smallest mean of the two nodes that make up the edge. We use the \texttt{ggplot2}
package to make the plots \cite{ggplot2_package}.

<<echo=TRUE, include=TRUE, fig.width=5, fig.height=3, fig.align='center'>>=

# Plot edges, omitting those with absolute partial correlation < 0.5 in every 
# group.
g = dot_plot(mixglasso.clustering, p.corrs.thresh=0.5, dot.size.range=c(1,5))
@

Finally, we might want to compare the observed values of the nodes linked by
specific edges across groups. Function \texttt{scatter\_plot} will generate plots for
a specified list of edges.

<<echo=TRUE, include=TRUE, fig.width=7, fig.height=7, fig.align='center'>>=

# Specify edges
node.pairs = rbind(c(9,10), c(2,5),c(4,9))

# Create scatter plots of specified edges
g = scatter_plot(mixglasso.clustering, data=sim.result$data,
						node.pairs=node.pairs, cex=0.5)
@

\subsection{Exporting Results}

Our package offers the option to export the inferred networks as a comma-separated
values (CSV) text file. Like the plotting functions, function 
\texttt{export\_network} can be invoked on the output of \texttt{het\_cv\_glmnet} 
and \texttt{mixglasso}.

<<echo=TRUE, include=TRUE, eval=FALSE>>=

# Save network in CSV format, omitting edges with absolute partial correlation
# less than 0.25.
#export_network(mixglasso.clustering, file='nethet_network.csv',
#							 p.corrs.thresh=0.25)
@

This creates a CSV file encoding a table with one row for each edge with partial
correlation above the threshold, and columns indicating the nodes linked by the
edge, the absolute partial correlation, the sign of the partial correlation, and
the group or cluster in which the edge occurred.

If the user wishes to use the Cytoscape \cite{cytoscape} software to analyse 
the network further, we note that the output of \texttt{export\_network} can be
loaded into Cytoscape, provided the option \texttt{quote=FALSE} is set.

<<echo=TRUE, include=TRUE, eval=FALSE>>=

# Save network in CSV format suitable for Cytoscape import
#export_network(mixglasso.clustering, file='nethet_network.csv',
#							 p.corrs.thresh=0.25, quote=FALSE)
@

<<echo=TRUE, include=TRUE, eval=TRUE>>=
sessionInfo()
@

\bibliography{nethet}
\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cvglasso.R
\name{het_cv_glasso}
\alias{het_cv_glasso}
\title{Cross-validated glasso on heterogeneous dataset with grouping}
\usage{
het_cv_glasso(data, grouping = rep(1, dim(data)[1]), mc.flag = FALSE,
  use.package = "huge", normalise = FALSE, verbose = FALSE, ...)
}
\arguments{
\item{data}{The heterogenous network data. Needs to be 
a num.samples by dim.samples matrix or dataframe.}

\item{grouping}{The grouping of samples; a vector of length num.samples,
with num.groups unique elements.}

\item{mc.flag}{Whether to use parallel processing via package mclapply to
distribute the glasso estimation over different groups.}

\item{use.package}{'glasso' for glasso package, or 'huge' for huge package 
(default)}

\item{normalise}{If TRUE, normalise the columns of the data matrix before 
running glasso.}

\item{verbose}{If TRUE, output progress.}

\item{...}{Further parameters to be passed to \code{screen_cv.glasso}.}
}
\value{
Returns a list with named elements 'Sig', 'SigInv', 'Mu', 'Sigma.diag', 
'group.names' and 'var.names. 
The variables Sig and SigInv are arrays of size dim.samples by dim.samples 
by num.groups, where the first two dimensions contain the (inverse)
covariance matrix for the network obtained by running glasso on group k. Variables 
Mu and Sigma.diag contain the mean and variance of the input data,
and group.names and var.names contains the names for the groups and
variables in the data (if specified as colnames of the input data matrix).
}
\description{
Run glasso on a heterogeneous dataset to obtain networks (inverse covariance 
matrices) of the variables in the dataset for each pre-specified group of 
samples.
}
\details{
This function runs the graphical lasso with 
cross-validation to determine the best parameter lambda for each group of 
samples. Note that this function defaults to using package huge (rather than
package glasso) unless otherwise specified, as it tends to be more 
numerically stable.
}
\examples{
n = 100
p = 25

# Generate networks with random means and covariances. 
sim.result = sim_mix_networks(n, p, n.comp=3)

test.data = sim.result$data
test.labels = sim.result$comp

# Reconstruct networks for each component
networks = het_cv_glasso(data=test.data, grouping=test.labels) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auxfunc.R
\name{export_network}
\alias{export_network}
\title{Export networks as a CSV table.}
\usage{
export_network(net.clustering, file = "network_table.csv",
  node.names = rownames(net.clustering$Mu),
  group.names = sort(unique(net.clustering$comp)),
  p.corrs.thresh = 0.2, ...)
}
\arguments{
\item{net.clustering}{A network clustering object as returned by 
\code{\link{screen_cv.glasso}} or \code{\link{mixglasso}}.}

\item{file}{Filename to save the network table under.}

\item{node.names}{Names for the nodes in the network. If NULL, names from 
net.clustering will be used.}

\item{group.names}{Names for the clusters or groups. If NULL, names from 
net.clustering will be used (by default these are integets 1:numClusters).}

\item{p.corrs.thresh}{Threshold applied to the absolute partial correlations. 
Edges that are below the threshold in all of the groups are not exported. 
Using a negative value will export all possible edges (including those 
with zero partial correlation).}

\item{...}{Further parameters passed to \link{write.csv}.}
}
\value{
Function does not return anything.
}
\description{
This function takes the output of \code{\link{het_cv_glasso}} or 
\code{\link{mixglasso}} and exports it as a text table in CSV format, where each
entry in the table records an edge in one group and its partial correlation.
}
\examples{
n = 500
p = 10
s = 0.9
n.comp = 3

# Create different mean vectors
Mu = matrix(0,p,n.comp)

# Define non-zero means in each group (non-overlapping)
nonzero.mean = split(sample(1:p),rep(1:n.comp,length=p))

# Set non-zero means to fixed value
for(k in 1:n.comp){
	Mu[nonzero.mean[[k]],k] = -2/sqrt(ceiling(p/n.comp))
}

# Generate data
sim.result = sim_mix_networks(n, p, n.comp, s, Mu=Mu)
mixglasso.result = mixglasso(sim.result$data, n.comp=3)
mixglasso.clustering = mixglasso.result$models[[mixglasso.result$bic.opt]]

\dontrun{
# Save network in CSV format suitable for Cytoscape import
export_network(mixglasso.clustering, file='nethet_network.csv',
						 p.corrs.thresh=0.25, quote=FALSE)
}
}
\author{
Frank Dondelinger
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{lambda.max}
\alias{lambda.max}
\title{Lambdamax}
\usage{
lambda.max(x)
}
\arguments{
\item{x}{no descr}
}
\value{
no descr
}
\description{
Lambdamax
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixglasso.R
\name{glasso.parcor}
\alias{glasso.parcor}
\title{Graphical Lasso based on partial correlation penalty}
\usage{
glasso.parcor(s, rho, penalize.diagonal, maxiter = 1000, term = 10^{   
   -3 }, verbose = FALSE)
}
\arguments{
\item{s}{no descr}

\item{rho}{no descr}

\item{penalize.diagonal}{no descr}

\item{maxiter}{no descr}

\item{term}{no descr}

\item{verbose}{set to TRUE to print out progress.}
}
\value{
w; wi; iter
}
\description{
Graphical Lasso based on partial correlation penalty
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{twosample_single_regr}
\alias{twosample_single_regr}
\title{old single-split function for diffregr}
\usage{
twosample_single_regr(y1, y2, x1, x2, n.screen.pop1 = 100,
  n.screen.pop2 = 100, screen.meth = "screen_cvmin.lasso",
  compute.evals = "est2.my.ev3.diffregr")
}
\arguments{
\item{y1}{no descr}

\item{y2}{no descr}

\item{x1}{no descr}

\item{x2}{no descr}

\item{n.screen.pop1}{no descr}

\item{n.screen.pop2}{no descr}

\item{screen.meth}{no descr}

\item{compute.evals}{no descr}
}
\value{
no descr
}
\description{
Old single-split function for diffregr
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{my.ev2.diffregr}
\alias{my.ev2.diffregr}
\title{Computation eigenvalues}
\usage{
my.ev2.diffregr(Sig, act, act1, act2)
}
\arguments{
\item{Sig}{no descr}

\item{act}{no descr}

\item{act1}{no descr}

\item{act2}{no descr}
}
\value{
no descr
}
\description{
Computation eigenvalues
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netplot.R
\name{plot.diffregr}
\alias{plot.diffregr}
\title{Plotting function for object of class 'diffregr'}
\usage{
\method{plot}{diffregr}(x, ...)
}
\arguments{
\item{x}{object of class 'diffregr'}

\item{...}{Further arguments.}
}
\value{
Histogram over multi-split p-values.
}
\description{
Plotting function for object of class 'diffregr'
}
\author{
nicolas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{agg.score.iriz.shift}
\alias{agg.score.iriz.shift}
\title{Irizarry aggregate score (shift)}
\usage{
agg.score.iriz.shift(ttstat, geneset, gene.name)
}
\arguments{
\item{ttstat}{no descr}

\item{geneset}{no descr}

\item{gene.name}{no descr}
}
\value{
no descr
}
\description{
Irizarry aggregate score (shift)
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nethet.R
\docType{package}
\name{NetHet-package}
\alias{NetHet-package}
\title{NetHet-package}
\description{
A bioconductor package for high-dimensional exploration of biological network heterogeneity
}
\details{
Includes:
  *Network-based clustering (MixGLasso)
  *Differential network (DiffNet)
  *Differential regression (DiffRegr)
  *Gene-set analysis based on graphical models (GGMGSA)
  *Plotting functions for exploring network heterogeneity
}
\references{
St{\"a}dler, N. and Mukherjee, S. (2013). Two-Sample Testing in High-Dimensional Models.
Preprint \url{http://arxiv.org/abs/1210.4584}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{error.bars}
\alias{error.bars}
\title{Error bars for plotCV}
\usage{
error.bars(x, upper, lower, width = 0.02, ...)
}
\arguments{
\item{x}{no descr}

\item{upper}{no descr}

\item{lower}{no descr}

\item{width}{no descr}

\item{...}{no descr}
}
\value{
no descr
}
\description{
Error bars for plotCV
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{ww.mat}
\alias{ww.mat}
\title{Weight-matrix and eigenvalues}
\usage{
ww.mat(imat, act, act1, act2)
}
\arguments{
\item{imat}{no descr}

\item{act}{I_uv}

\item{act1}{I_u}

\item{act2}{I_v}
}
\value{
no descr
}
\description{
Calculates weight-matrix and eigenvalues
}
\details{
calculation based on true information matrix
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netplot.R
\name{dot_plot}
\alias{dot_plot}
\title{Create a plot showing the edges with the highest partial correlation in any cluster.}
\usage{
dot_plot(net.clustering, p.corrs.thresh = 0.25, hard.limit = 50,
  display = TRUE, node.names = rownames(net.clustering$Mu),
  group.names = sort(unique(net.clustering$comp)),
  dot.size.range = c(3, 12))
}
\arguments{
\item{net.clustering}{A network clustering object as returned by 
\code{\link{het_cv_glasso}} or \code{\link{mixglasso}}.}

\item{p.corrs.thresh}{Cutoff for the partial correlations; only edges with absolute 
partial correlation > p.corrs.thresh (in any cluster) will be displayed.}

\item{hard.limit}{Additional hard limit on the number of edges to display. If 
p.corrs.thresh results in more edges than hard.limit, only hard.limit edges with the
highest partial correlation are returned.}

\item{display}{If TRUE, print the plot to the current output device.}

\item{node.names}{Names for the nodes in the network.}

\item{group.names}{Names for the clusters or groups.}

\item{dot.size.range}{Graphical parameter for scaling the size of the circles (dots)
representing an edge in each cluster.}
}
\value{
Returns a ggplot2 object. If display=TRUE, additionally displays the 
plot.
}
\description{
This function takes the output of \code{\link{het_cv_glasso}} or 
\code{\link{mixglasso}} and creates a plot of the highest scoring edges along the
y axis, where, the edge in each cluster is represented by a circle whose area
is proportional to the smallest mean of the two nodes that make up the edge,
and the position along the y axis shows the partial correlation of the edge.
}
\examples{
n = 500
p = 10
s = 0.9
n.comp = 3

# Create different mean vectors
Mu = matrix(0,p,n.comp)

# Define non-zero means in each group (non-overlapping)
nonzero.mean = split(sample(1:p),rep(1:n.comp,length=p))

# Set non-zero means to fixed value
for(k in 1:n.comp){
	Mu[nonzero.mean[[k]],k] = -2/sqrt(ceiling(p/n.comp))
}

# Generate data
sim.result = sim_mix_networks(n, p, n.comp, s, Mu=Mu)
mixglasso.result = mixglasso(sim.result$data, n.comp=3)
mixglasso.clustering = mixglasso.result$models[[mixglasso.result$bic.opt]]

dot_plot(mixglasso.clustering, p.corrs.thresh=0.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{q.matrix.diffregr3}
\alias{q.matrix.diffregr3}
\title{Computation Q matrix}
\usage{
q.matrix.diffregr3(beta.a, beta.b, beta, sig.a, sig.b, sig, Sig, act.a,
  act.b, ss)
}
\arguments{
\item{beta.a}{no descr}

\item{beta.b}{no descr}

\item{beta}{no descr}

\item{sig.a}{no descr}

\item{sig.b}{no descr}

\item{sig}{no descr}

\item{Sig}{no descr}

\item{act.a}{no descr}

\item{act.b}{no descr}

\item{ss}{no descr}
}
\value{
no descr
}
\description{
Computation Q matrix
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixglasso.R
\name{sparse_conc}
\alias{sparse_conc}
\title{Generates sparse inverse covariance matrices}
\usage{
sparse_conc(p, K, s, s.common, magn.nz = 0.5, scale.parcor = TRUE)
}
\arguments{
\item{p}{Dimensionality of inverse covariance matrix}

\item{K}{Number of inverse covariance matrices}

\item{s}{Number of non-zero entries per inverse covariance matrix}

\item{s.common}{Number of non-zero entries shared across different inverse covariance matrices}

\item{magn.nz}{Magnitude of non-zero elements}

\item{scale.parcor}{Should SigInv be scaled to have diagonal equal one, siginv=parcor ?}
}
\value{
SigInv: list of inverse covariance matrices
}
\description{
Generates sparse inverse covariance matrices
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{inf.mat}
\alias{inf.mat}
\title{Information Matrix of Gaussian Graphical Model}
\usage{
inf.mat(Sig, include.mean = FALSE)
}
\arguments{
\item{Sig}{Sig=solve(SigInv) true covariance matrix under H0}

\item{include.mean}{no descr}
}
\value{
no descr
}
\description{
Compute Information Matrix of Gaussian Graphical Model
}
\details{
computes E_0[s(Y;Omega)s(Y;Omega)'] where s(Y;Omega)=(d/dOmega) LogLik
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netplot.R
\name{buildDotPlotDataFrame}
\alias{buildDotPlotDataFrame}
\title{Build up dataframe for plotting dot plot with ggplot2}
\usage{
buildDotPlotDataFrame(net.clustering, cluster.names, node.names)
}
\arguments{
\item{net.clustering}{Clustering}

\item{cluster.names}{Cluster names}

\item{node.names}{Node names}
}
\value{
A data frame for plotting the dotPlot with ggplot2 is returned. 
Column P.Corr contains the partial correlations of each edge as a numeric, 
column Mean contains the minimum mean expression of the two proteins
(e.g. if the edge is e(p1, p2), then the column contains min(mean(p1), 
mean(p2))), column Edge contains the name of the edge as a character
string of the form "p1-p2" and column Type contains the cluster name of the 
cluster that the edge belongs to as a character string.
}
\description{
Internal function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{ggmgsa_multisplit}
\alias{ggmgsa_multisplit}
\title{Multi-split GGMGSA (parallelized computation)}
\usage{
ggmgsa_multisplit(x1, x2, b.splits = 50, gene.sets, gene.names,
  gs.names = NULL, method.p.adjust = "fdr",
  order.adj.agg = "agg-adj", mc.flag = FALSE, mc.set.seed = TRUE,
  mc.preschedule = TRUE, mc.cores = getOption("mc.cores", 2L),
  verbose = TRUE, ...)
}
\arguments{
\item{x1}{Expression matrix for condition 1 (mean zero is required).}

\item{x2}{Expression matrix for condition 2 (mean zero is required).}

\item{b.splits}{Number of random data splits (default=50).}

\item{gene.sets}{List of gene-sets.}

\item{gene.names}{Gene names. Each column in x1 (and x2) corresponds
to a gene.}

\item{gs.names}{Gene-set names (default=NULL).}

\item{method.p.adjust}{Method for p-value adjustment (default='fdr').}

\item{order.adj.agg}{Order of aggregation and adjustment of p-values.
Options: 'agg-adj' (default), 'adj-agg'.}

\item{mc.flag}{If \code{TRUE} use parallel execution for each b.splits via function 
\code{mclapply} of package \code{parallel}.}

\item{mc.set.seed}{See mclapply. Default=TRUE}

\item{mc.preschedule}{See mclapply. Default=TRUE}

\item{mc.cores}{Number of cores to use in parallel execution. Defaults to
mc.cores option if set, or 2 otherwise.}

\item{verbose}{If TRUE, show output progess.}

\item{...}{Other arguments (see diffnet_singlesplit).}
}
\value{
List consisting of
\item{medagg.pval}{Median aggregated p-values}
\item{meinshagg.pval}{Meinshausen aggregated p-values}
\item{pval}{matrix of p-values before correction and adjustement, dim(pval)=(number of gene-sets)x(number of splits)}
\item{teststatmed}{median aggregated test-statistic}
\item{teststatmed.bic}{median aggregated bic-corrected test-statistic}
\item{teststatmed.aic}{median aggregated aic-corrected test-statistic}
\item{teststat}{matrix of test-statistics, dim(teststat)=(number of gene-sets)x(number of splits)}
\item{rel.edgeinter}{normalized intersection of edges in condition 1 and 2}
\item{df1}{degrees of freedom of GGM obtained from condition 1}
\item{df2}{degrees of freedom of GGM obtained from condition 2}
\item{df12}{degrees of freedom of GGM obtained from pooled data (condition 1 and 2)}
}
\description{
Multi-split GGMGSA (parallelized computation)
}
\details{
Computation can be parallelized over many data splits.
}
\examples{
#######################################################
##This example illustrates the use of GGMGSA         ##
#######################################################


## Generate networks
set.seed(1)
p <- 9#network with p nodes
n <- 40
hub.net <- generate_2networks(p,graph='hub',n.hub=3,n.hub.diff=1)#generate hub networks
invcov1 <- hub.net[[1]]
invcov2 <- hub.net[[2]]
plot_2networks(invcov1,invcov2,label.pos=0,label.cex=0.7)

## Generate data
library('mvtnorm')
x1 <- rmvnorm(n,mean = rep(0,p), sigma = cov2cor(solve(invcov1)))
x2 <- rmvnorm(n,mean = rep(0,p), sigma = cov2cor(solve(invcov2)))

## Run DiffNet
# fit.dn <- diffnet_multisplit(x1,x2,b.splits=2,verbose=FALSE)
# fit.dn$medagg.pval

## Identify hubs with 'gene-sets'
gene.names <- paste('G',1:p,sep='')
gsets <- split(gene.names,rep(1:3,each=3))

## Run GGM-GSA
fit.ggmgsa <- ggmgsa_multisplit(x1,x2,b.splits=2,gsets,gene.names,verbose=FALSE)
summary(fit.ggmgsa)
fit.ggmgsa$medagg.pval#median aggregated p-values
p.adjust(apply(fit.ggmgsa$pval,1,median),method='fdr')#or: first median aggregation,
                                                      #second fdr-correction






}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{ww.mat.diffregr}
\alias{ww.mat.diffregr}
\title{Computation M matrix and eigenvalues}
\usage{
ww.mat.diffregr(Sig, act, act1, act2)
}
\arguments{
\item{Sig}{no descr}

\item{act}{no descr}

\item{act1}{no descr}

\item{act2}{no descr}
}
\value{
no descr
}
\description{
Computation M matrix and eigenvalues
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auxfunc.R
\name{summary.diffregr}
\alias{summary.diffregr}
\title{Summary function for object of class 'diffregr'}
\usage{
\method{summary}{diffregr}(object, ...)
}
\arguments{
\item{object}{object of class 'diffregr}

\item{...}{Other arguments}
}
\value{
aggregated p-values
}
\description{
Summary function for object of class 'diffregr'
}
\author{
nicolas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixglasso.R
\name{MStepGlasso}
\alias{MStepGlasso}
\title{MStep of MixGLasso}
\usage{
MStepGlasso(x, chromosome = NULL, u, v = NULL, lambda, gamma, pen,
  penalize.diagonal, equal.prob.trans = NULL, term, model = "hmm")
}
\arguments{
\item{x}{no descr}

\item{chromosome}{no descr}

\item{u}{no descr}

\item{v}{no descr}

\item{lambda}{no descr}

\item{gamma}{no descr}

\item{pen}{no descr}

\item{penalize.diagonal}{no descr}

\item{equal.prob.trans}{no descr}

\item{term}{no descr}

\item{model}{no descr}
}
\value{
list consisting of mix.prob, Mu, Sig, SigInv
}
\description{
MStep of MixGLasso
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{screen_cvsqrt.lasso}
\alias{screen_cvsqrt.lasso}
\title{Cross-validated Lasso screening and sqrt-truncation.}
\usage{
screen_cvsqrt.lasso(x, y)
}
\arguments{
\item{x}{Predictor matrix.}

\item{y}{Response vector.}
}
\value{
Active-set.
}
\description{
Cross-validated Lasso screening and sqrt-truncation.
}
\details{
Computes Lasso coefficients (cross-validation optimal lambda). Truncates
smallest coefficients to zero, such that there are no more than sqrt(n)
non-zero coefficients.
}
\examples{
screen_cvsqrt.lasso(matrix(rnorm(5000),50,100),rnorm(50))
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auxfunc.R
\name{summary.ggmgsa}
\alias{summary.ggmgsa}
\title{Summary function for object of class 'ggmgsa'}
\usage{
\method{summary}{ggmgsa}(object, ...)
}
\arguments{
\item{object}{object of class 'ggmgsa'}

\item{...}{Other arguments}
}
\value{
aggregated p-values
}
\description{
Summary function for object of class 'ggmgsa'
}
\author{
nicolas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netsim.R
\name{sim_mix_networks}
\alias{sim_mix_networks}
\title{sim_mix_networks}
\usage{
sim_mix_networks(n, p, n.comp, sparsity = 0.7, mix.prob = rep(1/n.comp,
  n.comp), Mu = NULL, Sig = NULL, ...)
}
\arguments{
\item{n}{Number of data points to simulate.}

\item{p}{Dimensionality of the data.}

\item{n.comp}{Number of components of the mixture model.}

\item{sparsity}{Determines the proportion of non-zero off-diagonal entries.}

\item{mix.prob}{Mixture probabilities for the components; defaults to 
uniform distribution.}

\item{Mu}{Means for the mixture components, a p by n.comp matrix. If NULL, 
sampled from a standard Gaussian.}

\item{Sig}{Covariances for the mixture components, a p by p by n.comp 
array. If NULL,
generated using \code{\link{generate_inv_cov}}.}

\item{...}{Further arguments passed to \code{\link{sim_mix}}.}
}
\value{
A list with components:
\code{Mu} Means of the mixture components.
\code{Sig} Covariances of the mixture components.
\code{data} Simulated data, a n by p matrix.
\code{S} Component assignments, a vector of length n.
}
\description{
Generate inverse covariances, means, mixing probabilities, and simulate 
data from resulting mixture model.
}
\details{
This function generates n.comp mean vectors from a standard Gaussian and
n.comp covariance matrices, with at most (1-sparsity)*p(p-1)/2
non-zero off-diagonal entries, where the non-zero entries are sampled from a 
beta distribution. Then it uses \code{\link{sim_mix}} to simulate from a 
mixture model with these means and covariance matrices.

Means Mu and covariance matrices Sig can also be supplied by the user.
}
\examples{
# Generate dataset with 100 samples of dimensionality 30, and 4 components
test.data = sim_mix_networks(n=100, p=30, n.comp=4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{ggmgsa_singlesplit}
\alias{ggmgsa_singlesplit}
\title{Single-split GGMGSA}
\usage{
ggmgsa_singlesplit(x1, x2, gene.sets, gene.names,
  method.p.adjust = "fdr", verbose = TRUE, ...)
}
\arguments{
\item{x1}{centered (scaled) data for condition 1}

\item{x2}{centered (scaled) data for condition 2}

\item{gene.sets}{List of gene-sets.}

\item{gene.names}{Gene names. Each column in x1 (and x2) corresponds
to a gene.}

\item{method.p.adjust}{Method for p-value adjustment (default='fdr').}

\item{verbose}{If TRUE, show output progess.}

\item{...}{Other arguments (see diffnet_singlesplit).}
}
\value{
List of results.
}
\description{
Single-split GGMGSA
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{logratio}
\alias{logratio}
\title{Log-likelihood-ratio statistics used in DiffNet}
\usage{
logratio(x1, x2, x, sig1, sig2, sig, mu1, mu2, mu)
}
\arguments{
\item{x1}{data-matrix sample 1}

\item{x2}{data-matrix sample 2}

\item{x}{pooled data-matrix}

\item{sig1}{covariance sample 1}

\item{sig2}{covariance sample 2}

\item{sig}{pooled covariance}

\item{mu1}{mean sample 1}

\item{mu2}{mean sample 2}

\item{mu}{pooled mean}
}
\value{
Returns a list with named elements 'twiceLR', 'sig1', 'sig2', 'sig'.
        'twiceLR' is twice the log-likelihood-ratio statistic.
}
\description{
Log-likelihood-ratio statistics used in Differential Network
}
\examples{
x1=matrix(rnorm(100),50,2)
x2=matrix(rnorm(100),50,2)
logratio(x1,x2,rbind(x1,x2),diag(1,2),diag(1,2),diag(1,2),c(0,0),c(0,0),c(0,0))$twiceLR
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auxfunc.R
\name{print.nethetsummary}
\alias{print.nethetsummary}
\title{Print function for object of class 'nethetsummmary'}
\usage{
\method{print}{nethetsummary}(x, ...)
}
\arguments{
\item{x}{object of class 'nethetsummary'}

\item{...}{Other arguments}
}
\value{
Function does not return anything.
}
\description{
Print function for object of class 'nethetsummary'
}
\author{
frankd
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{q.matrix4}
\alias{q.matrix4}
\title{q.matrix4}
\usage{
q.matrix4(b.mat, act.a, act.b, ss)
}
\arguments{
\item{b.mat}{no descr}

\item{act.a}{no descr}

\item{act.b}{no descr}

\item{ss}{no descr}
}
\value{
no descr
}
\description{
q.matrix4
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{t2cov.lr}
\alias{t2cov.lr}
\title{Classical likelihood-ratio test}
\usage{
t2cov.lr(x1, x2, include.mean = FALSE)
}
\arguments{
\item{x1}{no descr}

\item{x2}{no descr}

\item{include.mean}{no descr}
}
\value{
no descr
}
\description{
Classical likelihood-ratio test (equality of covariance matrices)
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixglasso.R
\name{EXPStep.mix}
\alias{EXPStep.mix}
\title{Performs EStep}
\usage{
EXPStep.mix(logphi, mix.prob)
}
\arguments{
\item{logphi}{no descr}

\item{mix.prob}{no descr}
}
\value{
list consiting of
\item{u}{responsibilities}
\item{LL}{loglikelihood}
}
\description{
Performs EStep
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netsim.R
\name{generate_2networks}
\alias{generate_2networks}
\title{Generate sparse invcov with overlap}
\usage{
generate_2networks(p, graph = "random", n.nz = rep(p, 2),
  n.nz.common = p, n.hub = 2, n.hub.diff = 1, magn.nz.diff = 0.8,
  magn.nz.common = 0.9, magn.diag = 0, emin = 0.1, verbose = FALSE)
}
\arguments{
\item{p}{number of nodes}

\item{graph}{'random' or 'hub'}

\item{n.nz}{number of edges per graph (only for graph='random')}

\item{n.nz.common}{number of edges incommon between graphs (only for 
graph='random')}

\item{n.hub}{number of hubs (only for graph='hub')}

\item{n.hub.diff}{number of different hubs}

\item{magn.nz.diff}{default=0.9}

\item{magn.nz.common}{default=0.9}

\item{magn.diag}{default=0}

\item{emin}{default=0.1 (see ?huge.generator)}

\item{verbose}{If verbose=FALSE then tracing output is disabled.}
}
\value{
Two sparse inverse covariance matrices with overlap
}
\description{
Generate two sparse inverse covariance matrices with overlap
}
\examples{
n <- 70
p <- 30

## Specifiy sparse inverse covariance matrices,
## with number of edges in common equal to ~ 0.8*p
gen.net <- generate_2networks(p,graph='random',n.nz=rep(p,2),
                              n.nz.common=ceiling(p*0.8))

invcov1 <- gen.net[[1]]
invcov2 <- gen.net[[2]]

plot_2networks(invcov1,invcov2,label.pos=0,label.cex=0.7)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{gsea.iriz.scale}
\alias{gsea.iriz.scale}
\title{Irizarry approach (scale only)}
\usage{
gsea.iriz.scale(x1, x2, gene.sets, gene.names, gs.names = NULL,
  method.p.adjust = "fdr", alternative = "two-sided")
}
\arguments{
\item{x1}{no descr}

\item{x2}{no descr}

\item{gene.sets}{no descr}

\item{gene.names}{no descr}

\item{gs.names}{no descr}

\item{method.p.adjust}{no descr}

\item{alternative}{no descr}
}
\value{
no descr
}
\description{
Irizarry approach (scale only)
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{diffregr_singlesplit}
\alias{diffregr_singlesplit}
\title{Differential Regression (single-split version).}
\usage{
diffregr_singlesplit(y1, y2, x1, x2, split1, split2,
  screen.meth = "screen_cvtrunc.lasso",
  compute.evals = "est2.my.ev3.diffregr",
  method.compquadform = "imhof", acc = 1e-04, epsabs = 1e-10,
  epsrel = 1e-10, show.warn = FALSE, n.perm = NULL, ...)
}
\arguments{
\item{y1}{Response vector condition 1.}

\item{y2}{Response vector condition 2.}

\item{x1}{Predictor matrix condition 1.}

\item{x2}{Predictor matrix condition 2.}

\item{split1}{Samples condition 1 used in screening-step.}

\item{split2}{Samples condition 2 used in screening-step.}

\item{screen.meth}{Screening method (default='screen_cvtrunc.lasso').}

\item{compute.evals}{Method to estimate the weights in the weighted-sum-of-chi2s distribution.
The default and (currently) the only available option 
is the method 'est2.my.ev3.diffregr'.}

\item{method.compquadform}{Algorithm for computing distribution function
of weighted-sum-of-chi2 (default='imhof').}

\item{acc}{See ?davies (default=1e-4).}

\item{epsabs}{See ?imhof (default=1e-10).}

\item{epsrel}{See ?imhof (default=1e-10).}

\item{show.warn}{Show warnings (default=FALSE)?}

\item{n.perm}{Number of permutation for "split-perm" p-value (default=NULL).}

\item{...}{Other arguments specific to screen.meth.}
}
\value{
List consisting of
\item{pval.onesided}{"One-sided" p-value.}
\item{pval.twosided}{"Two-sided" p-value. Ignore all "*.twosided results.}
\item{teststat}{2 times Log-likelihood-ratio statistics}
\item{weights.nulldistr}{Estimated weights of weighted-sum-of-chi2s.}
\item{active}{List of active-sets obtained in screening step.}
\item{beta}{Regression coefficients (MLE) obtaind in cleaning-step.}
}
\description{
Differential Regression (single-split version).
}
\details{
Intercepts in regression models are assumed to be zero (mu1=mu2=0).
You might need to center the input data prior to running
Differential Regression.
}
\examples{

##set seed
set.seed(1)

##number of predictors / sample size
p <- 100
n <- 80

##predictor matrices
x1 <- matrix(rnorm(n*p),n,p)
x2 <- matrix(rnorm(n*p),n,p)

##active-sets and regression coefficients
act1 <- sample(1:p,5)
act2 <- c(act1[1:3],sample(setdiff(1:p,act1),2))
beta1 <- beta2 <- rep(0,p)
beta1[act1] <- 0.5
beta2[act2] <- 0.5

##response vectors 
y1 <- x1\%*\%as.matrix(beta1)+rnorm(n,sd=1)
y2 <- x2\%*\%as.matrix(beta2)+rnorm(n,sd=1)

##run diffregr
split1 <- sample(1:n,50)#samples for screening (condition 1)
split2 <- sample(1:n,50)#samples for screening (condition 2)
fit <- diffregr_singlesplit(y1,y2,x1,x2,split1,split2)
fit$pval.onesided#p-value


}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netplot.R
\name{plot.diffnet}
\alias{plot.diffnet}
\title{Plotting function for object of class 'diffnet'}
\usage{
\method{plot}{diffnet}(x, ...)
}
\arguments{
\item{x}{object of class 'diffnet'}

\item{...}{Further arguments.}
}
\value{
Histogram over multi-split p-values.
}
\description{
Plotting function for object of class 'diffnet'
}
\author{
nicolas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netplot.R
\name{scatter_plot}
\alias{scatter_plot}
\title{Create a scatterplot showing correlation between specific nodes in the network
for each pre-specified group.}
\usage{
scatter_plot(net.clustering, data, node.pairs, display = TRUE,
  node.names = rownames(net.clustering$Mu),
  group.names = sort(unique(net.clustering$comp)), cex = 1)
}
\arguments{
\item{net.clustering}{A network clustering object as returned by 
\code{\link{het_cv_glasso}} or \code{\link{mixglasso}}.}

\item{data}{Observed data for the nodes, a numObs by numNodes matrix. Note 
that nodes need to be in the same ordering as in node.names.}

\item{node.pairs}{A matrix of size numPairs by 2, where each row contains a 
pair of nodes to display. If node.names is specified, names in node.pairs
must correspond to elements of node.names.}

\item{display}{If TRUE, print the plot to the current output device.}

\item{node.names}{Names for the nodes in the network. If NULL, names from 
net.clustering will be used.}

\item{group.names}{Names for the clusters or groups. If NULL, names from 
net.clustering will be used (by default these are integets 1:numClusters).}

\item{cex}{Scale factor for text and symbols in plot.}
}
\value{
Returns a ggplot2 object. If display=TRUE, additionally displays the 
plot.
}
\description{
This function takes the output of \code{\link{het_cv_glasso}} or 
\code{\link{mixglasso}} and creates a plot showing the correlation between specified
node pairs in the network for all groups. The subplots for each node pair are 
arranged in a numPairs by numGroups grid. Partial correlations associated 
with each node pair are also displayed.
}
\examples{
n = 500
p = 10
s = 0.9
n.comp = 3

# Create different mean vectors
Mu = matrix(0,p,n.comp)

# Define non-zero means in each group (non-overlapping)
nonzero.mean = split(sample(1:p),rep(1:n.comp,length=p))

# Set non-zero means to fixed value
for(k in 1:n.comp){
	Mu[nonzero.mean[[k]],k] = -2/sqrt(ceiling(p/n.comp))
}

# Generate data
sim.result = sim_mix_networks(n, p, n.comp, s, Mu=Mu)
mixglasso.result = mixglasso(sim.result$data, n.comp=3)
mixglasso.clustering = mixglasso.result$models[[mixglasso.result$bic.opt]]

# Specify edges
node.pairs = rbind(c(1,3), c(6,9),c(7,8))

# Create scatter plots of specified edges
scatter_plot(mixglasso.clustering, data=sim.result$data,
					 node.pairs=node.pairs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{est2.my.ev2}
\alias{est2.my.ev2}
\title{Weights of sum-w-chi2}
\usage{
est2.my.ev2(sig1, sig2, sig, act1, act2, act, include.mean = FALSE)
}
\arguments{
\item{sig1}{no descr}

\item{sig2}{no descr}

\item{sig}{no descr}

\item{act1}{no descr}

\item{act2}{no descr}

\item{act}{no descr}

\item{include.mean}{no descr}
}
\value{
no descr
}
\description{
Compute weights of sum-w-chi2 (2nd order simplification)
}
\details{
*expansion of W in two directions ("dimf>dimg direction" & "dimf>dimg direction") 
*simplified computation of weights is obtained by assuming H0 and that X_u~X_v holds
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{gsea.iriz}
\alias{gsea.iriz}
\title{Irizarry approach for gene-set testing}
\usage{
gsea.iriz(x1, x2, gene.sets, gene.names, gs.names = NULL,
  method.p.adjust = "fdr", alternative = "two-sided")
}
\arguments{
\item{x1}{Expression matrix (condition 1)}

\item{x2}{Expression matrix (condition 2)}

\item{gene.sets}{List of gene-sets}

\item{gene.names}{Gene names}

\item{gs.names}{Gene-set names}

\item{method.p.adjust}{Method for p-value adjustment (default='fdr')}

\item{alternative}{Default='two-sided' (uses two-sided p-values).}
}
\value{
List consisting of
\item{pval.shift}{p-values measuring shift}
\item{pval.scale}{p-values measuring scale}
\item{pval.combined}{combined p-values (minimum of pval.shift and pval.scale)}
}
\description{
Irizarry approach for gene-set testing
}
\details{
Implements the approach described in
"Gene set enrichment analysis made simple" by Irizarry et al (2011).
It tests for shift and/or change in scale of the distribution.
}
\examples{
n <- 100
p <- 20
x1 <- matrix(rnorm(n*p),n,p)
x2 <- matrix(rnorm(n*p),n,p)
gene.names <- paste('G',1:p,sep='')
gsets <- split(gene.names,rep(1:4,each=5))
fit <- gsea.iriz(x1,x2,gsets,gene.names)
fit$pvals.combined

x2[,1:3] <- x2[,1:3]+0.5#variables 1-3 of first gene-set are upregulated
fit <- gsea.iriz(x1,x2,gsets,gene.names)
fit$pvals.combined
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixglasso.R
\name{w.kldist}
\alias{w.kldist}
\title{Distance between comps based on symm. kl-distance}
\usage{
w.kldist(Mu, Sig)
}
\arguments{
\item{Mu}{no descr}

\item{Sig}{no descr}
}
\value{
list consisting of
\item{comp.kldist}{}
\item{min.comp.kldist}{}
}
\description{
Distance between comps based on symm. kl-distance
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{agg.score.iriz.scale}
\alias{agg.score.iriz.scale}
\title{Irizarry aggregate score (scale)}
\usage{
agg.score.iriz.scale(ttstat, geneset, gene.name)
}
\arguments{
\item{ttstat}{no descr}

\item{geneset}{no descr}

\item{gene.name}{no descr}
}
\value{
no descr
}
\description{
Irizarry aggregate score (scale)
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixglasso.R
\name{loglik_mix}
\alias{loglik_mix}
\title{Log-likelihood for mixture model}
\usage{
loglik_mix(x, mix.prob, Mu, Sig)
}
\arguments{
\item{x}{no descr}

\item{mix.prob}{no descr}

\item{Mu}{no descr}

\item{Sig}{no descr}
}
\value{
log-likelihood
}
\description{
Log-likelihood for mixture model
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{est2.my.ev3}
\alias{est2.my.ev3}
\title{Compute weights of sum-of-weighted-chi2s}
\usage{
est2.my.ev3(sig1, sig2, sig, act1, act2, act, include.mean = FALSE)
}
\arguments{
\item{sig1}{MLE (covariance matrix) sample 1}

\item{sig2}{MLE (covariance matrix) sample 2}

\item{sig}{Pooled MLE (covariance matrix)}

\item{act1}{Active-set sample 1}

\item{act2}{Active-set sample 2}

\item{act}{Pooled active-set}

\item{include.mean}{Should the mean be in cluded in the likelihood?}
}
\value{
Eigenvalues of M, respectively the weights.
}
\description{
Compute weights of sum-of-weighted-chi2s
}
\details{
*'2nd order simplification':
  1) Factor out (1-vi)^(d1+d2) "expansion in dimf>dimg direction (old terminology)"
  2) Factor out (1-mu)^d0 
*simplified computation of weights is obtained without further invoking H0, or assuming X_u~X_v
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{screen_cv.glasso}
\alias{screen_cv.glasso}
\title{Cross-validated glasso with additional thresholding}
\usage{
screen_cv.glasso(x, include.mean = FALSE, folds = min(10, dim(x)[1]),
  length.lambda = 20, lambdamin.ratio = ifelse(ncol(x) > nrow(x), 0.01,
  0.001), penalize.diagonal = FALSE, trunc.method = "linear.growth",
  trunc.k = 5, plot.it = FALSE, se = FALSE, use.package = "huge",
  verbose = FALSE)
}
\arguments{
\item{x}{The input data. Needs to be a num.samples by dim.samples matrix.}

\item{include.mean}{Include mean in likelihood. TRUE / FALSE (default).}

\item{folds}{Number of folds in the cross-validation (default=10).}

\item{length.lambda}{Length of lambda path to consider (default=20).}

\item{lambdamin.ratio}{Ratio lambda.min/lambda.max.}

\item{penalize.diagonal}{If TRUE apply penalization to diagonal of inverse
covariance as well. (default=FALSE)}

\item{trunc.method}{None / linear.growth (default) / sqrt.growth}

\item{trunc.k}{truncation constant, number of samples per predictor (default=5)}

\item{plot.it}{TRUE / FALSE (default)}

\item{se}{default=FALSE.}

\item{use.package}{'glasso' or 'huge' (default).}

\item{verbose}{If TRUE, output la.min, la.max and la.opt (default=FALSE).}
}
\value{
Returns a list with named elements 'rho.opt', 'w', 'wi', 'wi.orig', 
         'mu'. Variable rho.opt is the optimal (scaled) penalization 
         parameter (rho.opt=2*la.opt/n). Variable w is the estimated 
         covariance matrix. The variables wi and wi.orig are 
         matrices of size dim.samples by dim.samples containing the 
         truncated and untruncated inverse covariance matrix. Variable 
         mu is the mean of the input data.
}
\description{
Cross-validated glasso with additional thresholding
}
\details{
Run glasso on a single dataset, using cross-validation to estimate the
penalty parameter lambda. Performs additional thresholding (optionally).
}
\examples{
n=50
p=5
x=matrix(rnorm(n*p),n,p)
wihat=screen_cv.glasso(x,folds=2)$wi
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{mle.ggm}
\alias{mle.ggm}
\title{MLE in GGM}
\usage{
mle.ggm(x, wi, algorithm = "glasso_rho0", rho = NULL, include.mean)
}
\arguments{
\item{x}{no descr}

\item{wi}{no descr}

\item{algorithm}{no descr}

\item{rho}{no descr}

\item{include.mean}{no descr}
}
\value{
no descr
}
\description{
MLE in GGM
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auxfunc.R
\name{summary.diffnet}
\alias{summary.diffnet}
\title{Summary function for object of class 'diffnet'}
\usage{
\method{summary}{diffnet}(object, ...)
}
\arguments{
\item{object}{object of class 'diffnet'}

\item{...}{Other arguments.}
}
\value{
aggregated p-values
}
\description{
Summary function for object of class 'diffnet'
}
\author{
nicolas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auxfunc.R
\name{invcov2parcor}
\alias{invcov2parcor}
\title{Convert inverse covariance to partial correlation}
\usage{
invcov2parcor(invcov)
}
\arguments{
\item{invcov}{Inverse covariance matrix}
}
\value{
The partial correlation matrix.
}
\description{
Convert inverse covariance to partial correlation
}
\examples{
inv.cov = generate_inv_cov(p=25)
p.corr = invcov2parcor(inv.cov)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{cv.glasso}
\alias{cv.glasso}
\title{Crossvalidation for GLasso}
\usage{
cv.glasso(x, folds = 10, lambda, penalize.diagonal = FALSE,
  plot.it = FALSE, se = TRUE, include.mean = FALSE)
}
\arguments{
\item{x}{no descr}

\item{folds}{no descr}

\item{lambda}{lambda-grid (increasing!)}

\item{penalize.diagonal}{no descr}

\item{plot.it}{no descr}

\item{se}{no descr}

\item{include.mean}{no descr}
}
\value{
no descr
}
\description{
Crossvalidation for GLasso
}
\details{
8! lambda-grid has to be increasing (see glassopath)
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{mcov}
\alias{mcov}
\title{Compute covariance matrix}
\usage{
mcov(x, include.mean, covMethod = "ML")
}
\arguments{
\item{x}{no descr}

\item{include.mean}{no descr}

\item{covMethod}{no descr}
}
\value{
no descr
}
\description{
Compute covariance matrix
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{est2.my.ev3.diffregr}
\alias{est2.my.ev3.diffregr}
\title{Compute weights of sum-of-weighted-chi2s}
\usage{
est2.my.ev3.diffregr(y1, y2, x1, x2, beta1, beta2, beta, act1, act2, act)
}
\arguments{
\item{y1}{Response vector sample 1.}

\item{y2}{Response vector sample 2.}

\item{x1}{Predictor matrix sample 1.}

\item{x2}{Predictor matrix sample 2.}

\item{beta1}{MLE (regression coefficients) sample 1.}

\item{beta2}{MLE (regression coefficients) sample 2.}

\item{beta}{Pooled MLE (regression coefficients).}

\item{act1}{Active-set sample 1}

\item{act2}{Active-set sample 2}

\item{act}{Pooled active-set}
}
\value{
Eigenvalues of M, respectively the weights.
}
\description{
Compute weights of sum-of-weighted-chi2s
}
\details{
*'2nd order simplification':
  1) Factor out (1-vi)^(d1+d2) "expansion in dimf>dimg direction (old terminology)"
  2) Factor out (1-mu)^d0 
*simplified computation of weights is obtained without further invoking H0, or assuming X_u~X_v
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auxfunc.R
\name{summary.nethetclustering}
\alias{summary.nethetclustering}
\title{Summary function for object of class 'nethetclustering'}
\usage{
\method{summary}{nethetclustering}(object, ...)
}
\arguments{
\item{object}{object of class 'nethetclustering'}

\item{...}{Other arguments}
}
\value{
Network statistics (a 'nethetsummary' object)
}
\description{
Summary function for object of class 'nethetclustering'
}
\author{
frankd
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{make_grid}
\alias{make_grid}
\title{Make grid}
\usage{
make_grid(lambda.min, lambda.max, nr.gridpoints,
  method = "lambdagrid_mult")
}
\arguments{
\item{lambda.min}{no descr}

\item{lambda.max}{no descr}

\item{nr.gridpoints}{no descr}

\item{method}{no descr}
}
\value{
no descr
}
\description{
Make grid
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{est2.ww.mat2}
\alias{est2.ww.mat2}
\title{Weights of sum-w-chi2}
\usage{
est2.ww.mat2(sig1, sig2, sig, act1, act2, act, include.mean = FALSE)
}
\arguments{
\item{sig1}{no descr}

\item{sig2}{no descr}

\item{sig}{no descr}

\item{act1}{no descr}

\item{act2}{no descr}

\item{act}{no descr}

\item{include.mean}{no descr}
}
\value{
no descr
}
\description{
Compute weights of sum-w-chi2 (1st order simplification)
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{screen_cv1se.lasso}
\alias{screen_cv1se.lasso}
\title{Cross-validated Lasso screening (lambda.1se-rule)}
\usage{
screen_cv1se.lasso(x, y)
}
\arguments{
\item{x}{Predictor matrix}

\item{y}{Response vector}
}
\value{
Active-set
}
\description{
Cross-validated Lasso screening (lambda.1se-rule)
}
\examples{
screen_cv1se.lasso(matrix(rnorm(5000),50,100),rnorm(50))
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{mytrunc.method}
\alias{mytrunc.method}
\title{Additional thresholding}
\usage{
mytrunc.method(n, wi, method = "linear.growth", trunc.k = 5)
}
\arguments{
\item{n}{no descr}

\item{wi}{no descr}

\item{method}{no descr}

\item{trunc.k}{no descr}
}
\value{
no descr
}
\description{
Additional thresholding
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{beta.mat}
\alias{beta.mat}
\title{Compute beta-matrix}
\usage{
beta.mat(ind1, ind2, sig1, sig2, sig)
}
\arguments{
\item{ind1}{no descr}

\item{ind2}{no descr}

\item{sig1}{no descr}

\item{sig2}{no descr}

\item{sig}{no descr}
}
\value{
no descr
}
\description{
Compute beta-matrix
}
\details{
beta-matrix=E[s_ind1(Y;sig1) s_ind2(Y;sig2)'|sig]
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{ww.mat2}
\alias{ww.mat2}
\title{Calculates eigenvalues of weight-matrix (using 1st order simplification)}
\usage{
ww.mat2(imat, act, act1, act2)
}
\arguments{
\item{imat}{no descr}

\item{act}{I_uv}

\item{act1}{I_u}

\item{act2}{I_v}
}
\value{
no descr
}
\description{
Calculates eigenvalues of weight-matrix (using 1st order simplification)
}
\details{
calculation based on true information matrix
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{lambdagrid_mult}
\alias{lambdagrid_mult}
\title{Lambda-grid}
\usage{
lambdagrid_mult(lambda.min, lambda.max, nr.gridpoints)
}
\arguments{
\item{lambda.min}{no descr}

\item{lambda.max}{no descr}

\item{nr.gridpoints}{no descr}
}
\value{
no descr
}
\description{
Lambda-grid (log scale)
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{diffregr_multisplit}
\alias{diffregr_multisplit}
\title{Differential Regression (multi-split version).}
\usage{
diffregr_multisplit(y1, y2, x1, x2, b.splits = 50, frac.split = 1/2,
  screen.meth = "screen_cvtrunc.lasso", gamma.min = 0.05,
  compute.evals = "est2.my.ev3.diffregr",
  method.compquadform = "imhof", acc = 1e-04, epsabs = 1e-10,
  epsrel = 1e-10, show.warn = FALSE, n.perm = NULL,
  mc.flag = FALSE, mc.set.seed = TRUE, mc.preschedule = TRUE,
  mc.cores = getOption("mc.cores", 2L), ...)
}
\arguments{
\item{y1}{Response vector condition 1.}

\item{y2}{Response vector condition 2.}

\item{x1}{Predictor matrix condition 1.}

\item{x2}{Predictor matrix condition 2.}

\item{b.splits}{Number of splits (default=50).}

\item{frac.split}{Fraction train-data (screening) / test-data (cleaning) (default=0.5).}

\item{screen.meth}{Screening method (default='screen_cvtrunc.lasso').}

\item{gamma.min}{Tuning parameter in p-value aggregation of Meinshausen et al (2009) (default=0.05).}

\item{compute.evals}{Method to estimate the weights in the weighted-sum-of-chi2s distribution.
The default and (currently) the only available option 
is the method 'est2.my.ev3.diffregr'.}

\item{method.compquadform}{Algorithm for computing distribution function
of weighted-sum-of-chi2 (default='imhof').}

\item{acc}{See ?davies (default=1e-4).}

\item{epsabs}{See ?imhof (default=1e-10).}

\item{epsrel}{See ?imhof (default=1e-10).}

\item{show.warn}{Show warnings (default=FALSE)?}

\item{n.perm}{Number of permutation for "split-perm" p-value. Default=NULL, which means
that the asymptotic approximation is used.}

\item{mc.flag}{If \code{TRUE} use parallel execution for each b.splits via function 
\code{mclapply} of package \code{parallel}.}

\item{mc.set.seed}{See mclapply. Default=TRUE}

\item{mc.preschedule}{See mclapply. Default=TRUE}

\item{mc.cores}{Number of cores to use in parallel execution. Defaults to
mc.cores option if set, or 2 otherwise.}

\item{...}{Other arguments specific to screen.meth.}
}
\value{
List consisting of
\item{ms.pval}{p-values for all b.splits}
\item{ss.pval}{single-split p-value}
\item{medagg.pval}{median aggregated p-value}
\item{meinshagg.pval}{meinshausen aggregated p-value (meinshausen et al 2009)}
\item{teststat}{test statistics for b.splits}
\item{weights.nulldistr}{estimated weights}
\item{active.last}{active-sets obtained in last screening-step}
\item{beta.last}{constrained mle (regression coefficients) obtained in last cleaning-step}
}
\description{
Differential Regression (multi-split version).
}
\details{
Intercepts in regression models are assumed to be zero (mu1=mu2=0).
You might need to center the input data prior to running
Differential Regression.
}
\examples{
###############################################################
##This example illustrates the use of Differential Regression##
###############################################################

##set seed
set.seed(1)

## Number of predictors and sample size
p <- 100
n <- 80

## Predictor matrices
x1 <- matrix(rnorm(n*p),n,p)
x2 <- matrix(rnorm(n*p),n,p)

## Active-sets and regression coefficients
act1 <- sample(1:p,5)
act2 <- c(act1[1:3],sample(setdiff(1:p,act1),2))
beta1 <- beta2 <- rep(0,p)
beta1[act1] <- 0.5
beta2[act2] <- 0.5

## Response vectors under null-hypothesis
y1 <- x1\%*\%as.matrix(beta1)+rnorm(n,sd=1)
y2 <- x2\%*\%as.matrix(beta1)+rnorm(n,sd=1)

## Diffregr (asymptotic p-values)
fit.null <- diffregr_multisplit(y1,y2,x1,x2,b.splits=5)
fit.null$ms.pval#multi-split p-values
fit.null$medagg.pval#median aggregated p-values

## Response vectors under alternative-hypothesis
y1 <- x1\%*\%as.matrix(beta1)+rnorm(n,sd=1)
y2 <- x2\%*\%as.matrix(beta2)+rnorm(n,sd=1)

## Diffregr (asymptotic p-values)
fit.alt <- diffregr_multisplit(y1,y2,x1,x2,b.splits=5)
fit.alt$ms.pval
fit.alt$medagg.pval

## Diffregr (permutation-based p-values; 100 permutations)
fit.alt.perm <- diffregr_multisplit(y1,y2,x1,x2,b.splits=5,n.perm=100)
fit.alt.perm$ms.pval
fit.alt.perm$medagg.pval


}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{q.matrix.diffregr4}
\alias{q.matrix.diffregr4}
\title{Computation Q matrix}
\usage{
q.matrix.diffregr4(b.mat, act.a, act.b, ss)
}
\arguments{
\item{b.mat}{no descr}

\item{act.a}{no descr}

\item{act.b}{no descr}

\item{ss}{no descr}
}
\value{
no descr
}
\description{
Computation Q matrix
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{agg.pval}
\alias{agg.pval}
\title{P-value aggregation (Meinshausen et al 2009)}
\usage{
agg.pval(gamma, pval)
}
\arguments{
\item{gamma}{see Meinshausen et al 2009}

\item{pval}{vector of p-values}
}
\value{
inf-quantile aggregated p-value
}
\description{
P-value aggregation
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netsim.R
\name{generate_inv_cov}
\alias{generate_inv_cov}
\title{generate_inv_cov}
\usage{
generate_inv_cov(p = 162, sparsity = 0.7)
}
\arguments{
\item{p}{Dimensionality of the matrix.}

\item{sparsity}{Determined the proportion of non-zero off-diagonal entries.}
}
\value{
A p by p positive definite inverse covariance matrix.
}
\description{
Generate an inverse covariance matrix with a given sparsity and 
dimensionality
}
\details{
This function generates an inverse covariance matrix, with at most 
(1-sparsity)*p(p-1)
non-zero off-diagonal entries, where the non-zero entries are sampled from a 
beta distribution.
}
\examples{
generate_inv_cov(p=162)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{est2.my.ev2.diffregr}
\alias{est2.my.ev2.diffregr}
\title{Compute weights of sum-w-chi2 (2nd order simplification)}
\usage{
est2.my.ev2.diffregr(y1, y2, x1, x2, beta1, beta2, beta, act1, act2, act)
}
\arguments{
\item{y1}{no descr}

\item{y2}{no descr}

\item{x1}{no descr}

\item{x2}{no descr}

\item{beta1}{no descr}

\item{beta2}{no descr}

\item{beta}{no descr}

\item{act1}{no descr}

\item{act2}{no descr}

\item{act}{no descr}
}
\value{
no descr
}
\description{
*expansion of W in two directions ("dimf>dimg direction" & "dimf>dimg direction") 
*simplified computation of weights is obtained by assuming H0 and that X_u~X_v holds
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{plotCV}
\alias{plotCV}
\title{plotCV}
\usage{
plotCV(lambda, cv, cv.error, se = TRUE, type = "b", ...)
}
\arguments{
\item{lambda}{no descr}

\item{cv}{no descr}

\item{cv.error}{no descr}

\item{se}{no descr}

\item{type}{no descr}

\item{...}{no descr}
}
\value{
no descr
}
\description{
plotCV
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{aggpval}
\alias{aggpval}
\title{Meinshausen p-value aggregation}
\usage{
aggpval(pval, gamma.min = 0.05)
}
\arguments{
\item{pval}{Vector of p-values.}

\item{gamma.min}{See inf-quantile formula of Meinshausen et al 2009 (default=0.05).}
}
\value{
Aggregated p-value.
}
\description{
Meinshausen p-value aggregation.
}
\details{
Inf-quantile formula for p-value aggregation presented in Meinshausen et al 2009.
}
\examples{
pval=runif(50)
aggpval(pval)
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixglasso.R
\name{glasso.invcor}
\alias{glasso.invcor}
\title{Graphical Lasso based on inverse covariance penalty}
\usage{
glasso.invcor(s, rho, penalize.diagonal, term = 10^{     -3 })
}
\arguments{
\item{s}{no descr}

\item{rho}{no descr}

\item{penalize.diagonal}{no descr}

\item{term}{no descr}
}
\value{
w; wi; iter
}
\description{
Graphical Lasso based on inverse covariance penalty
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{beta.mat.diffregr}
\alias{beta.mat.diffregr}
\title{Computation beta matrix}
\usage{
beta.mat.diffregr(ind1, ind2, beta1, beta2, beta, sig1, sig2, sig, Sig)
}
\arguments{
\item{ind1}{no descr}

\item{ind2}{no descr}

\item{beta1}{no descr}

\item{beta2}{no descr}

\item{beta}{no descr}

\item{sig1}{no descr}

\item{sig2}{no descr}

\item{sig}{no descr}

\item{Sig}{no descr}
}
\value{
no descr
}
\description{
Computation beta matrix
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{diffnet_singlesplit}
\alias{diffnet_singlesplit}
\title{Differential Network for user specified data splits}
\usage{
diffnet_singlesplit(x1, x2, split1, split2,
  screen.meth = "screen_bic.glasso", compute.evals = "est2.my.ev3",
  algorithm.mleggm = "glasso_rho0", include.mean = FALSE,
  method.compquadform = "imhof", acc = 1e-04, epsabs = 1e-10,
  epsrel = 1e-10, show.warn = FALSE, save.mle = FALSE, ...)
}
\arguments{
\item{x1}{Data-matrix sample 1.
You might need to center and scale your data-matrix.}

\item{x2}{Data-matrix sample 2.
You might need to center and scale your data-matrix.}

\item{split1}{Samples (condition 1) used in screening step.}

\item{split2}{Samples (condition 2) used in screening step.}

\item{screen.meth}{Screening procedure. Options: 'screen_bic.glasso' (default),
'screen_cv.glasso', 'screen_shrink' (not recommended).}

\item{compute.evals}{Method to estimate the weights in the weighted-sum-of-chi2s distribution.
The default and (currently) the only available option 
is the method 'est2.my.ev3'.}

\item{algorithm.mleggm}{Algorithm to compute MLE of GGM. The algorithm 'glasso_rho' is the
default and (currently) the only available option.}

\item{include.mean}{Should sample specific means be included in hypothesis?
Use include.mean=FALSE (default and recommended) which assumes mu1=mu2=0
and tests the hypothesis H0: Omega_1=Omega_2.}

\item{method.compquadform}{Method to compute distribution function of weighted-sum-of-chi2s
(default='imhof').}

\item{acc}{See ?davies (default 1e-04).}

\item{epsabs}{See ?imhof (default 1e-10).}

\item{epsrel}{See ?imhof (default 1e-10).}

\item{show.warn}{Should warnings be showed (default=FALSE)?}

\item{save.mle}{Should MLEs be in the output list (default=FALSE)?}

\item{...}{Additional arguments for screen.meth.}
}
\value{
list consisting of
\item{pval.onesided}{p-value}
\item{pval.twosided}{ignore this output}
\item{teststat}{log-likelihood-ratio test statistic}
\item{weights.nulldistr}{estimated weights}
\item{active}{active-sets obtained in screening-step}
\item{sig}{constrained mle (covariance) obtained in cleaning-step}
\item{wi}{constrained mle (inverse covariance) obtained in cleaning-step}
\item{mu}{mle (mean) obtained in cleaning-step}
}
\description{
Differential Network for user specified data splits
}
\details{
Remark:

* If include.mean=FALSE, then x1 and x2 have mean zero and DiffNet tests
  the hypothesis H0: Omega_1=Omega_2. You might need to center x1 and x2.
* If include.mean=TRUE, then DiffNet tests the hypothesis
  H0: mu_1=mu_2 & Omega_1=Omega_2
* However, we recommend to set include.mean=FALSE and to test equality of the means
  separately. 
* You might also want to scale x1 and x2, if you are only interested in 
  differences due to (partial) correlations.
}
\examples{

##set seed
set.seed(1)

##sample size and number of nodes
n <- 40
p <- 10

##specifiy sparse inverse covariance matrices
gen.net <- generate_2networks(p,graph='random',n.nz=rep(p,2),
                              n.nz.common=ceiling(p*0.8))
invcov1 <- gen.net[[1]]
invcov2 <- gen.net[[2]]
plot_2networks(invcov1,invcov2,label.pos=0,label.cex=0.7)

##get corresponding correlation matrices
cor1 <- cov2cor(solve(invcov1))
cor2 <- cov2cor(solve(invcov2))

##generate data under alternative hypothesis
library('mvtnorm')
x1 <- rmvnorm(n,mean = rep(0,p), sigma = cor1)
x2 <- rmvnorm(n,mean = rep(0,p), sigma = cor2)

##run diffnet
split1 <- sample(1:n,20)#samples for screening (condition 1)
split2 <- sample(1:n,20)#samples for screening (condition 2)
dn <- diffnet_singlesplit(x1,x2,split1,split2)
dn$pval.onesided#p-value
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{perm.diffregr_teststat}
\alias{perm.diffregr_teststat}
\title{Auxiliary function for computation of "split-perm" p-value.}
\usage{
perm.diffregr_teststat(y1, y2, y12, x1, x2, x12)
}
\arguments{
\item{y1}{Response vector condition 1.}

\item{y2}{Response vector condition 2.}

\item{y12}{Pooled response vector.}

\item{x1}{Predictor matrix condition 1.}

\item{x2}{Predictor matrix condition 2.}

\item{x12}{Pooled predictor matrix}
}
\value{
Test statistic (log-likelihood-ratio statistic).
}
\description{
Auxiliary function for computation of "split-perm" p-value.
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{est2.ww.mat.diffregr}
\alias{est2.ww.mat.diffregr}
\title{Estimate weights}
\usage{
est2.ww.mat.diffregr(y1, y2, x1, x2, beta1, beta2, beta, act1, act2, act)
}
\arguments{
\item{y1}{no descr}

\item{y2}{no descr}

\item{x1}{no descr}

\item{x2}{no descr}

\item{beta1}{no descr}

\item{beta2}{no descr}

\item{beta}{no descr}

\item{act1}{no descr}

\item{act2}{no descr}

\item{act}{no descr}
}
\value{
no descr
}
\description{
Estimate weights
}
\details{
estimate W-matrix (using plug-in estimates of Beta-matrix); calculate eigenvalues(W-matrix)
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{q.matrix3}
\alias{q.matrix3}
\title{Compute Q-matrix}
\usage{
q.matrix3(sig, sig.a, sig.b, act.a, act.b, ss)
}
\arguments{
\item{sig}{no descr}

\item{sig.a}{no descr}

\item{sig.b}{no descr}

\item{act.a}{no descr}

\item{act.b}{no descr}

\item{ss}{no descr}
}
\value{
no descr
}
\description{
Compute Q-matrix
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{q.matrix.diffregr}
\alias{q.matrix.diffregr}
\title{Computation Q matrix}
\usage{
q.matrix.diffregr(Sig, a, b, s)
}
\arguments{
\item{Sig}{no descr}

\item{a}{no descr}

\item{b}{no descr}

\item{s}{no descr}
}
\value{
no descr
}
\description{
Computation Q matrix
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{diffregr_pval}
\alias{diffregr_pval}
\title{Computation "split-asym" p-values.}
\usage{
diffregr_pval(y1, y2, x1, x2, beta1, beta2, beta, act1, act2, act,
  compute.evals, method.compquadform, acc, epsabs, epsrel, show.warn,
  n.perm)
}
\arguments{
\item{y1}{Response vector condition 1.}

\item{y2}{Response vector condition 2.}

\item{x1}{Predictor matrix condition 1.}

\item{x2}{Predictor matrix condition 2.}

\item{beta1}{Regression coefficients condition 1.}

\item{beta2}{Regression coefficients condition 2.}

\item{beta}{Pooled regression coefficients.}

\item{act1}{Active-set condition 1.}

\item{act2}{Active-set condition 2.}

\item{act}{Pooled active-set.}

\item{compute.evals}{Method for computation of weights.}

\item{method.compquadform}{Method to compute distribution function of w-sum-of-chi2.}

\item{acc}{See ?davies.}

\item{epsabs}{See ?imhof.}

\item{epsrel}{See ?imhof.}

\item{show.warn}{Show warnings?}

\item{n.perm}{Number of permutations.}
}
\value{
P-value, test statistic, estimated weights.
}
\description{
Computation "split-asym"/"split-perm" p-values.
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixglasso.R
\name{func.uinit}
\alias{func.uinit}
\title{Initialization of MixGLasso}
\usage{
func.uinit(x, n.comp, init = "kmeans", my.cl = NULL,
  nstart.kmeans = 1, iter.max.kmeans = 10, modelname.hc = "EII")
}
\arguments{
\item{x}{Observed data}

\item{n.comp}{Number of mixture components}

\item{init}{Method used for initialization
init={'cl.init','r.means','random','kmeans','kmeans.hc','hc'}}

\item{my.cl}{Initial cluster assignments;
need to be provided if init='cl.init' (otherwise this param is ignored)}

\item{nstart.kmeans}{Number of random starts in kmeans; default=1}

\item{iter.max.kmeans}{Maximal number of iteration in kmeans; default=10}

\item{modelname.hc}{Model class used in hc; default='EII'}
}
\value{
a list consisting of
\item{u}{responsibilities}
}
\description{
Initialization of responsibilities
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netplot.R
\name{plot.ggmgsa}
\alias{plot.ggmgsa}
\title{Plotting function for object of class 'ggmgmsa'}
\usage{
\method{plot}{ggmgsa}(x, ...)
}
\arguments{
\item{x}{object of class 'ggmgsa'}

\item{...}{Further arguments.}
}
\value{
Boxplot of single-split p-values.
}
\description{
Plotting function for object of class 'ggmgsa'
}
\author{
nicolas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixglasso.R
\name{mixglasso_init}
\alias{mixglasso_init}
\title{mixglasso_init}
\usage{
mixglasso_init(x, n.comp, lambda, u.init, mix.prob.init, gamma = 0.5,
  pen = "glasso.parcor", penalize.diagonal = FALSE, term = 10^{    
  -3 }, miniter = 5, maxiter = 1000, min.compsize = 5,
  show.trace = FALSE)
}
\arguments{
\item{x}{Input data matrix}

\item{n.comp}{Number of mixture components}

\item{lambda}{Regularization parameter}

\item{u.init}{Initial responsibilities}

\item{mix.prob.init}{Initial component probablities}

\item{gamma}{Determines form of penalty}

\item{pen}{Determines form of penalty: glasso.parcor (default), glasso.invcov, glasso.invcor}

\item{penalize.diagonal}{Should the diagonal of the inverse covariance matrix be penalized ? Default=FALSE (recommended)}

\item{term}{Termination criterion of EM algorithm. Default=10^-3}

\item{miniter}{Minimal number of EM iteration before 'stop EM if any(compsize)<min.compsize' applies. Default=5}

\item{maxiter}{Maximal number of EM iteration. Default=1000}

\item{min.compsize}{Stop EM if any(compsize)<min.compsize; Default=5}

\item{show.trace}{Should information during execution be printed ? Default=FALSE}
}
\value{
list consisting of
\item{mix.prob}{Component probabilities}
\item{Mu}{Component specific mean vectors}
\item{Sig}{Component specific covariance matrices}
\item{SigInv}{Component specific inverse covariance matrices}
\item{iter}{Number of EM iterations}
\item{loglik}{Log-likelihood}
\item{bic}{-loglik+log(n)*DF/2}
\item{mmdl}{-loglik+penmmdl/2}
\item{u}{Component responsibilities}
\item{comp}{Component assignments}
\item{compsize}{Size of components}
\item{pi.comps}{Component probabilities}
\item{warn}{Warnings during EM algorithm}
}
\description{
mixglasso_init (initialization and lambda set by user)
}
\details{
This function runs mixglasso; requires initialization (u.init,mix.prob.init)
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{logratio.diffregr}
\alias{logratio.diffregr}
\title{Log-likelihood ratio statistics for Differential Regression.}
\usage{
logratio.diffregr(y1, y2, y, xx1, xx2, xx, beta1, beta2, beta)
}
\arguments{
\item{y1}{Response vector condition 1.}

\item{y2}{Response vector condition 2.}

\item{y}{Pooled response vector.}

\item{xx1}{Predictor matrix condition 1.}

\item{xx2}{Predictor matrix condition 2.}

\item{xx}{Pooled predictor matrix}

\item{beta1}{Regression coefficients condition 1.}

\item{beta2}{Regression coefficients condition 2.}

\item{beta}{Pooled regression coefficients.}
}
\value{
2 times log-likelihood ratio statistics.
}
\description{
Log-likelihood ratio statistics for Differential Regression.
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{my.ttest}
\alias{my.ttest}
\title{T-test}
\usage{
my.ttest(x1, x2)
}
\arguments{
\item{x1}{no descr}

\item{x2}{no descr}
}
\value{
no descr
}
\description{
T-test (equal variances)
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixglasso.R
\name{bwprun_mixglasso}
\alias{bwprun_mixglasso}
\title{bwprun_mixglasso}
\usage{
bwprun_mixglasso(x, n.comp.min = 1, n.comp.max, lambda = sqrt(2 *
  nrow(x) * log(ncol(x)))/2, pen = "glasso.parcor",
  selection.crit = "mmdl", term = 10^{     -3 }, min.compsize = 5,
  init = "kmeans.hc", my.cl = NULL, modelname.hc = "VVV",
  nstart.kmeans = 1, iter.max.kmeans = 10, reinit.out = FALSE,
  reinit.in = FALSE, mer = TRUE, del = TRUE, ...)
}
\arguments{
\item{x}{Input data matrix}

\item{n.comp.min}{Minimum number of components. Take n.comp.min=1 !}

\item{n.comp.max}{Maximum number of components}

\item{lambda}{Regularization parameter. Default=sqrt(2*n*log(p))/2}

\item{pen}{Determines form of penalty: glasso.parcor (default), glasso.invcov, glasso.invcor}

\item{selection.crit}{Selection criterion. Default='mmdl'}

\item{term}{Termination criterion of EM algorithm. Default=10^-3}

\item{min.compsize}{Stop EM if any(compsize)<min.compsize; Default=5}

\item{init}{Initialization. Method used for initialization
init={'cl.init','r.means','random','kmeans','kmeans.hc','hc'}. Default='kmeans.hc'}

\item{my.cl}{Initial cluster assignments; need to be provided if init='cl.init' (otherwise this param is ignored). Default=NULL}

\item{modelname.hc}{Model class used in hc. Default="VVV"}

\item{nstart.kmeans}{Number of random starts in kmeans; default=1}

\item{iter.max.kmeans}{Maximal number of iteration in kmeans; default=10}

\item{reinit.out}{Re-initialization if compsize<min.compsize (at the start of algorithm) ?}

\item{reinit.in}{Re-initialization if compsize<min.compsize (at the bwprun-loop level of algorithm) ?}

\item{mer}{Merge closest comps for initialization}

\item{del}{Delete smallest comp for initialization}

\item{...}{Other arguments. See mixglasso_init}
}
\value{
list consisting of
\item{selcrit}{Selcrit for all models with number of components between n.comp.min and n.comp.max}
\item{res.init}{Initialization for all components}
\item{comp.name}{List of names of components. Indicates which states where merged/deleted during backward pruning}
\item{re.init.in}{Logical vector indicating whether re-initialization was performed or not}
\item{fit.mixgl.selcrit}{Results for model with optimal number of components. List see mixglasso_init}
}
\description{
Mixglasso with backward pruning
}
\details{
This function runs mixglasso with various number of mixture components:
It starts with a too large number of components and iterates towards solutions
with smaller number of components by initializing using previous solutions.
}
\examples{
##generate data
set.seed(1)
n <- 1000
n.comp <- 3
p <- 10

# Create different mean vectors
Mu <- matrix(0,p,n.comp)

nonzero.mean <- split(sample(1:p),rep(1:n.comp,length=p))
for(k in 1:n.comp){
  Mu[nonzero.mean[[k]],k] <- -2/sqrt(ceiling(p/n.comp))
}

sim <- sim_mix_networks(n, p, n.comp, Mu=Mu)

##run mixglasso

fit <-  bwprun_mixglasso(sim$data,n.comp=1,n.comp.max=5,selection.crit='bic')
plot(fit$selcrit,ylab='bic',xlab='Num.Comps',type='b')
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{t2diagcov.lr}
\alias{t2diagcov.lr}
\title{Diagonal-restricted likelihood-ratio test}
\usage{
t2diagcov.lr(x1, x2, include.mean = FALSE)
}
\arguments{
\item{x1}{no descr}

\item{x2}{no descr}

\item{include.mean}{no descr}
}
\value{
no descr
}
\description{
Diagonal-restricted likelihood-ratio test
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netsim.R
\name{sim_mix}
\alias{sim_mix}
\title{Simulate from mixture model.}
\usage{
sim_mix(n, n.comp, mix.prob, Mu, Sig, dist = "norm", df = 2)
}
\arguments{
\item{n}{sample size}

\item{n.comp}{number of mixture components ("comps")}

\item{mix.prob}{mixing probablities (need to sum to 1)}

\item{Mu}{matrix of component-specific mean vectors}

\item{Sig}{array of component-specific covariance matrices}

\item{dist}{'norm' for Gaussian components, 't' for t-distributed components}

\item{df}{degrees of freedom of the t-distribution (not used for Gaussian 
distribution), default=2}
}
\value{
a list consisting of:
\item{S}{component assignments}
\item{X}{observed data matrix}
}
\description{
Simulate from mixture model with multi-variate Gaussian or t-distributed 
components.
}
\examples{
n.comp = 4
p = 5 # dimensionality
Mu = matrix(rep(0, p), p, n.comp)
Sigma = array(diag(p), c(p, p, n.comp))
mix.prob = rep(0.25, n.comp)

sim_mix(100, n.comp, mix.prob, Mu, Sigma)
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{hugepath}
\alias{hugepath}
\title{Graphical Lasso path with huge package}
\usage{
hugepath(s, rholist, penalize.diagonal = NULL, trace = NULL)
}
\arguments{
\item{s}{no descr}

\item{rholist}{no descr}

\item{penalize.diagonal}{no descr}

\item{trace}{no descr}
}
\value{
no descr
}
\description{
Graphical Lasso path with huge package
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{bic.glasso}
\alias{bic.glasso}
\title{BIC.glasso}
\usage{
bic.glasso(x, lambda, penalize.diagonal = FALSE, plot.it = TRUE,
  use.package = "huge", include.mean = FALSE)
}
\arguments{
\item{x}{no descr}

\item{lambda}{no descr}

\item{penalize.diagonal}{no descr}

\item{plot.it}{no descr}

\item{use.package}{no descr}

\item{include.mean}{no descr}
}
\value{
no descr
}
\description{
BIC.glasso
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netsim.R
\name{getinvcov}
\alias{getinvcov}
\title{Generate an inverse covariance matrix with a given sparsity and 
dimensionality}
\usage{
getinvcov(p, s, a.diff = 5, b.diff = 5, magn.diag = 0, emin = 0.1)
}
\arguments{
\item{p}{Dimensionality}

\item{s}{Sparsity}

\item{a.diff}{binomial parameter}

\item{b.diff}{binomial parameter}

\item{magn.diag}{Magnitude}

\item{emin}{e min}
}
\value{
Inverse covariance matrix
Internal function
}
\description{
Generate an inverse covariance matrix with a given sparsity and 
dimensionality
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{shapiro_screen}
\alias{shapiro_screen}
\title{Filter "non-normal" genes}
\usage{
shapiro_screen(x1, x2, sign.level = 0.001)
}
\arguments{
\item{x1}{expression matrix (condition 1)}

\item{x2}{expression matrix (condition 2)}

\item{sign.level}{sign.level in Shapiro-Wilk tests (default: sign.level=0.001)}
}
\value{
list consisting of
\item{x1.filt}{expression matrix (condition 1) after filtering}
\item{x2.filt}{expression matrix (condition 2) after filtering}
}
\description{
Filter "non-normal" genes
}
\details{
Discarding genes which have Shapiro-Wilk p-value (corrected for multiplicity)
smaller than sign.level in either of the two conditions. We used sign.level=0.001
in the GGMGSA paper.
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixglasso.R
\name{symmkldist}
\alias{symmkldist}
\title{Compute symmetric kull-back leibler distance}
\usage{
symmkldist(mu1, mu2, sig1, sig2)
}
\arguments{
\item{mu1}{no descr}

\item{mu2}{no descr}

\item{sig1}{no descr}

\item{sig2}{no descr}
}
\value{
symmetric kull-back leibler distance
}
\description{
Compute symmetric kull-back leibler distance
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netplot.R
\name{plot.nethetclustering}
\alias{plot.nethetclustering}
\title{Plot networks}
\usage{
\method{plot}{nethetclustering}(x,
  node.names = rownames(net.clustering$Mu),
  group.names = sort(unique(net.clustering$comp)),
  p.corrs.thresh = 0.2, print.pdf = FALSE, pdf.filename = "networks",
  ...)
}
\arguments{
\item{x}{A network clustering object as returned by 
\code{\link{screen_cv.glasso}} or \code{\link{mixglasso}}.}

\item{node.names}{Names for the nodes in the network. If NULL, names from 
net.clustering will be used.}

\item{group.names}{Names for the clusters or groups. If NULL, names from 
net.clustering will be used (by default these are integets 1:numClusters).}

\item{p.corrs.thresh}{Threshold applied to the absolute partial correlations. 
Edges that are below the threshold in all of the groups are not displayed.}

\item{print.pdf}{If TRUE, save the output as a PDF file.}

\item{pdf.filename}{If \code{print.pdf} is TRUE, specifies the file name of
the output PDF file.}

\item{...}{Further arguments}
}
\value{
Returns NULL and prints out the networks (or saves them to pdf if
\code{print.pdf} is TRUE. The networks are displayed as a series of nComps+1
plots, where in the first plot edge widths are shown according to 
the maximum partial correlation of the edge over all groups. The following plots
show the edges for each group. Positive partial correlation edges are shown in
black, negative ones in blue. If an edge is below the threshold on the absolute
partial correlation, it is displayed in gray or light blue respectively.
}
\description{
This function takes the output of \code{\link{screen_cv.glasso}} or 
\code{\link{mixglasso}} and creates a network plot using the network library.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{screen_cvmin.lasso}
\alias{screen_cvmin.lasso}
\title{Cross-validation lasso screening (lambda.min-rule)}
\usage{
screen_cvmin.lasso(x, y)
}
\arguments{
\item{x}{Predictor matrix}

\item{y}{Response vector}
}
\value{
Active-set
}
\description{
Cross-validated Lasso screening (lambda.min-rule)
}
\examples{
screen_cvmin.lasso(matrix(rnorm(5000),50,100),rnorm(50))
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{diffnet_multisplit}
\alias{diffnet_multisplit}
\title{Differential Network}
\usage{
diffnet_multisplit(x1, x2, b.splits = 50, frac.split = 1/2,
  screen.meth = "screen_bic.glasso", include.mean = FALSE,
  gamma.min = 0.05, compute.evals = "est2.my.ev3",
  algorithm.mleggm = "glasso_rho0", method.compquadform = "imhof",
  acc = 1e-04, epsabs = 1e-10, epsrel = 1e-10, show.warn = FALSE,
  save.mle = FALSE, verbose = TRUE, mc.flag = FALSE,
  mc.set.seed = TRUE, mc.preschedule = TRUE,
  mc.cores = getOption("mc.cores", 2L), ...)
}
\arguments{
\item{x1}{Data-matrix sample 1.
You might need to center and scale your data-matrix.}

\item{x2}{Data-matrix sample 1.
You might need to center and scale your data-matrix.}

\item{b.splits}{Number of splits (default=50).}

\item{frac.split}{Fraction train-data (screening) / test-data (cleaning) (default=0.5).}

\item{screen.meth}{Screening procedure. Options: 'screen_bic.glasso' (default),
'screen_cv.glasso', 'screen_shrink' (not recommended).}

\item{include.mean}{Should sample specific means be included in hypothesis?
Use include.mean=FALSE (default and recommended) which assumes mu1=mu2=0
and tests the hypothesis H0: Omega_1=Omega_2.}

\item{gamma.min}{Tuning parameter in p-value aggregation of Meinshausen et al (2009). (Default=0.05).}

\item{compute.evals}{Method to estimate the weights in the weighted-sum-of-chi2s distribution.
The default and (currently) the only available option 
is the method 'est2.my.ev3'.}

\item{algorithm.mleggm}{Algorithm to compute MLE of GGM. The algorithm 'glasso_rho' is the
default and (currently) the only available option.}

\item{method.compquadform}{Method to compute distribution function of weighted-sum-of-chi2s
(default='imhof').}

\item{acc}{See ?davies (default 1e-04).}

\item{epsabs}{See ?imhof (default 1e-10).}

\item{epsrel}{See ?imhof (default 1e-10).}

\item{show.warn}{Should warnings be showed (default=FALSE)?}

\item{save.mle}{If TRUE, MLEs (inverse covariance matrices for samples 1 and 2)
are saved for all b.splits. The median aggregated inverse covariance matrix
is provided in the output as 'medwi'. The default is save.mle=FALSE.}

\item{verbose}{If TRUE, show output progress.}

\item{mc.flag}{If \code{TRUE} use parallel execution for each b.splits via function 
\code{mclapply} of package \code{parallel}.}

\item{mc.set.seed}{See mclapply. Default=TRUE}

\item{mc.preschedule}{See mclapply. Default=TRUE}

\item{mc.cores}{Number of cores to use in parallel execution. Defaults to
mc.cores option if set, or 2 otherwise.}

\item{...}{Additional arguments for screen.meth.}
}
\value{
list consisting of
\item{ms.pval}{p-values for all b.splits}
\item{ss.pval}{single-split p-value}
\item{medagg.pval}{median aggregated p-value}
\item{meinshagg.pval}{meinshausen aggregated p-value (meinshausen et al 2009)}
\item{teststat}{test statistics for b.splits}
\item{weights.nulldistr}{estimated weights}
\item{active.last}{active-sets obtained in last screening-step}
\item{medwi}{median of inverse covariance matrices over b.splits}
\item{sig.last}{constrained mle (covariance matrix) obtained in last cleaning-step}
\item{wi.last}{constrained mle (inverse covariance matrix) obtained in last cleaning-step}
}
\description{
Differential Network
}
\details{
Remark:

* If include.mean=FALSE, then x1 and x2 have mean zero and DiffNet tests
  the hypothesis H0: Omega_1=Omega_2. You might need to center x1 and x2.
* If include.mean=TRUE, then DiffNet tests the hypothesis
  H0: mu_1=mu_2 & Omega_1=Omega_2
* However, we recommend to set include.mean=FALSE and to test equality of the means
  separately. 
* You might also want to scale x1 and x2, if you are only interested in 
  differences due to (partial) correlations.
}
\examples{

############################################################
##This example illustrates the use of Differential Network##
############################################################


##set seed
set.seed(1)

##sample size and number of nodes
n <- 40
p <- 10

##specifiy sparse inverse covariance matrices
gen.net <- generate_2networks(p,graph='random',n.nz=rep(p,2),
                              n.nz.common=ceiling(p*0.8))
invcov1 <- gen.net[[1]]
invcov2 <- gen.net[[2]]
plot_2networks(invcov1,invcov2,label.pos=0,label.cex=0.7)

##get corresponding correlation matrices
cor1 <- cov2cor(solve(invcov1))
cor2 <- cov2cor(solve(invcov2))

##generate data under null hypothesis (both datasets have the same underlying
## network)
library('mvtnorm')
x1 <- rmvnorm(n,mean = rep(0,p), sigma = cor1)
x2 <- rmvnorm(n,mean = rep(0,p), sigma = cor1)

##run diffnet (under null hypothesis)
dn.null <- diffnet_multisplit(x1,x2,b.splits=1,verbose=FALSE)
dn.null$ss.pval#single-split p-value

##generate data under alternative hypothesis (datasets have different networks)
x1 <- rmvnorm(n,mean = rep(0,p), sigma = cor1)
x2 <- rmvnorm(n,mean = rep(0,p), sigma = cor2)

##run diffnet (under alternative hypothesis)
dn.altn <- diffnet_multisplit(x1,x2,b.splits=1,verbose=FALSE)
dn.altn$ss.pval#single-split p-value
dn.altn$medagg.pval#median aggregated p-value

##typically we would choose a larger number of splits
# dn.altn <- diffnet_multisplit(x1,x2,b.splits=10,verbose=FALSE)
# dn.altn$ms.pval#multi-split p-values
# dn.altn$medagg.pval#median aggregated p-value
# plot(dn.altn)#histogram of single-split p-values
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{gsea.t2cov}
\alias{gsea.t2cov}
\title{GSA using T2cov-test}
\usage{
gsea.t2cov(x1, x2, gene.sets, gene.names, gs.names = NULL,
  method = "t2cov.lr", method.p.adjust = "fdr")
}
\arguments{
\item{x1}{expression matrix (condition 1)}

\item{x2}{expression matrix (condition 2)}

\item{gene.sets}{list of gene-sets}

\item{gene.names}{gene names}

\item{gs.names}{gene-set names}

\item{method}{method for testing equality of covariance matrices}

\item{method.p.adjust}{method for p-value adjustment (default: 'fdr')}
}
\value{
list of results
}
\description{
GSA using T2cov-test
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixglasso.R
\name{tr}
\alias{tr}
\title{Compute trace of matrix}
\usage{
tr(m)
}
\arguments{
\item{m}{no descr}
}
\value{
trace of matrix
}
\description{
Compute trace of matrix
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{gsea.highdimT2}
\alias{gsea.highdimT2}
\title{GSA based on HighdimT2}
\usage{
gsea.highdimT2(x1, x2, gene.sets, gene.names, gs.names = NULL,
  method = "test.sd", method.p.adjust = "fdr")
}
\arguments{
\item{x1}{no descr}

\item{x2}{no descr}

\item{gene.sets}{no descr}

\item{gene.names}{no descr}

\item{gs.names}{no descr}

\item{method}{no descr}

\item{method.p.adjust}{no descr}
}
\value{
no descr
}
\description{
GSA based on HighdimT2
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{cv.fold}
\alias{cv.fold}
\title{Make folds}
\usage{
cv.fold(n, folds = 10)
}
\arguments{
\item{n}{no descr}

\item{folds}{no descr}
}
\value{
no descr
}
\description{
Make folds
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{test.sd}
\alias{test.sd}
\title{High-Dim Two-Sample Test (Srivastava, 2006)}
\usage{
test.sd(x1, x2)
}
\arguments{
\item{x1}{no descr}

\item{x2}{no descr}
}
\value{
no descr
}
\description{
High-Dim Two-Sample Test (Srivastava, 2006)
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{diffnet_pval}
\alias{diffnet_pval}
\title{P-value calculation}
\usage{
diffnet_pval(x1, x2, x, sig1, sig2, sig, mu1, mu2, mu, act1, act2, act,
  compute.evals, include.mean, method.compquadform, acc, epsabs, epsrel,
  show.warn)
}
\arguments{
\item{x1}{no descr}

\item{x2}{no descr}

\item{x}{no descr}

\item{sig1}{no descr}

\item{sig2}{no descr}

\item{sig}{no descr}

\item{mu1}{no descr}

\item{mu2}{no descr}

\item{mu}{no descr}

\item{act1}{no descr}

\item{act2}{no descr}

\item{act}{no descr}

\item{compute.evals}{no descr}

\item{include.mean}{no descr}

\item{method.compquadform}{no descr}

\item{acc}{no descr}

\item{epsabs}{no descr}

\item{epsrel}{no descr}

\item{show.warn}{no descr}
}
\value{
no descr
}
\description{
P-value calculation
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixglasso.R
\name{mixglasso}
\alias{mixglasso}
\title{mixglasso}
\usage{
mixglasso(x, n.comp, lambda = sqrt(2 * nrow(x) * log(ncol(x)))/2,
  pen = "glasso.parcor", init = "kmeans.hc", my.cl = NULL,
  modelname.hc = "VVV", nstart.kmeans = 1, iter.max.kmeans = 10,
  term = 10^{     -3 }, min.compsize = 5, save.allfits = FALSE,
  filename = "mixglasso_fit.rda", mc.flag = FALSE,
  mc.set.seed = FALSE, mc.preschedule = FALSE,
  mc.cores = getOption("mc.cores", 2L), ...)
}
\arguments{
\item{x}{Input data matrix}

\item{n.comp}{Number of mixture components. If n.comp is a vector, 
\code{mixglasso} will estimate a model for each number of mixture components,
and return a list of models, as well as their BIC and MMDL scores and the index of the
best model according to each score.}

\item{lambda}{Regularization parameter. Default=sqrt(2*n*log(p))/2}

\item{pen}{Determines form of penalty: glasso.parcor (default) to penalise the
partial correlation matrix, glasso.invcov to penalise the inverse covariance 
matrix (this corresponds to classical graphical lasso), glasso.invcor to 
penalise the inverse correlation matrix.}

\item{init}{Initialization. Method used for initialization
init={'cl.init','r.means','random','kmeans','kmeans.hc','hc'}. Default='kmeans'}

\item{my.cl}{Initial cluster assignments;
need to be provided if init='cl.init' (otherwise this param is ignored). Default=NULL}

\item{modelname.hc}{Model class used in hc. Default="VVV"}

\item{nstart.kmeans}{Number of random starts in kmeans; default=1}

\item{iter.max.kmeans}{Maximal number of iteration in kmeans; default=10}

\item{term}{Termination criterion of EM algorithm. Default=10^-3}

\item{min.compsize}{Stop EM if any(compsize)<min.compsize; Default=5}

\item{save.allfits}{If TRUE, save output of mixglasso for all k's.}

\item{filename}{If \code{save.allfits} is TRUE, output of mixglasso will be
saved as \code{paste(filename, _fit.mixgl_k.rda, sep='')}.}

\item{mc.flag}{If \code{TRUE} use parallel execution for each n.comp via function 
\code{mclapply} of package \code{parallel}.}

\item{mc.set.seed}{See mclapply. Default=FALSE}

\item{mc.preschedule}{See mclapply. Default=FALSE}

\item{mc.cores}{Number of cores to use in parallel execution. Defaults to
mc.cores option if set, or 2 otherwise.}

\item{...}{Other arguments. See mixglasso_init}
}
\value{
A list with elements:
\item{models}{List with each element i containing an S3 object of class 
'nethetclustering' that contains the result of fitting the mixture 
graphical lasso model with n.comps[i] components. See the documentation of
mixglasso_ncomp_fixed for the description of this object.}
\item{bic}{BIC for all fits.}
\item{mmdl}{Minimum description length score for all fits.}
\item{comp}{Component assignments for all fits.}
\item{bix.opt}{Index of model with optimal BIC score.}
\item{mmdl.opt}{Index of model with optimal MMDL score.}
}
\description{
mixglasso
}
\details{
Runs mixture of graphical lasso network clustering with one or several
numbers of mixture components.
}
\examples{


###########################################
##This an example of how to use MixGLasso##
###########################################

##generate data
set.seed(1)
n <- 1000
n.comp <- 3
p <- 10

# Create different mean vectors
Mu <- matrix(0,p,n.comp)

nonzero.mean <- split(sample(1:p),rep(1:n.comp,length=p))
for(k in 1:n.comp){
  Mu[nonzero.mean[[k]],k] <- -2/sqrt(ceiling(p/n.comp))
}

sim <- sim_mix_networks(n, p, n.comp, Mu=Mu)

##run mixglasso
set.seed(1)
fit1 <-  mixglasso(sim$data,n.comp=1:6)
fit1$bic
set.seed(1)
fit2 <-  mixglasso(sim$data,n.comp=6)
fit2$bic
set.seed(1)
fit3 <-  mixglasso(sim$data,n.comp=1:6,lambda=0)
set.seed(1)
fit4 <-  mixglasso(sim$data,n.comp=1:6,lambda=Inf)
#set.seed(1)
#fit5 <-  bwprun_mixglasso(sim$data,n.comp=1,n.comp.max=5,selection.crit='bic')
#plot(fit5$selcrit,ylab='bic',xlab='Num.Comps',type='b')

##compare bic
library('ggplot2')
plotting.frame <- 
  data.frame(BIC= c(fit1$bic, fit3$bic, fit4$bic), 
             Num.Comps=rep(1:6, 3), 
             Lambda=rep(c('Default', 
                          'Lambda = 0',
                          'Lambda = Inf'),
                        each=6))

p <- ggplot(plotting.frame) + 
	geom_line(aes(x=Num.Comps, y=BIC, colour=Lambda))

print(p)
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{screen_cvtrunc.lasso}
\alias{screen_cvtrunc.lasso}
\title{Cross-validated Lasso screening and additional truncation.}
\usage{
screen_cvtrunc.lasso(x, y, k.trunc = 5)
}
\arguments{
\item{x}{Predictor matrix.}

\item{y}{Response vector.}

\item{k.trunc}{Truncation constant="number of samples per predictor" (default=5).}
}
\value{
Active-set.
}
\description{
Cross-validated Lasso screening and additional truncation.
}
\details{
Computes Lasso coefficients (cross-validation optimal lambda). Truncates
smallest coefficients to zero, such that there are no more than n/k.trunc
non-zero coefficients.
}
\examples{
screen_cvtrunc.lasso(matrix(rnorm(5000),50,100),rnorm(50))
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{screen_full}
\alias{screen_full}
\title{Screen_full}
\usage{
screen_full(x, include.mean = NULL, length.lambda = NULL,
  trunc.method = NULL, trunc.k = NULL)
}
\arguments{
\item{x}{no descr}

\item{include.mean}{no descr}

\item{length.lambda}{no descr}

\item{trunc.method}{no descr}

\item{trunc.k}{no descr}
}
\value{
no descr
}
\description{
Screen_full
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auxfunc.R
\name{invcov2parcor_array}
\alias{invcov2parcor_array}
\title{Convert inverse covariance to partial correlation for several inverse
covariance matrices collected in an array.}
\usage{
invcov2parcor_array(invcov.array)
}
\arguments{
\item{invcov.array}{Array of inverse covariance matrices, of dimension
numNodes by numNodes by numComps.}
}
\value{
Array of partial correlation matrices of dimension numNodes by 
numNodes by numComps
}
\description{
Convert inverse covariance to partial correlation for several inverse
covariance matrices collected in an array.
}
\examples{
invcov.array = sapply(1:5, function(x) generate_inv_cov(p=25), simplify='array')
p.corr = invcov2parcor_array(invcov.array)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixglasso.R
\name{mixglasso_ncomp_fixed}
\alias{mixglasso_ncomp_fixed}
\title{mixglasso_ncomp_fixed}
\usage{
mixglasso_ncomp_fixed(x, n.comp, lambda = sqrt(2 * nrow(x) *
  log(ncol(x)))/2, pen = "glasso.parcor", init = "kmeans.hc",
  my.cl = NULL, modelname.hc = "VVV", nstart.kmeans = 1,
  iter.max.kmeans = 10, term = 10^{     -3 }, min.compsize = 5, ...)
}
\arguments{
\item{x}{Input data matrix}

\item{n.comp}{Number of mixture components}

\item{lambda}{Regularization parameter. Default=sqrt(2*n*log(p))/2}

\item{pen}{Determines form of penalty: glasso.parcor (default), glasso.invcov, glasso.invcor}

\item{init}{Initialization. Method used for initialization
init={'cl.init','r.means','random','kmeans','kmeans.hc','hc'}. Default='kmeans'}

\item{my.cl}{Initial cluster assignments;
need to be provided if init='cl.init' (otherwise this param is ignored). Default=NULL}

\item{modelname.hc}{Model class used in hc. Default="VVV"}

\item{nstart.kmeans}{Number of random starts in kmeans; default=1}

\item{iter.max.kmeans}{Maximal number of iteration in kmeans; default=10}

\item{term}{Termination criterion of EM algorithm. Default=10^-3}

\item{min.compsize}{Stop EM if any(compsize)<min.compsize; Default=5}

\item{...}{Other arguments. See mixglasso_init}
}
\value{
see return mixglasso_init. list consisting of
\item{mix.prob}{}
\item{Mu}{}
\item{Sig}{}
\item{SigInv}{}
\item{iter}{}
\item{loglik}{}
\item{bic}{-loglik+log(n)*DF/2}
\item{mmdl}{-loglik+penmmdl/2}
\item{u}{responsibilities}
\item{comp}{component assignments}
\item{compsize}{size of components}
\item{pi.comps}{}
\item{warn}{warnings during optimization}
}
\description{
mixglasso_ncomp_fixed
}
\details{
This function runs mixglasso
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{perm.diffregr_pval}
\alias{perm.diffregr_pval}
\title{Computation "split-perm" p-value.}
\usage{
perm.diffregr_pval(y1, y2, x1, x2, act1, act2, act, n.perm)
}
\arguments{
\item{y1}{Response vector condition 1.}

\item{y2}{Response vector condition 2.}

\item{x1}{Predictor matrix condition 1.}

\item{x2}{Predictor matrix condition 2.}

\item{act1}{Active-set condition 1.}

\item{act2}{Active-set condition 2.}

\item{act}{Pooled active-set.}

\item{n.perm}{Number of permutations.}
}
\value{
Permutation based p-value.
}
\description{
Computation "split-perm" p-value.
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixglasso.R
\name{sumoffdiag}
\alias{sumoffdiag}
\title{Sum of non-diag elements of a matrix}
\usage{
sumoffdiag(m)
}
\arguments{
\item{m}{no descr}
}
\value{
Sum of non-diag elements
}
\description{
Sum of non-diag elements of a matrix
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{screen_aic.glasso}
\alias{screen_aic.glasso}
\title{AIC-tuned glasso with additional thresholding}
\usage{
screen_aic.glasso(x, include.mean = TRUE, length.lambda = 20,
  lambdamin.ratio = ifelse(ncol(x) > nrow(x), 0.01, 0.001),
  penalize.diagonal = FALSE, plot.it = FALSE,
  trunc.method = "linear.growth", trunc.k = 5, use.package = "huge",
  verbose = FALSE)
}
\arguments{
\item{x}{The input data. Needs to be a num.samples by dim.samples matrix.}

\item{include.mean}{Include mean in likelihood. TRUE / FALSE (default).}

\item{length.lambda}{Length of lambda path to consider (default=20).}

\item{lambdamin.ratio}{Ratio lambda.min/lambda.max.}

\item{penalize.diagonal}{If TRUE apply penalization to diagonal of inverse
covariance as well. (default=FALSE)}

\item{plot.it}{TRUE / FALSE (default)}

\item{trunc.method}{None / linear.growth (default) / sqrt.growth}

\item{trunc.k}{truncation constant, number of samples per predictor (default=5)}

\item{use.package}{'glasso' or 'huge' (default).}

\item{verbose}{If TRUE, output la.min, la.max and la.opt (default=FALSE).}
}
\value{
Returns a list with named elements 'rho.opt', 'wi', 'wi.orig'.
         Variable rho.opt is the optimal (scaled) penalization parameter (rho.opt=2*la.opt/n). 
         The variables wi and wi.orig are matrices of size dim.samples by dim.samples 
         containing the truncated and untruncated inverse covariance matrix.
}
\description{
AIC-tuned glasso with additional thresholding
}
\examples{
n=50
p=5
x=matrix(rnorm(n*p),n,p)
wihat=screen_aic.glasso(x,length.lambda=5)$wi
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{est2.ww.mat2.diffregr}
\alias{est2.ww.mat2.diffregr}
\title{Estimate weights}
\usage{
est2.ww.mat2.diffregr(y1, y2, x1, x2, beta1, beta2, beta, act1, act2, act)
}
\arguments{
\item{y1}{no descr}

\item{y2}{no descr}

\item{x1}{no descr}

\item{x2}{no descr}

\item{beta1}{no descr}

\item{beta2}{no descr}

\item{beta}{no descr}

\item{act1}{no descr}

\item{act2}{no descr}

\item{act}{no descr}
}
\value{
no descr
}
\description{
Estimate weights
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netplot.R
\name{plot_2networks}
\alias{plot_2networks}
\title{Plot two networks (GGMs)}
\usage{
plot_2networks(invcov1, invcov2, node.label = paste("X", 1:nrow(invcov1),
  sep = ""), main = c("", ""), ...)
}
\arguments{
\item{invcov1}{Inverse covariance matrix of GGM1.}

\item{invcov2}{Inverse covariance matrix of GGM2.}

\item{node.label}{Names of nodes.}

\item{main}{Vector (two elements) with network names.}

\item{...}{Other arguments (see plot.network).}
}
\value{
Figure with two panels (for each network).
}
\description{
Plot two networks (GGMs)
}
\examples{
n <- 70
p <- 30

## Specifiy sparse inverse covariance matrices,
## with number of edges in common equal to ~ 0.8*p
gen.net <- generate_2networks(p,graph='random',n.nz=rep(p,2),
                              n.nz.common=ceiling(p*0.8))

invcov1 <- gen.net[[1]]
invcov2 <- gen.net[[2]]

plot_2networks(invcov1,invcov2,label.pos=0,label.cex=0.7)
}
\author{
nicolas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{my.ttest2}
\alias{my.ttest2}
\title{T-test}
\usage{
my.ttest2(x1, x2)
}
\arguments{
\item{x1}{no descr}

\item{x2}{no descr}
}
\value{
no descr
}
\description{
T-test (unequal variances)
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{test.t2}
\alias{test.t2}
\title{HotellingsT2}
\usage{
test.t2(x1, x2)
}
\arguments{
\item{x1}{no descr}

\item{x2}{no descr}
}
\value{
no descr
}
\description{
HotellingsT2
}
\author{
n.stadler
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffregr.R
\name{screen_cvfix.lasso}
\alias{screen_cvfix.lasso}
\title{Cross-validated Lasso screening and upper bound on number of predictors.}
\usage{
screen_cvfix.lasso(x, y, no.predictors = 10)
}
\arguments{
\item{x}{Predictor matrix.}

\item{y}{Response vector.}

\item{no.predictors}{Upper bound on number of active predictors,}
}
\value{
Active-set.
}
\description{
Cross-validated Lasso screening and upper bound on number of predictors
}
\details{
Computes Lasso coefficients (cross-validation optimal lambda). Truncates
smalles coefficients to zero such that there are no more than no.predictors
non-zero coefficients
}
\examples{
screen_cvfix.lasso(matrix(rnorm(5000),50,100),rnorm(50))
}
\author{
n.stadler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffnet.R
\name{screen_shrink}
\alias{screen_shrink}
\title{Shrinkage approach for estimating Gaussian graphical model}
\usage{
screen_shrink(x, include.mean = NULL, trunc.method = "linear.growth",
  trunc.k = 5)
}
\arguments{
\item{x}{The input data. Needs to be a num.samples by dim.samples matrix.}

\item{include.mean}{Include mean in likelihood. TRUE / FALSE (default).}

\item{trunc.method}{None / linear.growth (default) / sqrt.growth}

\item{trunc.k}{truncation constant, number of samples per predictor (default=5)}
}
\value{
Returns a list with named elements 'rho.opt', 'wi', 'wi.orig'.
         Variable rho.opt=NULL (no tuning parameter involved). 
         The variables wi and wi.orig are matrices of size dim.samples by dim.samples 
         containing the truncated and untruncated inverse covariance matrix.
}
\description{
Shrinkage approach for estimating Gaussian graphical model
}
\author{
n.stadler
}
\keyword{Removed}
\keyword{examples}
\keyword{export}
\keyword{fixed.}
\keyword{internal}
\keyword{is}
\keyword{n=50}
\keyword{p=5}
\keyword{package}
\keyword{parcor}
\keyword{until}
\keyword{wihat=screen_shrink(x)$wi}
\keyword{x=matrix(rnorm(n*p),n,p)}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmgsa.R
\name{my.p.adjust}
\alias{my.p.adjust}
\title{P-value adjustment}
\usage{
my.p.adjust(p, method = "fdr")
}
\arguments{
\item{p}{Vector of p-values.}

\item{method}{Method for p-value adjustment (default='fdr').}
}
\value{
Vector of adjusted p-values.
}
\description{
P-value adjustment
}
\author{
n.stadler
}
\keyword{internal}
